"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateDatabaseModel = exports.EnableStrumenta = void 0;
const SQLParser = __importStar(require("@strumenta/sql-parser"));
const MetadataOperations = __importStar(require("./MetadataOperations"));
const StepOperations = __importStar(require("./StepOperations"));
const Logging = __importStar(require("./Logging"));
exports.EnableStrumenta = true;
const StepsLogger = Logging.GetLogger(Logging.LoggingArea.Steps);
///// FORMATTING DATA MODEL
/**
 * @param predecessors an array of step IDs for any current predecessors
 * @param allStepsData
 * @param locations
 *
 * @returns a database model to be used in with strumenta's editorFunctions.
 * currently assumes we're creating the model from predecessors only
 */
const CreateDatabaseModel = (predecessors, allStepsData, locations) => {
    const tables = CreateTables(predecessors, allStepsData, locations);
    // right now we're not configuring separate schemas/databases, so the data structure here is really just to adhere to their typing
    const schemas = { "": CreateSchema(tables) };
    const databases = { "": CreateDatabase(schemas) };
    // format any functions - do we even use these right now?
    const functions = CreateFunctions();
    const defaultDBModel = new SQLParser.DefaultDatabaseModel(databases, functions);
    return defaultDBModel;
};
exports.CreateDatabaseModel = CreateDatabaseModel;
/// HELPERS
const CreateDatabase = (schemas) => {
    return {
        schemas,
        getSchemas: () => schemas,
        getSchema: (schemaName) => schemas[schemaName],
        // assuming the first schema is the default for now
        getDefaultSchema: () => Object.values(schemas)[0]
    };
};
const CreateSchema = (tables) => {
    return {
        tables,
        getTables: () => tables,
        getTable: (tableName) => tables[tableName]
    };
};
const CreateTables = (predecessors, allStepsData, locations) => {
    let tableDictionary = {};
    predecessors.forEach((predecessorID) => {
        const columns = MetadataOperations.getColumnsForStep(allStepsData, predecessorID);
        if (!columns) {
            // TODO: better logging here
            StepsLogger.errorContext({}, "predecessor no longer exists");
        }
        else {
            const columnDict = {};
            columns.forEach(column => columnDict[column.name] = { type: new SQLParser.NamedType(column.dataType) });
            const tableLocation = StepOperations.getTableLocation(locations, allStepsData[predecessorID].operation);
            tableDictionary[tableLocation.tableName] = CreateTable(columnDict);
        }
    });
    return tableDictionary;
};
const CreateTable = (columns) => {
    return {
        columns,
        getColumns: () => columns,
        getColumn: (columnName) => columns[columnName]
    };
};
const CreateFunctions = () => {
    return {
        AVG: {
            arguments: [{}],
            type: SQLParser.functionType(new SQLParser.NamedType("float"), SQLParser.TYPE_ANY_NUMBER)
        }
    };
};
//# sourceMappingURL=Autocomplete.js.map