"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DisableNonCLIConsoleLogs = exports.ExitCLISafe = exports.VerifyAllRefreshItems = exports.VerifyAllPlanItems = exports.VerifyAllDeployItems = exports.ECLICommands = void 0;
const Templates = __importStar(require("./Templates"));
const R = __importStar(require("runtypes"));
const Logging = __importStar(require("./Logging"));
const Firebase = __importStar(require("./Firebase"));
const LogCLI = Logging.GetLogger(Logging.LoggingArea.CLI);
const UserCredentialsDeployType = R.Record({
    snowflakeUsername: R.String,
    snowflakePassword: R.String.optional(),
    snowflakeRole: R.String.optional(),
    snowflakeWarehouse: R.String.optional(),
});
const UserCredentialsRefreshType = R.Record({
    snowflakePassword: R.String.optional(),
});
const BaseEnvironmentType = R.Record({
    environmentID: R.Union(R.String, R.Number),
});
const EnvironmentRefreshType = BaseEnvironmentType.extend({
    jobID: R.String.optional(),
});
const DeployConfig = R.Record({
    userCredentials: UserCredentialsDeployType,
});
const RefreshConfig = R.Record({
    userCredentials: UserCredentialsRefreshType,
    runDetails: EnvironmentRefreshType,
});
const Plan = R.Record({
    version: R.Number,
    targetEnvironment: R.Number,
    plan: R.Unknown,
    environmentState: R.Unknown,
    gitInfo: R.Unknown,
});
const PlanConfig = R.Record({
    runDetails: BaseEnvironmentType,
});
const Token = R.String;
var ECLICommands;
(function (ECLICommands) {
    ECLICommands["Refresh"] = "refresh";
    ECLICommands["Deploy"] = "deploy";
    ECLICommands["Plan"] = "plan";
    ECLICommands["Help"] = "help";
    ECLICommands["Admin"] = "admin";
})(ECLICommands = exports.ECLICommands || (exports.ECLICommands = {}));
const VerifyAllDeployItems = (plan, config, token) => {
    const runTypesAndObjectsToVerify = [
        { runType: Plan, object: plan },
        { runType: DeployConfig, object: config },
        { runType: Token, object: token },
    ];
    return Templates.ValidateMultipleRunTypesAndReturnSuccessOrErrorMessages(runTypesAndObjectsToVerify);
};
exports.VerifyAllDeployItems = VerifyAllDeployItems;
const VerifyAllPlanItems = (config, token) => {
    const runTypesAndObjectsToVerify = [
        { runType: PlanConfig, object: config },
        { runType: Token, object: token },
    ];
    return Templates.ValidateMultipleRunTypesAndReturnSuccessOrErrorMessages(runTypesAndObjectsToVerify);
};
exports.VerifyAllPlanItems = VerifyAllPlanItems;
const VerifyAllRefreshItems = (config, token) => {
    const runTypesAndObjectsToVerify = [
        { runType: RefreshConfig, object: config },
        { runType: Token, object: token },
    ];
    return Templates.ValidateMultipleRunTypesAndReturnSuccessOrErrorMessages(runTypesAndObjectsToVerify);
};
exports.VerifyAllRefreshItems = VerifyAllRefreshItems;
/**
 * Should be used in place of all process.exit() or process.exitCode = x instances. process.exit() kills all processes immediately and we lose logs and can
 * cause other issues, process.exitCode = x will hang indefinitely if firebase apps aren't deleted first
 * @param exitCode normal nodeJS exit code that would be passed to process.exit or process.exitCode
 */
const ExitCLISafe = (exitCode) => {
    // force exit if FB fails to exit (or less likely, logs fail to send) in 5 seconds
    const forceExitTimeout = setTimeout(() => {
        LogCLI.alert("CLI exit timeout");
        // inner timeout is so the above log can be sent to Datadog; proper methods for waiting on winston transports don't work on our datadog transport
        setTimeout(() => process.exit(exitCode), 1000);
    }, 5000);
    try {
        const firebase = Firebase.justFirebase;
        // Firebase Apps is an App[]
        if (firebase.apps.length) {
            firebase.apps.forEach((app) => {
                firebase.app(app.name).delete();
            });
        }
        clearTimeout(forceExitTimeout);
        process.exitCode = exitCode;
    }
    catch (error) {
        LogCLI.error("Error while attempting to close Firebase apps in CLI");
    }
};
exports.ExitCLISafe = ExitCLISafe;
const DisableNonCLIConsoleLogs = () => {
    const allLoggersConfigs = {};
    Object.values(Logging.LoggingArea).forEach(logArea => {
        if (logArea !== Logging.LoggingArea.CLI) {
            allLoggersConfigs[logArea] = {
                transports: [Logging.GetHttpTransport(), Logging.GetOurTransport()]
            };
        }
    });
    const CLIAreaInfo = Logging.LoggingAreas[Logging.LoggingArea.CLI];
    allLoggersConfigs[Logging.LoggingArea.CLI] = {
        transports: Logging.GetDefaultTransports(CLIAreaInfo)
    };
    Logging.ReconfigureLoggers(allLoggersConfigs);
};
exports.DisableNonCLIConsoleLogs = DisableNonCLIConsoleLogs;
//# sourceMappingURL=CLIOperations.js.map