"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllStepsForJob = exports.JobValue = exports.useCreateJobTreeData = exports.JobsTreeCategory = exports.useCreateJobRowData = exports.listenJobChangeCB = exports.useCurrentJob = void 0;
const react_1 = require("react");
const react_redux_1 = require("react-redux");
const CommonOperations = __importStar(require("./CommonOperations"));
const StepOperations = __importStar(require("./StepOperations"));
const State = __importStar(require("./State"));
const Common = __importStar(require("./Common"));
const useCurrentJob = () => {
    const currentJob = (0, react_redux_1.useSelector)(state => state.currentJob);
    return currentJob;
};
exports.useCurrentJob = useCurrentJob;
const listenJobChangeCB = (change, dispatch) => {
    const { type, doc } = change;
    if (type === 'added') {
        dispatch(State.getCreateJobAction(parseInt(doc.id), doc.data()));
    }
    else if (type === 'modified') {
        dispatch(State.getModifyJobAction(parseInt(doc.id), doc.data()));
    }
    else if (type === 'removed') {
        dispatch(State.getDeleteJobAction(parseInt(doc.id)));
    }
    else {
        console.log("unknown update type", type);
        console.assert(false);
    }
};
exports.listenJobChangeCB = listenJobChangeCB;
const useCreateJobRowData = () => {
    return (0, react_1.useCallback)((jobs) => {
        let jobsRowData = [{
                value: 'Jobs',
                heirarchy: [StepOperations.PayloadType.allJobsFolder],
                key: StepOperations.PayloadType.allJobsFolder,
                type: StepOperations.PayloadType.allJobsFolder
            }];
        Object.keys(jobs).forEach(jobCounter => {
            jobsRowData.push({
                type: StepOperations.PayloadType.job,
                heirarchy: [StepOperations.PayloadType.allJobsFolder, jobCounter],
                value: jobs[jobCounter].name,
                key: CommonOperations.generateKey([StepOperations.PayloadType.allJobsFolder, jobCounter]),
                jobCounter: parseInt(jobCounter)
            });
        });
        const sortedjobsRowData = Common.SortObjectsByField(jobsRowData, 'value');
        return sortedjobsRowData;
    }, []);
};
exports.useCreateJobRowData = useCreateJobRowData;
var JobsTreeCategory;
(function (JobsTreeCategory) {
    JobsTreeCategory["subgraphs"] = "Subgraphs";
    JobsTreeCategory["nodes"] = "Nodes";
})(JobsTreeCategory = exports.JobsTreeCategory || (exports.JobsTreeCategory = {}));
const useCreateJobTreeData = () => {
    return (0, react_1.useCallback)((jobData, subgraphs, steps) => {
        let jobsTreeData = [
            {
                value: JobsTreeCategory.subgraphs,
                heirarchy: [JobsTreeCategory.subgraphs],
                key: JobsTreeCategory.subgraphs
            },
            {
                value: JobsTreeCategory.nodes,
                heirarchy: [JobsTreeCategory.nodes],
                key: JobsTreeCategory.nodes
            }
        ];
        // create entries for subgraphs
        jobData.subgraphs.forEach(folderCounter => {
            const subgraphData = Common.getValueSafe(subgraphs, [folderCounter], null);
            if (!subgraphData) {
                console.info('subgraph doesn\'t exist'); // in either of these cases, should we just remove the counter from the job's metadata?
            }
            else {
                jobsTreeData.push({
                    value: subgraphs[folderCounter].name,
                    heirarchy: [JobsTreeCategory.subgraphs, subgraphs[folderCounter].name],
                    key: CommonOperations.generateKey([JobsTreeCategory.subgraphs, folderCounter.toString()]),
                    type: JobValue.subgraphs,
                    id: folderCounter
                });
            }
        });
        // create entries for steps
        jobData.steps.forEach(stepCounter => {
            const stepData = Common.getValueSafe(steps, [stepCounter], null);
            if (!stepData) {
                console.info('step doesn\'t exist');
            }
            else {
                jobsTreeData.push({
                    value: steps[stepCounter].operation.name,
                    heirarchy: [JobsTreeCategory.nodes, steps[stepCounter].operation.name],
                    key: CommonOperations.generateKey([JobsTreeCategory.nodes, stepCounter.toString()]),
                    type: JobValue.steps,
                    id: stepCounter
                });
            }
        });
        return jobsTreeData;
    }, []);
};
exports.useCreateJobTreeData = useCreateJobTreeData;
var JobValue;
(function (JobValue) {
    JobValue["steps"] = "steps";
    JobValue["subgraphs"] = "subgraphs";
})(JobValue = exports.JobValue || (exports.JobValue = {}));
const getAllStepsForJob = (job, allSubgraphs) => {
    let allSteps = new Set([]);
    job.subgraphs.forEach(subgraphCounter => {
        const currentSubgraph = Common.getValueSafe(allSubgraphs, [subgraphCounter], null);
        if (!currentSubgraph) {
            console.info('subgraph doesn\'t exist');
        }
        else {
            currentSubgraph.steps.forEach(stepCounter => {
                allSteps.add(stepCounter);
            });
        }
    });
    job.steps.forEach(stepCounter => allSteps.add(stepCounter));
    return Array.from(allSteps);
};
exports.getAllStepsForJob = getAllStepsForJob;
//# sourceMappingURL=JobOperations.js.map