"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.hydrateMetadata = exports.ConvertColumnReferenceToHydratedColumn = exports.GetHydratedTestsForColumn = exports.GetHydratedTestsForNode = exports.AddGenericSysColAttribute = exports.formatHydratedColumnMetadata = exports.HydrateColumnWithoutAttributeFlags = exports.ParseRunTimeParametersStringToObject = void 0;
const Runner = __importStar(require("./Runner"));
const Common = __importStar(require("./Common"));
const MetadataOperations = __importStar(require("./MetadataOperations"));
const Templates = __importStar(require("./Templates"));
const StepTypes = __importStar(require("./StepTypes"));
const GraphOperations = __importStar(require("./GraphOperations"));
const TestingOperations = __importStar(require("./TestingOperations"));
const Logging = __importStar(require("./Logging"));
const hydrationLogger = Logging.GetLogger(Logging.LoggingArea.MetadataHydration);
const RunTimeParametersLogger = Logging.GetLogger(Logging.LoggingArea.MetadataOperations);
const getColumnName = (columns, inputCounter) => {
    const outputColumn = columns.find((column) => {
        return column.columnReference.columnCounter === inputCounter;
    });
    return Common.getValueSafe(outputColumn, ["name"], "No Predecessor Column");
};
const getSourceColumn = (sourceColumnData, allSteps, stepTypesYaml) => {
    const stepCounter = Common.getValueSafe(sourceColumnData, ['columnReferences', 0, 'stepCounter'], null);
    const columnCounter = Common.getValueSafe(sourceColumnData, ['columnReferences', 0, 'columnCounter'], null);
    const sourceStep = allSteps[stepCounter];
    if (!stepCounter || !columnCounter || !sourceStep)
        return {};
    const sourceColumn = MetadataOperations.findColumnByColumnCounter(sourceStep.operation, columnCounter);
    if (!sourceColumn)
        return {};
    return {
        node: FormatHydratedNodeMetadata(sourceStep.operation, stepCounter, stepTypesYaml),
        column: ConvertV0ToV1ColumnMetadata(sourceColumn)
    };
};
const formatHydratedMetadataMultiSources = (SQLOperation, allSteps, stepTypesYaml, specifiedIndex) => {
    if (SQLOperation.sqlType === Runner.ESQLType.source) {
        return null;
    }
    else {
        let output;
        hydrationLogger.debug(`in formatHydratedMetadataMultiSources: formatting for ${SQLOperation.name} at specifiedIndex: ${specifiedIndex}`);
        if (specifiedIndex !== null) {
            output = [formatHydratedSourceMetadata(SQLOperation, allSteps, stepTypesYaml, specifiedIndex)];
        }
        else {
            if (!!SQLOperation.isMultisource) {
                output = SQLOperation.metadata.sourceMapping.map((mapping, index) => {
                    hydrationLogger.debug(`in formatHydratedMetadataMultiSources: formatting at index ${index}. (multisource)`);
                    return formatHydratedSourceMetadata(SQLOperation, allSteps, stepTypesYaml, index);
                });
            }
            else {
                output = [formatHydratedSourceMetadata(SQLOperation, allSteps, stepTypesYaml, 0)];
            }
        }
        return output;
    }
};
const formatHydratedSourceMetadata = (SQLOperation, allSteps, stepTypesYaml, index) => {
    const sqlType = SQLOperation.sqlType;
    const stepTypeParsed = StepTypes.parseStepTypeYAML(stepTypesYaml, sqlType);
    const sourceMap = SQLOperation.metadata.sourceMapping[index];
    const sourceMetadata = {
        name: sourceMap.name,
        columns: SQLOperation.metadata.columns.map((column) => {
            const sourceColumnData = column.sourceColumnReferences[index];
            const formattedColumn = (0, exports.formatHydratedColumnMetadata)(column);
            const hashDetails = hydrateHashColumn(SQLOperation, column, allSteps, stepTypesYaml, index);
            const newColumn = Object.assign(Object.assign({}, formattedColumn), { id: column.columnReference.columnCounter, transform: sourceColumnData ? sourceColumnData.transform : "", sourceColumns: [getSourceColumn(sourceColumnData, allSteps, stepTypesYaml)] });
            if (hashDetails !== null) {
                newColumn.hashDetails = hashDetails;
            }
            return newColumn;
        }),
        join: sourceMap.join.joinCondition,
        dependencies: FormatHydratedSourceNodes({ operation: SQLOperation }, index, allSteps, stepTypesYaml),
        customSQL: MetadataOperations.getCustomSQL({ operation: SQLOperation }, index)
    };
    (0, exports.AddGenericSysColAttribute)(sourceMetadata.columns, sqlType, { [sqlType]: stepTypeParsed });
    return sourceMetadata;
};
const FormatHydratedSourceNodes = (stepData, specifiedIndex, allSteps, stepTypesYaml) => {
    const dependencyStepCountersForSource = GraphOperations.getDependenciesForStep(allSteps, stepData, specifiedIndex);
    return dependencyStepCountersForSource.map(stepCounter => {
        const stepData = allSteps[stepCounter];
        if (!stepData)
            return {};
        return {
            node: FormatHydratedNodeMetadata(stepData.operation, stepCounter, stepTypesYaml),
            columns: stepData.operation.metadata.columns.map(column => (0, exports.formatHydratedColumnMetadata)(column))
        };
    });
};
const ParseRunTimeParametersStringToObject = (runTimeParametersToFormat) => {
    let runTimeParametersToReturn;
    if (runTimeParametersToFormat === "" || !runTimeParametersToFormat) {
        return runTimeParametersToReturn = {};
    }
    try {
        runTimeParametersToReturn = JSON.parse(runTimeParametersToFormat);
    }
    catch (error) {
        const errorMessage = "Run Time Parameters failed to parse.";
        RunTimeParametersLogger.errorContext({}, `${errorMessage} ${runTimeParametersToFormat}`, error);
        throw new Error(`Failed to parse runTimeParameters: ${error.message}`);
    }
    Common.assert(Logging.LoggingArea.Environments, !!runTimeParametersToReturn, `Run Time Parameters failed to parse because it contains invalid JSON. Run Time Parameters: ${runTimeParametersToFormat}`);
    return runTimeParametersToReturn;
};
exports.ParseRunTimeParametersStringToObject = ParseRunTimeParametersStringToObject;
const FormatHydratedNodeMetadata = (sqlOperation, stepCounter, stepTypesYaml) => {
    return {
        id: stepCounter,
        name: sqlOperation.name,
        nodeType: StepTypes.getStepTypeNameSafe(stepTypesYaml, sqlOperation.sqlType),
        location: {
            name: sqlOperation.locationName,
            id: sqlOperation.locationID,
        },
        description: sqlOperation.description || "",
        materializationType: sqlOperation.materializationType || Templates.GetDefaultMaterializationType(sqlOperation.sqlType),
        isMultisource: sqlOperation.isMultisource || false,
        override: {
            create: {
                enabled: sqlOperation.overrideSQL || false,
                script: MetadataOperations.getCustomSQL({ operation: sqlOperation }, 0)
            }
        },
        tests: (0, exports.GetHydratedTestsForNode)(sqlOperation)
    };
};
var ColumnMetadataVersions;
(function (ColumnMetadataVersions) {
    ColumnMetadataVersions["v0"] = "v0";
    ColumnMetadataVersions["v1"] = "v1";
})(ColumnMetadataVersions || (ColumnMetadataVersions = {}));
/**
 * currently this function is not being used, though it may be used in the future when greater
 *    version controlling for column metadata is required
 * @param column
 */
const lookupColumnMetadataVersion = (column) => {
    if (column.hasOwnProperty("keyColumnType")
        || column.hasOwnProperty("systemColumnType")
        || column.hasOwnProperty("columnReferences")
        || column.hasOwnProperty("sourceColumnReferences")) {
        return ColumnMetadataVersions.v0;
    }
    return ColumnMetadataVersions.v1;
};
const AddKeyColumnFlags = (column, output) => {
    for (let attribute in Runner.FirstClassAttributeFlags) {
        const attrValue = Runner.FirstClassAttributeFlags[attribute];
        if (!!column[attrValue]
            || (column.keyColumnType && column.keyColumnType === Runner.V0KeyColumnTypeFlagConversion[attrValue])) {
            output[attrValue] = true;
        }
    }
};
const AddSystemColumnFlags = (column, output) => {
    for (let attribute in Runner.systemColumnTypeFlags) {
        const attrValue = Runner.systemColumnTypeFlags[attribute];
        if (!!column[attrValue]
            || (column.systemColumnType && column.systemColumnType === Runner.V0SystemColumnTypeFlagConversion[attrValue])) {
            output[attrValue] = true;
        }
    }
};
/**
 * This function takes in an original object and a desired 'output' object and adds a set of defined
 *  'column selector' fields to the output object. This function was extracted out so it could be
 *  used for hydrating columns and source columns
 * The user config object is meant for user-defined fields
 */
const AddColumnSelectorFields = (column, output) => {
    //for adding first class attributes like isBusinessKey and isSurrogateKey
    AddKeyColumnFlags(column, output);
    //for adding system columns, like isSystemCreateDate, and isSystemUpdateDate
    AddSystemColumnFlags(column, output);
    if (!!column.config) {
        Object.entries(column.config).forEach(configEntry => {
            output[configEntry[0]] = configEntry[1];
        });
    }
};
const HydrateColumnWithoutAttributeFlags = (column) => {
    var _a;
    return {
        id: column.columnReference.columnCounter,
        name: column.name,
        dataType: column.dataType,
        description: column.description,
        nullable: column.nullable,
        defaultValue: (_a = column.defaultValue) !== null && _a !== void 0 ? _a : "",
        tests: (0, exports.GetHydratedTestsForColumn)(column)
    };
};
exports.HydrateColumnWithoutAttributeFlags = HydrateColumnWithoutAttributeFlags;
const ConvertV0ToV1ColumnMetadata = (column) => {
    /*
     * v1 is the standard for templates going forward - if it's v0, convert to v1
     * - if they exist:
     *             - as we're hydrating, we're checking for keyColumnType === KeyColumnTypes.primaryBusinessKey. (value from an enum we've been using)
     *             this translates to column.isBusinessKey
     *             - keyColumnType === surrogateKey translates to column.isSurrogateKey
     *             - systemColumnType == SystemColumnTypes
     */
    const output = (0, exports.HydrateColumnWithoutAttributeFlags)(column);
    /*
     *  Data Vault
     * need to make sure that data vault hash columns don't lose what hash they are descended from
     */
    if (!!column.descendedFrom) {
        output[`is${column.descendedFrom}`] = true;
    }
    if (!!column.hashDetails && column.hashDetails.hashType) {
        output[`is${column.hashDetails.hashType}`] = true;
    }
    AddColumnSelectorFields(column, output);
    return output;
};
const hydrateHashColumn = (SQLStep, column, allSteps, stepTypesYaml, index) => {
    const hashedColumns = column.hashedColumns;
    //only hash columns have this field
    if (hashedColumns === null || hashedColumns === void 0 ? void 0 : hashedColumns.length) {
        const allColumns = SQLStep.metadata.columns;
        // hashed columns includes empty rows in hash editor, filtering those out here
        const validHashedColumns = hashedColumns.filter(column => {
            return !!Common.getValueSafe(column, ["columnCounter"], null);
        });
        let columnMetadataForHash = [];
        validHashedColumns.forEach((hashedColumn) => {
            const { columnCounter } = hashedColumn;
            const matchingColumn = allColumns.find(column => column.columnReference.columnCounter == columnCounter);
            if (matchingColumn) {
                const formattedColumn = (0, exports.formatHydratedColumnMetadata)(matchingColumn);
                const sourceColumnData = matchingColumn.sourceColumnReferences[index];
                Common.assert(Logging.LoggingArea.MetadataHydration, !!sourceColumnData, `sourceColumnData undefined in hydrateHashColumn for column ${columnCounter} at index ${index}`);
                columnMetadataForHash.push(Object.assign(Object.assign({}, formattedColumn), { transform: (sourceColumnData === null || sourceColumnData === void 0 ? void 0 : sourceColumnData.transform) || "", sourceColumns: [getSourceColumn(sourceColumnData, allSteps, stepTypesYaml)] }));
            }
        });
        return { columns: columnMetadataForHash, algorithm: column.hashDetails.hashAlgorithm };
    }
    return null;
};
/**
 * This function is to be updated whenever a new version of metadata is added
 *    said update should add a new case for updating version x-1 to version x,
 *    and then return the new version x
 *
 * For greater controlling of metadata shape there is currently no actual version checking
 *    and the column is simply assumed to need converting to V1. This, along with
 *    lookupColumnMetadataVersion will need to be re-examined in the future when column
 *    metadata reaches v2 or v3
 */
const formatHydratedColumnMetadata = (column) => {
    // if (lookupColumnMetadataVersion(column) === ColumnMetadataVersions.v0) {
    return ConvertV0ToV1ColumnMetadata(column);
    /*
     * }
     * return column as FullHydratedColumn;
     */
};
exports.formatHydratedColumnMetadata = formatHydratedColumnMetadata;
const AddGenericSysColAttribute = (columns, sqlType, stepTypesParsed) => {
    const defaultSysColTypeFlags = new Set(Object.values(Runner.systemColumnTypeFlags));
    const nodeSpec = StepTypes.getNodeSpecSafe(stepTypesParsed, sqlType);
    const stepTypeSystemColumns = nodeSpec === null || nodeSpec === void 0 ? void 0 : nodeSpec.systemColumns;
    if (!!(stepTypeSystemColumns === null || stepTypeSystemColumns === void 0 ? void 0 : stepTypeSystemColumns.length)) {
        columns.forEach(column => {
            const matchingSysCol = stepTypeSystemColumns.find(sysCol => sysCol.displayName === column.name);
            if (!!(matchingSysCol === null || matchingSysCol === void 0 ? void 0 : matchingSysCol.attributeName) && !defaultSysColTypeFlags.has(matchingSysCol.attributeName)) {
                column[matchingSysCol.attributeName] = true;
            }
        });
    }
};
exports.AddGenericSysColAttribute = AddGenericSysColAttribute;
const InitializeHydratedTest = (test, templateString) => {
    const { name, description, continueOnFailure, runOrder } = test;
    return {
        name,
        description,
        continueOnFailure,
        runOrder,
        templateString
    };
};
const GetHydratedTestsForNode = (sqlOperation) => {
    const nodeTests = TestingOperations.GetAppliedNodeTestsSafe(sqlOperation);
    return nodeTests.map(test => {
        return InitializeHydratedTest(test, test.templateString);
    });
};
exports.GetHydratedTestsForNode = GetHydratedTestsForNode;
const GetHydratedTestsForColumn = (column) => {
    const columnTests = TestingOperations.GetAppliedColumnTestsDefinitions(column);
    return columnTests.map(test => {
        const templateString = test.getTemplateString(column);
        return InitializeHydratedTest(test, templateString);
    });
};
exports.GetHydratedTestsForColumn = GetHydratedTestsForColumn;
const ConvertColumnReferenceToHydratedColumn = (colReference, stepData) => {
    const columnsData = Common.getValueSafe(stepData, ["metadata", "columns"], []);
    const column = columnsData.find(col => {
        return col.columnReference.columnCounter === colReference.columnCounter;
    });
    if (!column) {
        return {};
    }
    else {
        return (0, exports.HydrateColumnWithoutAttributeFlags)(column);
    }
};
exports.ConvertColumnReferenceToHydratedColumn = ConvertColumnReferenceToHydratedColumn;
/**
 *
 * @param SQLStep sqlOperation metadata for step that's being hydrated
 * @param stepCounter stepID for the SQLStep arg
 * @param currentState
 * all state - right now only used for all steps, but is flexible for more additions in the future.
 * is also compatible with workspaceData being passed in when used in the deploy process
 * @param workspaceData
 * needs to be separate from above, because this is also used in the deploy process. deploy
 * workspace data (i.e. workspaceData.workspaces) doesn't include the necessary metadata for this process
 * (specifically currentWorkspace)
 * @param specifiedIndex for hydrating a specific source in multisource
 * @returns hydratedMetadata, formatted to be used by the JSON template API
 */
const hydrateMetadata = (SQLStep, stepCounter, allSteps, stepTypesYaml, workspaceData, specifiedIndex, runTimeParameters) => {
    // format columns
    hydrationLogger.info(`Hydrating metadata for step ${stepCounter} ${specifiedIndex ? `at specified index ${specifiedIndex}` : ""}`);
    StepTypes.ThrowIfStepTypeDoesntExist(stepTypesYaml, SQLStep.sqlType);
    const formattedColumns = SQLStep.metadata.columns.map(col => (0, exports.formatHydratedColumnMetadata)(col));
    /*
     * format storageLocations
     * convert currentWorkspaceData.mappings to array for IStorageLocationData
     */
    const currentWorkspaceData = workspaceData.workspaces[workspaceData.currentWorkspace];
    console.assert(!!currentWorkspaceData.id && !!currentWorkspaceData.name && !!currentWorkspaceData.mappings);
    const formattedStorageLocations = Object.values(currentWorkspaceData.mappings).map((mapping) => {
        return {
            database: mapping.database,
            schema: mapping.schema,
            name: mapping.locationName,
            id: mapping.id,
        };
    });
    // format workspace
    const formattedWorkspaceData = {
        //need to figure out why these are optional
        id: currentWorkspaceData.id,
        name: currentWorkspaceData.name
    };
    // format config - should not have to format actually
    const config = Common.getValueSafe(SQLStep, ["config"], {}); // this is adding a new attribute on our metadata, but will make it really easy to grab user specified attributes
    // format sources / multisourceColumns
    const formattedSources = formatHydratedMetadataMultiSources(SQLStep, allSteps, stepTypesYaml, specifiedIndex);
    // format node
    const nodeData = FormatHydratedNodeMetadata(SQLStep, stepCounter, stepTypesYaml);
    // format runTimeParameters
    const formattedRunTimeParameters = (0, exports.ParseRunTimeParametersStringToObject)(runTimeParameters);
    Common.assert(Logging.LoggingArea.Environments, !!formattedRunTimeParameters, `Formatted Run Time Parameters failed to parse because it contains invalid JSON. Run Time Parameters: ${runTimeParameters}`);
    // put it all together
    const hydratedMetadata = {
        columns: formattedColumns,
        storageLocations: formattedStorageLocations,
        workspace: formattedWorkspaceData,
        config,
        sources: formattedSources,
        node: nodeData,
        parameters: formattedRunTimeParameters
    };
    const stepTypeParsed = StepTypes.parseStepTypeYAML(stepTypesYaml, SQLStep.sqlType);
    (0, exports.AddGenericSysColAttribute)(hydratedMetadata.columns, SQLStep.sqlType, { [SQLStep.sqlType]: stepTypeParsed });
    return hydratedMetadata;
};
exports.hydrateMetadata = hydrateMetadata;
//# sourceMappingURL=MetadataHydration.js.map