"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScannerFuncDictionary = exports.ScanOrgsForMetadataProblems = void 0;
const SchedulerOperations = __importStar(require("./SchedulerOperations"));
/**
 * Function to be called using scheduler with --scanner arg. Arg will determine the scannerFunc to be used. Will scan all orgs within
 * given firestore org collection for a problem specified by scannerFunc
 * @param scannerFunc callback that is run for every org within the firestore to check for a specific metadata problem
 * @returns an array of objects containing orgs that have the problem specified by scannerFunc, and can include additional info
 * about the nature of the problem
 */
const ScanOrgsForMetadataProblems = (scannerFunc) => {
    const admin = SchedulerOperations.FirebaseExport;
    const firestore = admin.firestore();
    const orgsWithProblems = {};
    return firestore.collection("orgs").listDocuments()
        .then(docRefs => {
        const scannerPromises = docRefs.map(orgDocRef => {
            return scannerFunc(orgDocRef)
                .then(scanProblemInfo => {
                if (scanProblemInfo) {
                    orgsWithProblems[orgDocRef.id] = scanProblemInfo;
                }
            });
        });
        return Promise.all(scannerPromises);
    }).then(() => {
        return orgsWithProblems;
    });
};
exports.ScanOrgsForMetadataProblems = ScanOrgsForMetadataProblems;
// multiple dev envs will soon be supported, this is a testing/example function and may still be useful for the near future
const dupeDevEnvScanner = (orgDocRef) => {
    const workspaceCollRef = orgDocRef.collection("workspaces");
    const devWorkspaces = [];
    return workspaceCollRef.get()
        .then(workspaceSnapshot => {
        const allWorkspaceData = workspaceSnapshot.docs.map(workspaceDoc => workspaceDoc.data());
        allWorkspaceData.forEach(workspaceData => {
            if (workspaceData.devEnv) {
                devWorkspaces.push((workspaceData.id).toString());
            }
        });
        return devWorkspaces.length <= 1 ? null : { devEnvIDs: devWorkspaces };
    });
};
exports.ScannerFuncDictionary = {
    dupeDevEnv: dupeDevEnvScanner
};
//# sourceMappingURL=OrgScanner.js.map