"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRequestTreeFromJson = exports.ForAllNodes = exports.SaveRequestTree = exports.RequestTree = exports.RemoveNode = exports.RequestNode = exports.getRequestNodeFromJson = exports.GetContainingFolders = exports.Folder = exports.getRequestTreeDispatch = exports.getKey = exports.GetIDForKey = exports.GetTypeForKey = exports.RequestNodeType = void 0;
const Data = __importStar(require("./Data"));
var RequestNodeType;
(function (RequestNodeType) {
    RequestNodeType["folder"] = "folder";
    RequestNodeType["request"] = "request";
    RequestNodeType["task"] = "task";
    RequestNodeType["api"] = "api";
})(RequestNodeType = exports.RequestNodeType || (exports.RequestNodeType = {}));
const GetTypeForKey = (str) => {
    let res = str.match(`^(request|task|folder)-([0-9]+)$`);
    return (res[1]);
};
exports.GetTypeForKey = GetTypeForKey;
const GetIDForKey = (str) => {
    let res = str.match(`^(request|task|folder)-([0-9]+)$`);
    return parseInt(res[2]);
};
exports.GetIDForKey = GetIDForKey;
const getKey = (type, id) => {
    return `${type}-${id}`;
};
exports.getKey = getKey;
const getRequestTreeDispatch = (newTree) => {
    //return State.getChangeRequestTree(newTree)
};
exports.getRequestTreeDispatch = getRequestTreeDispatch;
class Folder {
    constructor() {
        this.folderID = 0;
        this.name = '';
    }
}
exports.Folder = Folder;
const GetContainingFolders = (taskID, requestNodes) => {
    let folderPath = [];
    for (let childIndex in requestNodes) {
        let child = requestNodes[childIndex];
        if (child.nodeID == taskID) {
            return child;
        }
        let path = (0, exports.GetContainingFolders)(taskID, child.children);
        if (path) {
            return [child, ...path];
        }
    }
    return null;
};
exports.GetContainingFolders = GetContainingFolders;
const getRequestNodeFromJson = (jsonObject) => {
    let newNode = new RequestNode(jsonObject.nodeType, jsonObject.nodeID);
    for (let child in jsonObject['children']) {
        let childNode = (0, exports.getRequestNodeFromJson)(jsonObject['children'][child]);
        newNode.children.push(childNode);
    }
    return newNode;
};
exports.getRequestNodeFromJson = getRequestNodeFromJson;
class RequestNode {
    constructor(nodeType, nodeID) {
        this.children = [];
        this.key = () => {
            return `${this.nodeType}-${this.nodeID}`;
        };
        this.clone = () => {
            let newNode = new RequestNode(this.nodeType, this.nodeID);
            for (let child in this.children) {
                newNode.children.push(this.children[child].clone());
            }
            return newNode;
        };
        this.callForKey = (key, cb) => {
            for (let child in this.children) {
                if (this.children[child].key() == key) {
                    cb(this.children[child], child, this.children);
                }
                else {
                    this.children[child].callForKey(key, cb);
                }
            }
        };
        this.nodeType = nodeType;
        this.nodeID = nodeID;
    }
}
exports.RequestNode = RequestNode;
const RemoveNode = (tree, id, type) => {
    let newTree = tree.clone();
    let keyToRemove = (0, exports.getKey)(type, id);
    newTree.callForKey(keyToRemove, (n, index, parentChildren) => {
        for (let i = 0; i < parentChildren.length; i++) {
            if (parentChildren[i].key() === keyToRemove) {
                parentChildren.splice(i, 1);
            }
        }
    });
    console.log("removed node", tree, newTree);
    return (0, exports.SaveRequestTree)(newTree);
};
exports.RemoveNode = RemoveNode;
class RequestTree {
    constructor() {
        this.children = [];
        this.clone = () => {
            let newTree = new RequestTree();
            for (let child in this.children) {
                newTree.children.push(this.children[child].clone());
            }
            return newTree;
        };
        this.callForKey = (key, cb) => {
            for (let child in this.children) {
                if (this.children[child].key() == key) {
                    cb(this.children[child], child, this.children);
                    return;
                }
                else {
                    this.children[child].callForKey(key, cb);
                }
            }
        };
    }
}
exports.RequestTree = RequestTree;
const SaveRequestTree = (requestTree) => {
    let persister = Data.getPersister();
    if (!persister) {
        console.assert(false);
    }
    // persister!.getStore().dispatch(State.getChangeRequestTree(requestTree))
};
exports.SaveRequestTree = SaveRequestTree;
const ForAllNodes = (requestNodes, cb) => {
    for (let node in requestNodes) {
        cb(requestNodes[node]);
        (0, exports.ForAllNodes)(requestNodes[node]['children'], cb);
    }
};
exports.ForAllNodes = ForAllNodes;
const getRequestTreeFromJson = (jsonObject) => {
    let newRequestTree = new RequestTree();
    if (!jsonObject || !('children' in jsonObject)) {
        console.log("got empty request tree? initializing?");
        return newRequestTree;
    }
    for (let child in jsonObject['children']) {
        newRequestTree.children.push((0, exports.getRequestNodeFromJson)(jsonObject['children'][child]));
    }
    return newRequestTree;
};
exports.getRequestTreeFromJson = getRequestTreeFromJson;
//# sourceMappingURL=RequestTree.js.map