"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CleanupRunsOnStartUp = exports.HEARTBEAT_CHECK_TIME = exports.StartHeartbeatInterval = exports.ReleaseWorkspaceAndFailRefresh = exports.ReleaseWorkspaceAndSucceedRefresh = exports.ReleaseWorkspaceAndFailDeploy = exports.ReleaseWorkspaceAndSucceedDeploy = exports.SecureWorkspaceAndStartRefresh = exports.SecureWorkspaceAndStartDeploy = void 0;
const Immer = __importStar(require("immer"));
const Common = __importStar(require("./Common"));
const Runner = __importStar(require("./Runner"));
const Logging = __importStar(require("./Logging"));
const Workspaces = __importStar(require("./Workspaces"));
const CommonOperations = __importStar(require("./CommonOperations"));
const Firebase = __importStar(require("./Firebase"));
const Logger = Logging.GetLogger(Logging.LoggingArea.RunTransaction);
const RUN_VERSION_NUMBER = 2;
/**
 * Transactionally acquire an environment and save runInfo before Starting a Job (Deploy/Refresh)
 * @param firebase
 * @param teamID
 * @param runInfo
 * @param workspaceID
 * @param newStatus
 * @returns
 */
const SecureWorkspaceAndStartRun = (firebase, teamID, runInfo, workspaceID, workspaceUpdaterFunction) => {
    const firestore = firebase.firestore();
    let newRunID;
    const loggerContext = { orgID: teamID, envID: workspaceID };
    const orgDocReference = CommonOperations.getOrgDataDocRefAdmin(firestore, teamID);
    const workspaceDocRefenerence = Workspaces.getWorkspacesCollectionRefAdmin(firestore, teamID).doc(workspaceID.toString());
    const fieldValue = Firebase.GetFieldValueFromFirebaseInstance(firebase);
    const orgRunReference = CommonOperations.getOrgDataDocRefAdmin(firestore, teamID).collection("runs");
    return new Promise((resolve, reject) => {
        return firestore.runTransaction((transaction) => {
            //all reads first, then writes in body
            return Promise.all([transaction.get(orgDocReference), transaction.get(workspaceDocRefenerence)]).then((transactionGetData) => {
                const [orgData, workspaceData] = transactionGetData;
                const properOrgData = orgData.data();
                const nextRunID = (properOrgData.runCounter || 0) + 1;
                newRunID = nextRunID;
                const properWorkspaceData = workspaceData.data();
                const workspaceDocUpdate = workspaceUpdaterFunction(properWorkspaceData, [nextRunID]);
                ReserveRunWithTransactionWrite(transaction, firebase, teamID, runInfo, nextRunID);
                //write new runCounter to orgDoc
                UpdateTransactionWrite(transaction, workspaceDocRefenerence, workspaceDocUpdate);
                UpdateTransactionWrite(transaction, orgDocReference, { runCounter: nextRunID });
            });
        })
            .then(() => {
            //move HeartBeat kickoff outside of transaction so we don't accidently start erroneous heartbeat ticks and uncouple from transaction
            Logger.infoContext(loggerContext, `Successfully secured Environment ${workspaceID} in org ${teamID} for run ${newRunID}`);
            const newRunDocReference = orgRunReference.doc(newRunID.toString());
            resolve({ runCounter: newRunID, CleanupHeartbeat: (0, exports.StartHeartbeatInterval)(fieldValue, newRunDocReference) });
        })
            .catch((error) => {
            var _a, _b;
            debugger;
            Logger.errorContext(loggerContext, `Failed to secure Environment ${workspaceID} in org ${teamID}: ${(_a = error.error) === null || _a === void 0 ? void 0 : _a.errorString},\n ${(_b = error.error) === null || _b === void 0 ? void 0 : _b.errorDetail}`);
            reject(error);
        });
    });
};
const SecureWorkspaceAndStartDeploy = (firebase, teamID, runInfo, workspaceID) => {
    return SecureWorkspaceAndStartRun(firebase, teamID, runInfo, workspaceID, CreateWorkspaceUpdateForDeployAcquisition);
};
exports.SecureWorkspaceAndStartDeploy = SecureWorkspaceAndStartDeploy;
const SecureWorkspaceAndStartRefresh = (firebase, teamID, runInfo, workspaceID) => {
    return SecureWorkspaceAndStartRun(firebase, teamID, runInfo, workspaceID, CreateWorkspaceUpdateForRefreshAcquisition);
};
exports.SecureWorkspaceAndStartRefresh = SecureWorkspaceAndStartRefresh;
const ReleaseWorkspaceAndSucceedDeploy = (firebase, teamID, workspaceID, runID) => {
    return ReleaseWorkspaceStatusAndEndRun(firebase, teamID, workspaceID, Runner.ERunStatus.completed, runID, CreateWorkspaceUpdateForSuccessfulDeployRelease);
};
exports.ReleaseWorkspaceAndSucceedDeploy = ReleaseWorkspaceAndSucceedDeploy;
const ReleaseWorkspaceAndFailDeploy = (firebase, teamID, workspaceID, runID) => {
    return ReleaseWorkspaceStatusAndEndRun(firebase, teamID, workspaceID, Runner.ERunStatus.failed, runID, CreateWorkspaceUpdateForFailedDeployRelease);
};
exports.ReleaseWorkspaceAndFailDeploy = ReleaseWorkspaceAndFailDeploy;
const ReleaseWorkspaceAndSucceedRefresh = (firebase, teamID, workspaceID, runID) => {
    return ReleaseWorkspaceStatusAndEndRun(firebase, teamID, workspaceID, Runner.ERunStatus.completed, runID, CreateWorkspaceUpdateForRefreshRelease);
};
exports.ReleaseWorkspaceAndSucceedRefresh = ReleaseWorkspaceAndSucceedRefresh;
const ReleaseWorkspaceAndFailRefresh = (firebase, teamID, workspaceID, runID) => {
    return ReleaseWorkspaceStatusAndEndRun(firebase, teamID, workspaceID, Runner.ERunStatus.failed, runID, CreateWorkspaceUpdateForRefreshRelease);
};
exports.ReleaseWorkspaceAndFailRefresh = ReleaseWorkspaceAndFailRefresh;
/**
 * Transactionally release an environment and mark a Job (Deploy/Refresh) done (or failed)
 * @param firebase
 * @param teamID
 * @param workspaceID
 * @param oldExpectedStatus
 * @param newStatus
 * @param runID
 * @returns
 */
const ReleaseWorkspaceStatusAndEndRun = (firebase, teamID, workspaceID, newRunStatus, runID, workspaceUpdaterFunction) => {
    const firestore = firebase.firestore();
    const loggerContext = { orgID: teamID, envID: workspaceID };
    const orgRunCollectionReference = CommonOperations.getOrgDataDocRefAdmin(firestore, teamID).collection("runs");
    const runDocReference = orgRunCollectionReference.doc(runID.toString());
    const workspaceDocRefenerence = Workspaces.getWorkspacesCollectionRefAdmin(firestore, teamID).doc(workspaceID.toString());
    const fieldValue = Firebase.GetFieldValueFromFirebaseInstance(firebase);
    return new Promise((resolve, reject) => {
        return firestore.runTransaction((transaction) => {
            return transaction.get(workspaceDocRefenerence)
                .then(workspaceDoc => {
                const workspaceData = workspaceDoc.data();
                const workspaceUpdate = workspaceUpdaterFunction(workspaceData, [runID]);
                UpdateTransactionWrite(transaction, workspaceDocRefenerence, workspaceUpdate);
                UpdateTransactionWrite(transaction, runDocReference, { runStatus: newRunStatus, endTime: fieldValue.serverTimestamp() });
            });
        })
            .then(() => {
            Logger.infoContext(loggerContext, `Successfully released Environment ${workspaceID} in org ${teamID} for run ${runID}`);
            resolve();
        })
            .catch((error) => {
            Logger.errorContext(loggerContext, `Failed to release Environment ${workspaceID} in org ${teamID} for run ${runID}. Error: ${error}`);
            reject(error);
        });
    });
};
const UpdateTransactionWrite = (transaction, documentRefenence, update) => {
    transaction.update(documentRefenence, update);
    return;
};
const SanitizeRunInfoForSave = (runInfo, newRunID) => {
    const newRunInfo = Immer.produce(runInfo, (draftState) => {
        if (runInfo.runDetails["dividedStepDataWithWorkspaceEdits"]) {
            delete draftState.runDetails["dividedStepDataWithWorkspaceEdits"];
            delete draftState.runDetails["fromWorkspaceData"];
        }
        draftState.userCredentials = null;
        draftState.id = newRunID;
        draftState.version = RUN_VERSION_NUMBER;
        draftState.runDetails["stepsInRun"] = 0;
    });
    return newRunInfo;
};
const ReserveRunWithTransactionWrite = (transaction, firebase, teamID, runInfo, runIDForNewRun) => {
    const newRunInfo = SanitizeRunInfoForSave(runInfo, runIDForNewRun);
    const firestore = firebase.firestore();
    const orgRunReference = CommonOperations.getOrgDataDocRefAdmin(firestore, teamID).collection("runs");
    const newRunDocReference = orgRunReference.doc(runIDForNewRun.toString());
    const fieldValue = Firebase.GetFieldValueFromFirebaseInstance(firebase);
    transaction.set(newRunDocReference, Object.assign(Object.assign({}, newRunInfo), { runStartTime: fieldValue.serverTimestamp(), runHeartbeat: fieldValue.serverTimestamp() }));
};
const CheckWorkspaceStatusIsAbleToChange = (workspaceData, oldExpectedStatuses, newStatus) => {
    /**
     * in the event that the status is undefined: for old development workspaces... we pass and set status next
     * using an array here allows us to have multiple options eg. deploys can start if we are in failed or
     * waiting state, but refresh is waiting/refresh only
     */
    if (workspaceData.devEnv) {
        throw Common.CreateErrorWithCode(400, `Unable to refresh ${workspaceData.name}`, `${workspaceData.name} is a development environment`);
    }
    else if (workspaceData.status && !oldExpectedStatuses.includes(workspaceData.status)) {
        throw Common.CreateErrorWithCode(400, `Unable to start ${newStatus}`, `${workspaceData.name} is ${workspaceData.status}.`);
    }
};
const CreateWorkspaceUpdateForDeployAcquisition = (workspaceData, runIDs) => {
    //deploys can start from any state, except deploy/refresh as something is already running
    const oldStatuses = [Runner.EWorkspaceHealth.waiting, Runner.EWorkspaceHealth.failedDeploy, Runner.EWorkspaceHealth.failedRefresh];
    //failedRefresh is deprecated, but we're keeping the value around to support existing environments
    CheckWorkspaceStatusIsAbleToChange(workspaceData, oldStatuses, Runner.EWorkspaceHealth.deploying);
    return { status: Runner.EWorkspaceHealth.deploying, currentlyRunningJobs: runIDs };
};
const CreateWorkspaceUpdateForSuccessfulDeployRelease = (workspaceData, runIDs) => {
    const oldStatuses = [Runner.EWorkspaceHealth.deploying];
    //if we're releasing a deploy, the only valid state for oldStatus is deploying
    CheckWorkspaceStatusIsAbleToChange(workspaceData, oldStatuses, Runner.EWorkspaceHealth.waiting);
    return { status: Runner.EWorkspaceHealth.waiting, currentlyRunningJobs: [] };
};
const CreateWorkspaceUpdateForFailedDeployRelease = (workspaceData, runIDs) => {
    const oldStatuses = [Runner.EWorkspaceHealth.deploying];
    //if we're releasing a deploy, the only valid state for oldStatus is deploying
    CheckWorkspaceStatusIsAbleToChange(workspaceData, oldStatuses, Runner.EWorkspaceHealth.failedDeploy);
    return { status: Runner.EWorkspaceHealth.failedDeploy, currentlyRunningJobs: [] };
};
const CreateWorkspaceUpdateForRefreshAcquisition = (workspaceData, runIDs) => {
    //refreshes can start from waiting or (old) failed Refresh
    const oldStatuses = [Runner.EWorkspaceHealth.waiting, Runner.EWorkspaceHealth.failedRefresh, Runner.EWorkspaceHealth.refreshing];
    //failedRefresh is deprecated, but we're keeping the value around to support existing environments
    CheckWorkspaceStatusIsAbleToChange(workspaceData, oldStatuses, Runner.EWorkspaceHealth.refreshing);
    let currentlyRunningJobs = [];
    if (workspaceData.currentlyRunningJobs) {
        currentlyRunningJobs = [...workspaceData.currentlyRunningJobs];
    }
    return { status: Runner.EWorkspaceHealth.refreshing, currentlyRunningJobs: [...currentlyRunningJobs, ...runIDs] };
};
/**
 * Used for the Refresh success and failure cases
 * @param workspaceData
 * @param teamID
 * @param workspaceID
 * @param runID
 * @returns
 */
const CreateWorkspaceUpdateForRefreshRelease = (workspaceData, runIDs) => {
    //refreshes can start from waiting or (old) failed Refresh
    const runIDSet = new Set(runIDs);
    const oldStatuses = [Runner.EWorkspaceHealth.refreshing];
    //failedRefresh is deprecated, but we're keeping the value around to support existing environments
    let nextStatus = Runner.EWorkspaceHealth.waiting;
    CheckWorkspaceStatusIsAbleToChange(workspaceData, oldStatuses, Runner.EWorkspaceHealth.waiting);
    let currentlyRunningJobs = [];
    if (workspaceData.currentlyRunningJobs) {
        currentlyRunningJobs = workspaceData.currentlyRunningJobs.filter(runningJobID => !runIDSet.has(runningJobID));
    }
    if (currentlyRunningJobs.length) {
        nextStatus = Runner.EWorkspaceHealth.refreshing;
    }
    return { status: nextStatus, currentlyRunningJobs };
};
/**
 * Configurable heartbeat check-in time to allow modifying the value on the fly
 */
const HEARTBEAT_CHECKIN_TIME = process.env.HEARTBEAT_CHECKIN_TIME ? parseInt(process.env.HEARTBEAT_CHECKIN_TIME) : 10;
const StartHeartbeatInterval = (fieldValue, runDocRef) => {
    const runIntervalID = setInterval(() => {
        return UpdateRunsHeartbeat(fieldValue, runDocRef);
    }, HEARTBEAT_CHECKIN_TIME * 1000);
    return () => {
        // Logger.info("Clearing Heartbeat interval for:", runDocRef)
        clearInterval(runIntervalID);
    };
};
exports.StartHeartbeatInterval = StartHeartbeatInterval;
/**
 * Heartbeat function to be used by Deploy(s)/Refresh(es) to update the heartbeat on an interval during those jobs
 * @param runDocRef
 * @returns
 */
const UpdateRunsHeartbeat = (fieldValue, runDocRef) => {
    return new Promise((resolve, reject) => {
        runDocRef.set({ runHeartbeat: fieldValue.serverTimestamp() }, { merge: true })
            .then(() => resolve())
            .catch(error => reject(error));
    });
};
exports.HEARTBEAT_CHECK_TIME = process.env.HEARTBEAT_CHECK_TIME ? parseInt(process.env.HEARTBEAT_CHECK_TIME) : 15;
const CheckIfRunHeartbeatIsLive = (run) => {
    const timeStamp = run.runHeartbeat;
    if (timeStamp === undefined) {
        return false;
    }
    else {
        const heartbeat = Common.GetDateObjectSafe(timeStamp);
        const comparisonTime = new Date((new Date()).getTime() - exports.HEARTBEAT_CHECK_TIME * 1000);
        return heartbeat >= comparisonTime;
    }
};
/**
 * @param firestore
 * @param orgID
 * @returns
 */
const CheckOrgRunForFailures = (firebase, orgID) => {
    return new Promise((resolve, reject) => {
        const firestore = firebase.firestore();
        const orgRunReference = CommonOperations.getOrgDataDocRefAdmin(firestore, orgID).collection("runs");
        const workspaceCollectionReference = CommonOperations.getOrgDataDocRefAdmin(firestore, orgID).collection("workspaces");
        //get a trimmed number of runs so we don't have to read through all
        const fieldValue = Firebase.GetFieldValueFromFirebaseInstance(firebase);
        const query = orgRunReference.where("runStatus", "==", Runner.ERunStatus.running);
        const transactionPromise = firestore.runTransaction((transaction) => {
            return transaction.get(query)
                .then((querySnapShot) => {
                const workspacesToMarkFailed = {};
                querySnapShot.forEach((run) => {
                    const runData = run.data();
                    //environmentID exists with two different properties depending on the runType
                    const environmentID = runData.runDetails.environmentID;
                    const heartbeatIsFresh = CheckIfRunHeartbeatIsLive(runData);
                    //if bad, do below, otherwise: return/skip next
                    if (!heartbeatIsFresh && (runData.version <= RUN_VERSION_NUMBER || !runData.version)) {
                        let newStatus = Runner.EWorkspaceHealth.waiting;
                        if (runData.runType === Runner.ERunType.deploy) {
                            newStatus = Runner.EWorkspaceHealth.failedDeploy;
                        }
                        if (!workspacesToMarkFailed[environmentID]) {
                            workspacesToMarkFailed[environmentID] = { workspaceID: environmentID, newStatus, runIDs: [] };
                        }
                        workspacesToMarkFailed[environmentID].runIDs.push(parseInt(run.id));
                    }
                });
                const workspaceDataPromises = Object.keys(workspacesToMarkFailed).map(workspaceID => transaction.get(workspaceCollectionReference.doc(workspaceID)));
                //read all workspaceData to make sure we mark the correct currentlyRunningJobs
                return Promise.all(workspaceDataPromises) //last reads
                    .then((workspaceDataArray) => {
                    const allWorkspacesData = workspaceDataArray.map(doc => doc.data());
                    //writes must follow reads
                    allWorkspacesData.forEach(workspaceData => {
                        const workspaceUpdateInfo = workspacesToMarkFailed[workspaceData.id];
                        const { runIDs, newStatus, workspaceID } = workspaceUpdateInfo;
                        //replace below with either update for failed (refresh) or failedDeploy
                        let dataForWorkspaceDocUpdate;
                        if (newStatus === Runner.EWorkspaceHealth.failedDeploy) {
                            dataForWorkspaceDocUpdate = CreateWorkspaceUpdateForFailedDeployRelease(workspaceData, runIDs);
                        }
                        else {
                            dataForWorkspaceDocUpdate = CreateWorkspaceUpdateForRefreshRelease(workspaceData, runIDs);
                        }
                        runIDs.forEach(runID => {
                            Logger.info(`Marking run ${runID} failed for bad check-in in ${orgID} for workspace ${workspaceID}`);
                            UpdateTransactionWrite(transaction, orgRunReference.doc(`${runID}`), { endTime: fieldValue.serverTimestamp(), runStatus: Runner.ERunStatus.failed });
                        });
                        UpdateTransactionWrite(transaction, workspaceCollectionReference.doc(workspaceID), dataForWorkspaceDocUpdate);
                    });
                });
            });
        });
        try {
            transactionPromise
                .then(() => {
                resolve();
            }).catch(error => {
                Logger.errorContext({}, "PRIMARY_SCHEDULER: Error during Heartbeat check/cleanup: ", error);
                reject(error);
            });
        }
        catch (error) {
            Logger.errorContext({}, "PRIMARY_SCHEDULER: Error during Heartbeat check/cleanup: ", error);
            reject(error);
        }
    });
};
const CleanupRunsOnStartUp = (firebase) => {
    return new Promise((resolve, reject) => {
        Logger.info("cleanup start", (new Date()).getTime());
        const orgDocReference = CommonOperations.getOrgsCollectionAdmin(firebase.firestore());
        return orgDocReference.where("runCounter", ">=", 1).get()
            .then((querySnapshot) => {
            const allRunUpdatesPromises = [];
            querySnapshot.forEach((orgDoc) => {
                Logger.debug(`checking org ${orgDoc.id}`);
                //creates an array of transactions (1 per org where we run this work)
                allRunUpdatesPromises.push(CheckOrgRunForFailures(firebase, orgDoc.id));
            });
            return Promise.all(allRunUpdatesPromises);
        }).then(() => {
            resolve();
        }).catch(error => {
            Logger.error("PRIMARY_SCHEDULER: Error during Run(s) Cleanup routine: ", error);
            reject(error);
        });
    });
};
exports.CleanupRunsOnStartUp = CleanupRunsOnStartUp;
//# sourceMappingURL=RunTransaction.js.map