"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UpdateCoalesceFirebaseUser = exports.CheckIsValidAdminUpdate = exports.BECLI_HandleRefresh = exports.BECLI_HandleDeploy = exports.RunSQLWithoutScheduler = exports.FinishSnowflakeQueryWithSuccess = exports.isValidRequest = exports.LogRequestInfo = exports.CleanObjectForLogging = exports.AuthenticateFirebaseTokenAndRetrieveTeamInfoForCLI = exports.AuthenticateFirebaseTokenAndRetrieveTeamInfoForScheduler = exports.FinishWithError = exports.FirebaseExport = exports.EErrorMessages = exports.RunnerBackendLogger = void 0;
const admin = __importStar(require("firebase-admin"));
const Logging = __importStar(require("./Logging"));
const Firebase = __importStar(require("./Firebase"));
const Runner = __importStar(require("./Runner"));
const Common = __importStar(require("./Common"));
const Templates = __importStar(require("./Templates"));
const ConnectionOperations = __importStar(require("./ConnectionOperations"));
const CommonOperations = __importStar(require("./CommonOperations"));
const TeamOperations = __importStar(require("./TeamOperations"));
const Runs = __importStar(require("./Runs"));
const Workspaces = __importStar(require("./Workspaces"));
const RunTransaction = __importStar(require("./RunTransaction"));
const AuthLogger = Logging.GetLogger(Logging.LoggingArea.Auth);
let uniqueNameCounter = 0;
exports.RunnerBackendLogger = Logging.GetLogger(Logging.LoggingArea.RunnerBackend);
var EErrorMessages;
(function (EErrorMessages) {
    EErrorMessages["clientSecretError"] = "Cannot read property 'currentSecretID' of undefined";
})(EErrorMessages = exports.EErrorMessages || (exports.EErrorMessages = {}));
let FirebaseExportTemp;
/**
 * added isCommandLine, since this code is in Shared now, it means that it gets bundled and put in the frontend
 * which was not being accounted for and was causing weird crashes whenever changes to Shared files occurred and * were saved
 */
if (Common.isCommandLine() && Common.IsCloudScheduler()) {
    admin.initializeApp({
        credential: admin.credential.applicationDefault(),
    });
    FirebaseExportTemp = admin;
}
else {
    FirebaseExportTemp = Firebase.justFirebase; //typing is mad and can't resolve...
}
exports.FirebaseExport = FirebaseExportTemp;
const FinishWithError = (res, error) => {
    var _a, _b;
    let statusCode = 500;
    if (error.statusCode) {
        statusCode = error.statusCode;
    }
    exports.RunnerBackendLogger.error("FinishWithError", error);
    const payload = {
        error: {
            errorString: error.message || ((_a = error.error) === null || _a === void 0 ? void 0 : _a.errorString),
            errorDetail: error.stack || ((_b = error.error) === null || _b === void 0 ? void 0 : _b.errorDetail)
        }
    };
    res.status(statusCode).json(payload);
};
exports.FinishWithError = FinishWithError;
/**
 * Helper function to abstract the creation and return of Firebase instances
 * @param idToken
 * @returns
 */
const GetFirebaseForScheduler = (idToken) => {
    return new Promise((resolveFirebase, rejectError) => {
        return exports.FirebaseExport.auth().verifyIdToken(idToken)
            .then((decodedToken) => {
            resolveFirebase({ Firebase: exports.FirebaseExport, uid: decodedToken.uid });
        }).catch(error => {
            rejectError(error);
        });
    });
};
/**
 *
 * @param idToken
 * @returns
 */
const SetupFirebaseForCLI = (idToken) => {
    return new Promise((resolveFirebase, rejectError) => {
        const uniqueName = idToken + uniqueNameCounter++;
        const newFirebase = Firebase.justFirebase.initializeApp(Firebase.getFirebaseConfig(), uniqueName);
        let uid;
        return Runner.RunPost(`${CommonOperations.GetCloudSchedulerAddress()}/scheduler/generateLoginToken?token=${idToken}`, null)
            .then(response => {
            const customToken = response.data.customToken;
            uid = response.data.uid;
            return newFirebase.auth().signInWithCustomToken(customToken);
        }).then(() => {
            resolveFirebase({ Firebase: newFirebase, uid });
        }).catch((error) => {
            exports.RunnerBackendLogger.error("GetOrSetupFirebase", error);
            rejectError(error);
        });
    });
};
const CallGoogleTokenAuthEndpoint = (token) => {
    return Runner.RunPost(`https://securetoken.googleapis.com/v1/token?key=${Firebase.getFirebaseConfig().apiKey}`, { refresh_token: token, grant_type: "refresh_token" }).then((response) => {
        return response.data.id_token;
    });
};
const AuthenticateFirebaseTokenAndRetrieveTeamInfo = (refresh_token, id_token, FirebaseSetupFunction) => {
    let idToken;
    let firebase;
    let teamInfoStored;
    AuthLogger.debug("got auth request refresh:", refresh_token, "id token", id_token);
    return (!(id_token === undefined) ? Promise.resolve(id_token) : CallGoogleTokenAuthEndpoint(refresh_token))
        .then((id_token) => {
        idToken = id_token;
        return FirebaseSetupFunction(idToken);
    })
        .then((data) => {
        firebase = data.Firebase;
        return TeamOperations.GetTeamDataAdmin(firebase.firestore(), data.uid);
    })
        .then((teamInfo) => {
        teamInfoStored = teamInfo;
        const UserRolePrivilegeChecker = GetPrivilegesBasedOnUserRole(teamInfo.fbRole);
        return { teamInfo, firebase, idToken, UserRolePrivilegeChecker };
    })
        .catch(error => {
        const realError = new Error("Unable to authenticate, please ensure your token is valid");
        realError.name = "401";
        const userID = !!teamInfoStored ? teamInfoStored.fbUserID : "";
        const orgID = !!teamInfoStored ? teamInfoStored.fbTeamID : "";
        AuthLogger.errorContext({ userID, orgID }, "AuthenticateFirebaseTokenAndRetrieveTeamInfo", error);
        throw realError;
    });
};
const AuthenticateFirebaseTokenAndRetrieveTeamInfoForScheduler = (refresh_token, id_token) => {
    return AuthenticateFirebaseTokenAndRetrieveTeamInfo(refresh_token, id_token, GetFirebaseForScheduler);
};
exports.AuthenticateFirebaseTokenAndRetrieveTeamInfoForScheduler = AuthenticateFirebaseTokenAndRetrieveTeamInfoForScheduler;
const AuthenticateFirebaseTokenAndRetrieveTeamInfoForCLI = (refresh_token, id_token) => {
    return AuthenticateFirebaseTokenAndRetrieveTeamInfo(refresh_token, id_token, SetupFirebaseForCLI);
};
exports.AuthenticateFirebaseTokenAndRetrieveTeamInfoForCLI = AuthenticateFirebaseTokenAndRetrieveTeamInfoForCLI;
const RequestLoggingBlacklist = new Set([
    "password",
    "code",
    "codeVerifier",
    "accessToken",
    "token",
    "id_token",
    "clientSecret",
    "authCode",
    "snowflakePassword",
    "newPassword",
    "refresh_token",
    "uid",
    "keyPair"
]);
const CleanObjectForLogging = (obj) => {
    RequestLoggingBlacklist.forEach((val) => {
        delete obj[val];
    });
    Object.keys(obj).forEach((key) => {
        if (!!obj[key] && typeof obj[key] === "object" && !!Object.keys(obj[key]).length) {
            (0, exports.CleanObjectForLogging)(obj[key]);
        }
    });
    return obj;
};
exports.CleanObjectForLogging = CleanObjectForLogging;
const LogRequestInfo = (logContext, message, req) => {
    const reqItems = {
        body: Common.CreateDeepClone(req.body),
        query: Common.CreateDeepClone(req.query),
        params: Common.CreateDeepClone(req.params)
    };
    const cleanedRequest = (0, exports.CleanObjectForLogging)(reqItems);
    exports.RunnerBackendLogger.infoContext(logContext, message, cleanedRequest);
};
exports.LogRequestInfo = LogRequestInfo;
const isValidRequest = (req, res, queryType, bodyType) => {
    if (queryType) {
        const queryValidationResult = Templates.validateRunType(queryType, req.query);
        if (!queryValidationResult.success) {
            (0, exports.FinishWithError)(res, new Error(`failed to validate query params:${queryValidationResult.errorString}`));
            return false;
        }
    }
    if (bodyType) {
        const bodyValidationResult = Templates.validateRunType(bodyType, req.body);
        if (!bodyValidationResult.success) {
            (0, exports.FinishWithError)(res, new Error(`failed to validate request body:${bodyValidationResult.errorString}`));
            return false;
        }
    }
    return true;
};
exports.isValidRequest = isValidRequest;
/**
 * CreateQueryResponse creates a TemplateRun from the query response from Snowflake.
 * @param result
 * @param templateString
 * @returns
 */
const CreateQueryResponseObject = (result, templateString) => {
    var _a;
    /**
     * result.stmt comes from Snowflake. We don't have a type for this.
     * We are mapping the columns from the response object
     */
    const columns = (_a = result === null || result === void 0 ? void 0 : result.stmt) === null || _a === void 0 ? void 0 : _a.getColumns().map(col => {
        return {
            Name: col.getName(),
            DataType: col.getType()
        };
    });
    const apiResponse = {
        runDetails: {
            compiledSQL: templateString,
            exported_refs: [],
            invalid_exported_refs: [],
            ref_usage: {
                invalid_ref: [],
                invalid_ref_link: [],
                invalid_ref_no_link: [],
                ref: [],
                ref_link: [],
                ref_no_link: []
            }
        },
        data: {
            columns,
            rows: result.rows,
            status: result.status
        },
    };
    return apiResponse;
};
/**
 * function in charge of sending the response on a successful snowflake query
 * @param req
 * @param res
 * @param result
 */
const FinishSnowflakeQueryWithSuccess = (req, res, result) => {
    const apiResponse = CreateQueryResponseObject(result, req.body.templateString);
    res.status(200).send(apiResponse);
};
exports.FinishSnowflakeQueryWithSuccess = FinishSnowflakeQueryWithSuccess;
const RunSQLWithoutScheduler = (requestObject, connectionCache, teamInfoAndFirebase) => {
    return new Promise((resolve, reject) => {
        const { teamInfo } = teamInfoAndFirebase;
        const queryObj = { sqlText: requestObject.templateString };
        Common.assertWithContext(Logging.LoggingArea.RunnerBackend, !!requestObject.userContext, { orgID: teamInfo.fbTeamID, userID: teamInfo.fbUserID }, "Missing user context in (SnowflakeRunHandlerFunction)");
        const userContext = requestObject.userContext;
        const logContext = {
            orgID: teamInfo.fbTeamID,
            userID: teamInfo.fbUserID,
            envID: userContext.environmentID
        };
        const logStatement = `Back-end: ${requestObject.templateString}`;
        return Common.timedLoggerPromise(logStatement, logContext, Logging.LoggingArea.Perf, new Promise((timedResolve, timedReject) => {
            return connectionCache.GetConnectionAndRunQuery(queryObj, userContext, teamInfoAndFirebase)
                .then((result) => {
                timedResolve(result);
            })
                .catch((err) => {
                timedReject(err);
            });
        }))
            .then((result) => {
            const formattedResponse = CreateQueryResponseObject(result, requestObject.templateString);
            resolve(formattedResponse);
        })
            .catch((err) => {
            // if check is a temporary fix until CD-1966 is resolved. Once resolved, the below if block should be removed
            let error = err;
            if (err.message === EErrorMessages.clientSecretError) {
                error = new Error("Missing password or client secret - please update your environment settings");
            }
            reject(error);
        });
    });
};
exports.RunSQLWithoutScheduler = RunSQLWithoutScheduler;
/**
 * Standardized way of kicking off a deploy, whether from the CLI or Scheduler
 * @param runInfo
 * @param teamInfoAndFirebase
 * @param token
 * @param RunSQL
 * @returns
 */
const BECLI_HandleDeploy = (runInfo, teamInfoAndFirebase, RunSQL, connection) => {
    var _a, _b, _c;
    const { teamInfo, teamInfo: { fbTeamID: teamID, fbUserID: userID }, firebase } = teamInfoAndFirebase;
    const firestore = firebase.firestore();
    const environmentID = (_a = runInfo.runDetails) === null || _a === void 0 ? void 0 : _a.toWorkspaceID;
    const newRunInfo = Object.assign(Object.assign({}, runInfo), { runDetails: ConnectionOperations.AddUserConnectionAndAccountToDeployRunInfo(connection, (_b = runInfo.userCredentials) === null || _b === void 0 ? void 0 : _b.snowflakeAccount, runInfo.runDetails, userID) });
    let statusSecured = false;
    runInfo.runStatus = Runner.ERunStatus.running;
    runInfo.runType = Runner.ERunType.deploy;
    runInfo.userID = userID;
    const logContext = Logging.CreateLogContext(teamID, environmentID, userID);
    exports.RunnerBackendLogger.infoContext(logContext, "Deployment started");
    Common.assert(Logging.LoggingArea.Deploy, ConnectionOperations.isValidUserConnectionConfiguration(connection, newRunInfo), `invalid user connection for deploy: 
		${JSON.stringify((0, exports.CleanObjectForLogging)(Common.CreateDeepClone(connection)))} 
		${JSON.stringify((0, exports.CleanObjectForLogging)(Common.CreateDeepClone(runInfo)))}
		`);
    const userContext = {
        accountName: ((_c = newRunInfo.userCredentials) === null || _c === void 0 ? void 0 : _c.snowflakeAccount) || "",
        user: ConnectionOperations.GetValueFromConnectionDetailsSafe(connection, ConnectionOperations.EUserConnectionDetail.user),
        role: ConnectionOperations.GetValueFromConnectionDetailsSafe(connection, ConnectionOperations.EUserConnectionDetail.role),
        warehouse: ConnectionOperations.GetValueFromConnectionDetailsSafe(connection, ConnectionOperations.EUserConnectionDetail.warehouse),
        environmentID: environmentID,
        orgID: teamID,
        password: newRunInfo.userCredentials.snowflakePassword || "",
        authenticator: connection.connectionType,
        keyPairKey: Common.getValueSafe(connection, ["connectionDetails", "keyPair"], null),
    };
    const userContextLessPassword = Object.assign({ password: null }, userContext);
    exports.RunnerBackendLogger.info("userContext: ", JSON.stringify(userContextLessPassword));
    const templateParams = {
        userContext: userContext,
        templateString: Templates.TemplateString.SELECT_ONE_SQL,
        macrosString: "",
        nodeMetadata: { operationName: "", mapping: {} },
    };
    return new Promise((resolve, reject) => {
        exports.RunnerBackendLogger.infoContext(logContext, "testing snowflake connection:", connection);
        return RunSQL(templateParams)
            .then(() => {
            exports.RunnerBackendLogger.infoContext(logContext, "Snowflake connection test successful");
            return RunTransaction.SecureWorkspaceAndStartDeploy(firebase, teamInfo.fbTeamID, runInfo, environmentID);
        })
            .then(({ runCounter, CleanupHeartbeat }) => {
            exports.RunnerBackendLogger.infoContext(logContext, "Run counter in start deploy:", runCounter);
            /**
             * This sub-chain ties the creation and cleanup of Firebase together at the same level in order to
             * route to the correct catch statements and prevent it from being cleaned up before the promise is
             * resolved.
             */
            const runDeployCompletion = Runs.runDeploy(firebase, teamInfo, runCounter, newRunInfo, RunSQL)
                .then(() => {
                exports.RunnerBackendLogger.infoContext(logContext, "runDeploy finished successfully");
                return RunTransaction.ReleaseWorkspaceAndSucceedDeploy(firebase, teamInfo.fbTeamID, environmentID, runCounter);
            })
                .catch((error) => {
                exports.RunnerBackendLogger.errorContext(logContext, "Error during deploy inner promise (runDeployCompletion):", error);
                return RunTransaction.ReleaseWorkspaceAndFailDeploy(firebase, teamInfo.fbTeamID, environmentID, runCounter)
                    .catch(error => {
                    exports.RunnerBackendLogger.errorContext(logContext, "Error during status release: ", error);
                })
                    .then(() => {
                    // AI: Must throw here so that the level above knows there was an error; otherwise chain will succeed falsely
                    throw error;
                });
            })
                .finally(() => {
                CleanupHeartbeat();
            });
            resolve({
                runCounter,
                runCompletion: runDeployCompletion
            });
        })
            .catch((error) => {
            exports.RunnerBackendLogger.errorContext(logContext, "Error during deploy: ", error);
            reject(error);
            if (statusSecured) {
                return Workspaces.MarkWorkspaceStatusFailedWithCatch(firebase.firestore(), teamID, environmentID, Runner.EWorkspaceHealth.deploying, Runner.EWorkspaceHealth.failedDeploy, logContext);
            }
        });
    });
};
exports.BECLI_HandleDeploy = BECLI_HandleDeploy;
const RunPreflightCheckOnRunInfo = (runInfo) => {
    const { runDetails } = runInfo;
    if (runDetails.jobID && (runDetails.includeNodesSelector || runDetails.excludeNodesSelector)) {
        throw new Error("Unable to process, don't provide a JobID and Node Include/Exclude Selector(s) for 1 Refresh Run");
    }
};
/**
 * Standardized way to kick off a Coalesce refresh, from the CLI or Scheduler
 * @param runInfo
 * @param teamInfoAndFirebase
 * @param token
 * @param RunSQL
 * @returns
 */
const BECLI_HandleRefresh = (runInfo, teamInfoAndFirebase, RunSQL, connection) => {
    var _a;
    const { teamInfo, teamInfo: { fbTeamID: teamID, fbUserID: userID }, firebase, } = teamInfoAndFirebase;
    const firestore = firebase.firestore();
    const environmentID = parseInt((_a = runInfo.runDetails) === null || _a === void 0 ? void 0 : _a.environmentID);
    let jobPromise;
    runInfo.runStatus = Runner.ERunStatus.running;
    runInfo.runType = Runner.ERunType.refresh;
    runInfo.userID = userID;
    const logContext = Logging.CreateLogContext(teamID, environmentID, userID);
    exports.RunnerBackendLogger.infoContext(logContext, "Refresh started");
    Common.assert(Logging.LoggingArea.Deploy, ConnectionOperations.isValidUserConnectionConfiguration(connection, runInfo), `invalid user connection for refresh: 
		${JSON.stringify((0, exports.CleanObjectForLogging)(Common.CreateDeepClone(connection)))} 
		${JSON.stringify((0, exports.CleanObjectForLogging)(Common.CreateDeepClone(runInfo)))}
		`);
    try {
        RunPreflightCheckOnRunInfo(runInfo);
    }
    catch (error) {
        return Promise.reject(error);
    }
    const jobID = runInfo.runDetails.jobID;
    if (!jobID) {
        exports.RunnerBackendLogger.infoContext(logContext, "No job ID, refreshing all steps");
        jobPromise = Promise.resolve(true);
    }
    else {
        exports.RunnerBackendLogger.infoContext(logContext, "Refreshing job:", jobID);
        jobPromise = CommonOperations.DocumentExistsAdmin(firestore, teamID, environmentID, "jobs", jobID);
    }
    return new Promise((resolve, reject) => {
        return ConnectionOperations.GetAccountFromEnvironment(firestore, teamID, environmentID)
            .then((accountName) => {
            const userContext = {
                accountName,
                user: ConnectionOperations.GetValueFromConnectionDetailsSafe(connection, ConnectionOperations.EUserConnectionDetail.user),
                role: ConnectionOperations.GetValueFromConnectionDetailsSafe(connection, ConnectionOperations.EUserConnectionDetail.role),
                warehouse: ConnectionOperations.GetValueFromConnectionDetailsSafe(connection, ConnectionOperations.EUserConnectionDetail.warehouse),
                environmentID: environmentID,
                orgID: teamID,
                password: runInfo.userCredentials.snowflakePassword || "",
                authenticator: connection.connectionType,
                keyPairKey: Common.getValueSafe(connection, ["connectionDetails", "keyPair"], null),
            };
            const userContextLessPassword = Object.assign({ password: null }, userContext);
            exports.RunnerBackendLogger.infoContext(logContext, "userContext in HandleRefresh: ", JSON.stringify(userContextLessPassword));
            const templateParams = {
                userContext: userContext,
                templateString: Templates.TemplateString.SELECT_ONE_SQL,
                macrosString: "",
                nodeMetadata: { operationName: "", mapping: {} }
            };
            exports.RunnerBackendLogger.infoContext(logContext, "testing snowflake connection:", connection);
            return RunSQL(templateParams);
        })
            .then(() => {
            exports.RunnerBackendLogger.infoContext(logContext, "snowflake connection test success");
            return jobPromise;
        })
            .then((jobExists) => {
            if (!jobExists) {
                throw Common.CreateErrorWithCode(400, `Unable to start job ${jobID}`, `Job ${jobID} did not exist`);
            }
            /**
             * If the workspace is already Deploying, Refreshing or in Failed Deploy state, this will fail
             * which is desired/expected
             */
            return RunTransaction.SecureWorkspaceAndStartRefresh(firebase, teamInfo.fbTeamID, runInfo, environmentID);
        })
            .then(({ runCounter, CleanupHeartbeat }) => {
            exports.RunnerBackendLogger.infoContext(logContext, "Run counter in start refresh:", runCounter);
            /**
             * This sub-chain ties the creation and cleanup of Firebase together at the same level in order to
             * route to the correct catch statements and prevent it from being cleaned up before the promise is
             * resolved.
             */
            const runRefreshCompletion = Runs.runRefresh(firebase, teamInfo, runCounter, runInfo, environmentID.toString(), RunSQL, runInfo.runTimeParameters, connection)
                .then(() => {
                exports.RunnerBackendLogger.infoContext(logContext, "runRefresh finished with no errors");
                return RunTransaction.ReleaseWorkspaceAndSucceedRefresh(firebase, teamInfo.fbTeamID, environmentID, runCounter);
            })
                .catch((error) => {
                exports.RunnerBackendLogger.errorContext(logContext, "Error during refresh:", error);
                return RunTransaction.ReleaseWorkspaceAndFailRefresh(firebase, teamInfo.fbTeamID, environmentID, runCounter)
                    .catch(error => {
                    exports.RunnerBackendLogger.errorContext(logContext, "Error during status release: ", error);
                })
                    .then(() => {
                    // AI: Must throw here so that the level above knows there was an error; otherwise chain will succeed falsely
                    throw error;
                });
            })
                .finally(() => {
                CleanupHeartbeat();
            });
            resolve({
                runCounter,
                runCompletion: runRefreshCompletion,
            });
            return runRefreshCompletion;
        })
            .catch((error) => {
            exports.RunnerBackendLogger.errorContext(logContext, "Error during refresh:", error);
            reject(error);
        });
    });
};
exports.BECLI_HandleRefresh = BECLI_HandleRefresh;
var EPrivilegesAvailableToRoles;
(function (EPrivilegesAvailableToRoles) {
    EPrivilegesAvailableToRoles["canUpdateUser"] = "canUpdateUser";
    EPrivilegesAvailableToRoles["canUpdateSuperUser"] = "canUpdateSuperUser";
})(EPrivilegesAvailableToRoles || (EPrivilegesAvailableToRoles = {}));
/**
 * Takes in a role from TeamOperations.EUserRoles, and returns a function memoized to that role
 * @param role taken from TeamOperations.EUserRoles
 * @returns a function that takes in a privilege, which then returns whether or not the originally passed
 * in role has permission to perform a given action
 */
const GetPrivilegesBasedOnUserRole = (role) => {
    const privileges = {
        [EPrivilegesAvailableToRoles.canUpdateUser]: [TeamOperations.EUserRoles.SUPER_USER, TeamOperations.EUserRoles.ADMIN],
        [EPrivilegesAvailableToRoles.canUpdateSuperUser]: [ /* SHOULD BE EMPTY */]
    };
    return (privilege) => {
        return privileges[privilege].includes(role);
    };
};
/**
 * A simple packager function is in CheckIsValidAdminUpdate
 * @param isValidBool
 * @param errorMessage
 * @returns an object with an isValid boolean, and an error object (which should be null if isValid is true)
 */
const CheckRoleResult = (isValidBool, errorMessage) => {
    return {
        isValid: isValidBool,
        error: {
            error: {
                errorDetail: errorMessage,
                errorString: errorMessage
            }
        }
    };
};
/**
 * A function for checking that an edit is valid. This includes both permissions, and making sure that the edit is complete
 * @param sendingUsersInfo user making the changes
 * @param receivingUsersInfo user being edited
 * @param HasPrivilegeForAction is a function used for checking if the user has permission for the current action based on their user role
 * @returns an object created by CheckRoleResult stating whether the user has permission to make their current action, and an error object
 * 	if they do not
 */
const CheckIsValidAdminUpdate = (sendingUsersInfo, receivingUsersInfo, HasPrivilegeForAction) => {
    if (!receivingUsersInfo.role || !receivingUsersInfo.firstName || !receivingUsersInfo.lastName) {
        return CheckRoleResult(false, "One or more fields were not correctly filled out");
    }
    else if (receivingUsersInfo.role === TeamOperations.EUserRoles.SUPER_USER && !HasPrivilegeForAction(EPrivilegesAvailableToRoles.canUpdateSuperUser)) {
        return CheckRoleResult(false, "Error: Admins cannot update super users");
    }
    else if (!HasPrivilegeForAction(EPrivilegesAvailableToRoles.canUpdateUser)) {
        return CheckRoleResult(false, "Error: Insufficient privileges to update users");
    }
    return CheckRoleResult(true, "");
};
exports.CheckIsValidAdminUpdate = CheckIsValidAdminUpdate;
/**
 * simple checker to make sure all the edit fields are filled out
 * @param userInfo
 * @returns a boolean on whether the edit fields are filled out
 */
const CheckIsValidUserInfoForEdit = (userInfo) => {
    const { firstName, lastName, role } = userInfo;
    return !!firstName && !!lastName && !!role;
};
/**
 * A firebase updating function with two paths. The first path  (if there is no userID), creates the user using set,
 * and the other updates the user using update
 * @param userInfo
 * @param firebase
 * @returns
 */
const UpdateCoalesceFirebaseUser = (userInfo, firebase) => {
    return new Promise((resolve, reject) => {
        const { currentTeam, role, firstName, lastName, email, password, userID } = userInfo;
        const firestore = firebase.firestore();
        if (!userID) {
            return firebase.auth().createUser({ email, password })
                .then((user) => {
                CommonOperations.getUserDocRefAdmin(firestore, user.uid).set({
                    email: user.email,
                    currentTeam,
                    role,
                    firstName,
                    lastName
                });
                resolve(user);
            })
                .catch(err => {
                reject(err);
            });
        }
        else {
            Common.assert(Logging.LoggingArea.Auth, CheckIsValidUserInfoForEdit(userInfo), "Error: Invalid User Info (UpdateCoalesceFirebaseUser)");
            return CommonOperations.getUserDocRefAdmin(firestore, userID).update(userInfo)
                .then(result => {
                resolve(result);
            })
                .catch(err => {
                reject(err);
            });
        }
    });
};
exports.UpdateCoalesceFirebaseUser = UpdateCoalesceFirebaseUser;
//# sourceMappingURL=SchedulerOperations.js.map