"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetNodesForRefreshJobFromSearchResult = exports.getSearchResult = void 0;
const Common = __importStar(require("./Common"));
const Logging = __importStar(require("./Logging"));
const Minimatch = __importStar(require("minimatch"));
const StepOperations = __importStar(require("./StepOperations"));
const GraphOperations = __importStar(require("./GraphOperations"));
const L = Logging.GetLogger(Logging.LoggingArea.Search);
const searchRegex = new RegExp(/(?<prefix>[\+]?)(?<tableName>(?:[^\+\ \r\n\[/\\]|\\.|\[(?:[^\r\n\]\\]|\\.)*\])+)(?<postfix>[\+](?![\+]))?\s*/g);
const searchWithRegex = (searchStringToUseParam, defaultSearchString, steps, gFinal) => {
    let stepsDict = {};
    //if undefined, use empty search string
    let searchStringToUse = searchStringToUseParam;
    if (!searchStringToUse) {
        searchStringToUse = defaultSearchString;
    }
    let unions = searchStringToUse.split(/\s+/).filter(s => s);
    //if a bunch of spaces, also use defaultSearchString
    if (!unions.length) {
        unions = [defaultSearchString];
    }
    L.debug("got unions", unions, "searchSTringTouse", searchStringToUse, "searchRegex", searchRegex, "defaultSearchString", defaultSearchString, "steps", steps);
    unions.forEach((u) => {
        const intersections = u.split(/,/);
        //intersections we gather a set of steps for each string - then compute final steps set
        let stepsSets = [];
        intersections.forEach((i) => {
            let intersectionStepSet = {};
            L.debug("got intersection", i);
            //find matches
            const matches = [...i.matchAll(searchRegex)];
            L.debug("got matches", i, matches);
            let matchesArray = matches.map((r) => {
                Common.assert(Logging.LoggingArea.Search, r.length == 4, "mismatched number of results");
                return { prefix: r[1], tableName: r[2], postfix: r[3] };
            });
            L.debug("got groups ", i, matchesArray);
            //matches based on name 
            const stepMatchRegex = (step, tableName) => {
                return Minimatch.match([step.operation.name], tableName).length;
            };
            //find steps that match
            matchesArray.forEach((m) => {
                Object.keys(steps).forEach((s) => {
                    if (!m) {
                        L.emerg("got an unexpected undefined value for match", searchStringToUse, matchesArray, i);
                        return;
                    }
                    if (stepMatchRegex(steps[s], m.tableName)) {
                        L.debug(`matched ${s} ${steps[s].operation.name} with ${m.tableName}`);
                        intersectionStepSet[s] = true;
                        if (m.prefix) {
                            const predecessors = StepOperations.getPredecessorsWithLevels(gFinal, s);
                            L.debug(`add predecessors${s} ${steps[s].operation.name} with ${m.tableName} predecessors:${predecessors}`);
                            Object.values(predecessors).forEach(setOfSteps => [...setOfSteps.keys()].forEach(step => intersectionStepSet[step] = true));
                        }
                        if (m.postfix) {
                            const successors = StepOperations.getSuccessorsWithLevels(gFinal, s);
                            L.debug(`add successors${s} ${steps[s].operation.name} with ${m.tableName} successsors${successors}`);
                            Object.values(successors).forEach(setOfSteps => [...setOfSteps.keys()].forEach(step => intersectionStepSet[step] = true));
                        }
                    }
                });
                stepsSets.push(intersectionStepSet);
            });
        });
        const finalStepsSet = stepsSets.reduce((prev, curr) => {
            return Object.fromEntries(Common.objectKeyIntersection(prev, curr).map((s) => {
                return [s, true];
            }));
        }, Object.assign({}, steps));
        stepsDict = Object.assign(Object.assign({}, stepsDict), finalStepsSet);
    });
    return stepsDict;
};
const getSearchResult = (includeSearchString, excludeSearchString, steps, gFinal) => {
    const includeStepsDict = searchWithRegex(`${includeSearchString}`, "*", steps, gFinal);
    const excludeStepsDict = searchWithRegex(excludeSearchString, "!*", steps, gFinal);
    let stepsResult = includeStepsDict;
    Object.keys(excludeStepsDict).forEach((s) => {
        delete stepsResult[s];
    });
    return {
        stepsResult: stepsResult
    };
};
exports.getSearchResult = getSearchResult;
const GetNodesForRefreshJobFromSearchResult = (includeSearchString, excludeSearchString, steps) => {
    const gFinal = GraphOperations.CalculateGFinalWithoutSizes(steps);
    const searchResult = (0, exports.getSearchResult)(includeSearchString, excludeSearchString, steps, gFinal);
    return Object.keys(searchResult.stepsResult);
};
exports.GetNodesForRefreshJobFromSearchResult = GetNodesForRefreshJobFromSearchResult;
//# sourceMappingURL=Search.js.map