"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RunSnowflakeQuery = exports.RoughSizeOfObject = exports.CheckForDeadConnectionErrorString = exports.CheckForDeadAccessToken = exports.deadAccessCodeErrorString = exports.ConnectionStorageClass = exports.IsValidConnection = exports.waitForValidConnectionHasConnection = exports.createInjectionWaiting = exports.createInjection = exports.ESnowflakeDataReturnStatus = exports.runGetRows = exports.runListTables = exports.runTestConnection = exports.runSQLStringWithTemplateParams = exports.salt = exports.CryptoJS = exports.v8 = void 0;
const Runner = __importStar(require("./Runner"));
const Common = __importStar(require("./Common"));
const Templates = __importStar(require("./Templates"));
const Logging = __importStar(require("./Logging"));
const MacroOperations = __importStar(require("./MacroOperations"));
const RunnerOperations = __importStar(require("./RunnerOperations"));
const Secrets = __importStar(require("./Secrets"));
const ConnectionOperations = __importStar(require("./ConnectionOperations"));
const CommonOperations = __importStar(require("./CommonOperations"));
const qs_1 = __importDefault(require("qs"));
let snowflakeSDK;
if (Common.isCommandLine()) {
    //dynamic import to satisfy frontend stuff
    snowflakeSDK = require("snowflake-sdk");
}
/**
 * exported to be used and reset to the real Node V8 by the scheduler/cli
 * since the webpack bundler/compiler couldn't resolve the module for the browser
 * v interesting, but not important now. just know that you can't use Node based built ins in the browser
 * since that isn't a node env)
 */
exports.v8 = null;
exports.CryptoJS = null;
exports.salt = null;
//helper for turning a snowflake error message  to a QueryResult with an error in it 
const ConnectionLogger = Logging.GetLogger(Logging.LoggingArea.Connection);
const RunnerBackendLogger = Logging.GetLogger(Logging.LoggingArea.RunnerBackend);
const handleSnowflakeError = (ret) => {
    return {
        success: false, error: Object.assign({ message: ret.errorMessage }, ret.error), rows: [], fields: []
    };
};
const runSQLStringWithTemplateParams = (userContext, templateString, macros, physicalLocations, token) => {
    return new Promise((resolve, reject) => {
        const templateParams = {
            userContext: userContext,
            templateString: templateString,
            nodeMetadata: { operationName: "", mapping: physicalLocations },
            macrosString: MacroOperations.getAllMacros(macros)
        };
        Runner.RunTemplate(templateParams, token).then(resolve).catch(reject);
    });
};
exports.runSQLStringWithTemplateParams = runSQLStringWithTemplateParams;
const runTestConnection = (userContext, token) => {
    ConnectionLogger.infoContext({
        envID: userContext.environmentID,
        orgID: userContext.orgID
    }, "(runTestConnection) Testing connection: ", { user: Logging.MaskLogValue(userContext.user), accountName: Logging.MaskLogValue(userContext.accountName) });
    return new Promise((resolve, reject) => {
        return (0, exports.runSQLStringWithTemplateParams)(userContext, Templates.TemplateString.SELECT_ONE_SQL, MacroOperations.refMacroDefinitions, {}, token).then(resolve).catch(reject);
    });
};
exports.runTestConnection = runTestConnection;
const runListTables = (userContext, physicalLocations) => {
    return new Promise((resolve, reject) => {
        let tableList = {};
        const runSQLPromises = Object.keys(physicalLocations).map(key => {
            const database = physicalLocations[key].database;
            const schema = physicalLocations[key].schema;
            const sqlToExecute = `SHOW OBJECTS IN ${database}.${schema}`;
            return (0, exports.runSQLStringWithTemplateParams)(userContext, sqlToExecute, MacroOperations.refMacroDefinitions, {}, null);
        });
        Promise.all(runSQLPromises).then((results) => {
            results.forEach(result => {
                result.data.rows.forEach((row) => {
                    const database = row.database_name;
                    const schema = row.schema_name;
                    const table = row.name;
                    if (!tableList[database]) {
                        tableList[database] = {};
                    }
                    if (!tableList[database][schema]) {
                        tableList[database][schema] = [];
                    }
                    tableList[database][schema].push(table);
                });
            });
            resolve(tableList);
        }).catch(reject);
    });
};
exports.runListTables = runListTables;
// runs the query
const runGetRows = (userContext, stepData, physicalLocations, macros, token) => {
    const { name, locationName } = stepData;
    const sqlToExecute = `SELECT * FROM {{ ref('${locationName}', '${name}') }} LIMIT 100`;
    const startTime = new Date();
    return (0, exports.runSQLStringWithTemplateParams)(userContext, sqlToExecute, macros, physicalLocations, token).then((result) => {
        console.log("Result: ", result);
        let queryResult = RunnerOperations.GetQueryResultFromTemplateRunResult(result, startTime);
        return queryResult;
        //todo:handle runTemplate failure
    });
};
exports.runGetRows = runGetRows;
var ESnowflakeDataReturnStatus;
(function (ESnowflakeDataReturnStatus) {
    ESnowflakeDataReturnStatus["success"] = "Success";
    ESnowflakeDataReturnStatus["overLimit"] = "Transaction exceeded data limit, returned a subset of rows";
})(ESnowflakeDataReturnStatus = exports.ESnowflakeDataReturnStatus || (exports.ESnowflakeDataReturnStatus = {}));
//PORTED FROM SCHEDULER SNOWFLAKE
//TESTING INJECTION POINTS 
exports.createInjection = Common.REGISTER_INJECTION("CREATE_CONNECTION");
exports.createInjectionWaiting = Common.REGISTER_INJECTION("CREATE_CONNECTION_WAITING");
exports.waitForValidConnectionHasConnection = Common.REGISTER_INJECTION("_WaitForValidConnection Has Connection");
const IsValidConnection = (context) => {
    let valid = true;
    if (!context.accountName || !context.user)
        valid = false;
    return valid;
};
exports.IsValidConnection = IsValidConnection;
var EConnectionLockStatus;
(function (EConnectionLockStatus) {
    EConnectionLockStatus["unlocked"] = "unlocked";
    EConnectionLockStatus["creating"] = "creating";
    EConnectionLockStatus["deleting"] = "deleting";
    EConnectionLockStatus["refreshing"] = "refreshing";
})(EConnectionLockStatus || (EConnectionLockStatus = {}));
const AlreadyDisconnectedErrorString = "Already disconnected.";
/**
 * This storage class is meant for holding active connections to snowflake for the purpose of 2FA/MFA
 */
class ConnectionStorageClass {
    constructor() {
        /**
         * =====PUBLIC METHODS=====
         */
        this.setup = () => {
            return Promise.resolve();
        };
        /**
         * There are cases in which a connection metadata object will exist with a null connection value. In that case the destroy process
         * 	for that connection is skipped
         */
        this.teardown = () => {
            return new Promise((resolve, reject) => {
                const promisedArray = [];
                Object.keys(this.connectionCache).forEach((org) => {
                    Object.keys(this.connectionCache[org]).forEach((env) => {
                        Object.keys(this.connectionCache[org][env]).forEach((user) => {
                            Object.keys(this.connectionCache[org][env][user]).forEach((connection) => {
                                promisedArray.push(new Promise((resolve, reject) => {
                                    if (!this.connectionCache[org][env][user][connection].connection) {
                                        resolve();
                                    }
                                    else {
                                        this.connectionCache[org][env][user][connection].connection.destroy((err, conn) => {
                                            //(do later, not needed now) MUTEX: use busy wait with _WaitForConnectionRefresh here until each connection is no longer being refreshed
                                            delete this.connectionCache[connection];
                                            if (err) {
                                                ConnectionLogger.error(`Failed to destroy connection (teardown) ${conn}`, err);
                                                reject(err);
                                            }
                                            else {
                                                ConnectionLogger.info("connection deleted from cache (teardown)", connection);
                                                resolve();
                                            }
                                        });
                                    }
                                }));
                            });
                        });
                    });
                });
                Promise.all(promisedArray).then(() => {
                    resolve();
                });
            });
        };
        /**
         * Waits for a valid connection, and then runs a snowflake query with the connection and the input query object. If the query fails
         * 	because either the snowflake connection died, or because the oauth access token was expired, it will either delete the connection
         * 	and retry, or attempt a token refresh and retry, respecitively
         * @param firebaseIDToken
         * @param queryObj
         * @param userContext
         * @param teamInfoAndFirebase
         * @returns resolves with the resolution of a snowflake query
         */
        this.GetConnectionAndRunQuery = (queryObj, userContext, teamInfoAndFirebase) => {
            const { fbTeamID, fbUserID } = teamInfoAndFirebase.teamInfo;
            const { environmentID, user, password, role, warehouse } = userContext;
            const logContext = { orgID: fbTeamID, userID: fbUserID, envID: environmentID };
            let connectionWithMetadata;
            return new Promise((resolve, reject) => {
                this._WaitForValidConnection(userContext, teamInfoAndFirebase, logContext)
                    .then((output) => {
                    if (this.forceTestRefresh) {
                        // setting the param to false prevents an infinite loop
                        this.forceTestRefresh = false;
                        throw Error(exports.deadAccessCodeErrorString);
                    }
                    return output;
                })
                    .then((output) => {
                    //this assignment is separate from the previous .then block to align with our testing methods for refreshes
                    connectionWithMetadata = output;
                    return (0, exports.RunSnowflakeQuery)(queryObj, connectionWithMetadata.connection, fbTeamID, fbUserID, environmentID)
                        .then(resolve);
                })
                    .catch((err) => {
                    //grab connection here and set to connectionWithMetadata (only if undefined)
                    let retryPromise;
                    if ((0, exports.CheckForDeadConnectionErrorString)(err)) {
                        ConnectionLogger.infoContext(logContext, `Expired connection detected`);
                        const isLocked = this._CheckIsLocked(connectionWithMetadata, logContext);
                        if (isLocked) {
                            retryPromise = Promise.resolve();
                        }
                        else {
                            retryPromise = this.DeleteConnectionKey(userContext, teamInfoAndFirebase, logContext);
                        }
                    }
                    else if ((0, exports.CheckForDeadAccessToken)(err)) {
                        ConnectionLogger.infoContext(logContext, `Expired OAuth access token detected`);
                        retryPromise = this._AttemptRefresh(userContext, teamInfoAndFirebase, logContext);
                    }
                    if (retryPromise) {
                        retryPromise.then(() => {
                            ConnectionLogger.infoContext(logContext, `Retrying query (GetConnectionAndRunQuery)`);
                            return this.GetConnectionAndRunQuery(queryObj, userContext, teamInfoAndFirebase)
                                .then(resolve)
                                .catch(reject);
                        })
                            .catch(reject);
                    }
                    else {
                        ConnectionLogger.errorContext(logContext, "GetConnectionAndRunQuery", err);
                        reject(err);
                    }
                });
            });
        };
        /**
         * Constructs a connection key based off inputted context info, and then grabs that connection from the connection cache. After that it
         * 	waits to get the connection lock and passes off to _DeleteConnectionKeyLocked. Will resolve early if it sees that another thread is
         * 	already deleting
         * @param userContext
         * @param teamInfoAndFirebase
         * @param id_token firebase id token
         * @returns nothing, it simply resolves after either seeing that its connection is already being deleted, or after receiving word back from
         * 	_DeleteConnectionKeyLocked that the deletion is finished
         */
        this.DeleteConnectionKey = (userContext, teamInfoAndFirebase, logContext) => {
            const { password, warehouse, role, user, accountName } = userContext;
            const connectionKey = this._GetConnectionKey(user || "", password || "", role || "", warehouse || "", accountName || "", logContext);
            const connectionWithMetadata = this._GetConnectionFromConnectionCache(connectionKey, logContext);
            const initialLockStatus = this._CheckLockStatus(connectionWithMetadata, logContext);
            return new Promise((resolve, reject) => {
                if (initialLockStatus === EConnectionLockStatus.deleting) {
                    resolve();
                }
                else {
                    this._GetLock(connectionWithMetadata, EConnectionLockStatus.deleting, logContext)
                        .then(() => {
                        return this._DeleteConnectionKeyLocked(connectionKey, logContext);
                    })
                        .then(resolve)
                        .catch(reject)
                        .finally(() => {
                        this._UnlockConnection(connectionWithMetadata, logContext);
                    });
                }
            });
        };
        /**
         * =====MUTEX METHODS=====
         */
        /**
         * Essentially a wrapper for _CheckIsLockedAndResolve. It will create a connection metadata object, and a connection if either are missing
         * @param userContext
         * @param teamInfoAndFirebase
         * @param id_token
         * @returns resolves with a connection metadata object that has an initialized connection
         */
        this._WaitForValidConnection = (userContext, teamInfoAndFirebase, logContext) => {
            const { password, warehouse, role, user, accountName } = userContext;
            //todo - make _GetConnectionKey and GetConnectionFromConnectionCache into one function or make a wrapper that does both
            const connectionKey = this._GetConnectionKey(user || "", password || "", role || "", warehouse || "", accountName || "", logContext);
            const initialConnectionCheck = this._GetConnectionFromConnectionCache(connectionKey, logContext);
            const hasConnectionMetadata = !!initialConnectionCheck;
            const hasConnection = !!Common.getValueSafe(initialConnectionCheck, ["connection"], null);
            return new Promise((resolve, reject) => {
                if (!hasConnection) {
                    //move this stuff into chckIsLockedAndResolved
                    ConnectionLogger.infoContext(logContext, `Creating Connection (_WaitForValidConnection)`);
                    let connectionToUse = initialConnectionCheck;
                    if (!hasConnectionMetadata) {
                        connectionToUse = this._SaveConnectionObjectWithNullConnection(connectionKey, logContext);
                    }
                    this._CreateAndSaveSnowflakeConnection(connectionToUse, userContext, teamInfoAndFirebase, connectionKey, logContext)
                        .then((createdConnection) => {
                        resolve(connectionToUse);
                    })
                        .catch(reject);
                }
                else {
                    ConnectionLogger.infoContext(logContext, `Using Cached Connection (_WaitForValidConnection)`);
                    Common.INJECTION_POINT_ASYNC(exports.waitForValidConnectionHasConnection, { this: this }).then(() => {
                        this._CheckIsLockedAndResolve(userContext, teamInfoAndFirebase, connectionKey, logContext).then(resolve).catch(reject);
                    });
                }
            });
        };
        /**
         * an alternate version of _CheckIsLockedAndResolve. whereas _CheckIsLockedAndResolve returns a connection once it is unlocked,
         * 	this one immediately locks the connection and returns it once it is unlocked
         * @param connectionWithMetadata
         * @param lockedStatusCode
         * @returns resolves with a connection metadata object that has an initialized connection
         */
        this._GetLock = (connectionWithMetadata, lockedStatusCode, logContext) => {
            return new Promise((resolve, reject) => {
                const checkIsLockedAndFinish = () => {
                    if (connectionWithMetadata.lockStatus === EConnectionLockStatus.unlocked) {
                        ConnectionLogger.infoContext(logContext, `Returning locked connection (_GetLock)`);
                        connectionWithMetadata.lockStatus = lockedStatusCode;
                        resolve(connectionWithMetadata);
                    }
                    else {
                        setTimeout(checkIsLockedAndFinish, 500);
                    }
                };
                checkIsLockedAndFinish();
            });
        };
        /**
         * simple helper method for setting the connection lockStatus to unlocked, and the one below returns what the lockStatus currently is
         */
        this._UnlockConnection = (connectionWithMetadata, logContext) => {
            this._AssertLocked(connectionWithMetadata, "_UnlockConnection", logContext);
            ConnectionLogger.infoContext(logContext, `Unlocking created`);
            connectionWithMetadata.lockStatus = EConnectionLockStatus.unlocked;
        };
        this._CheckLockStatus = (connectionWithMetadata, logContext) => {
            ConnectionLogger.infoContext(logContext, `Checking Lock Status (_CheckLockStatus)`);
            Common.assert(Logging.LoggingArea.RunnerBackend, !!connectionWithMetadata, "Connection error: connection undefined! (_CheckLockStatus)");
            return connectionWithMetadata.lockStatus;
        };
        /**
         * methods below are simple booleans for determining if a connection is locked/unlocked
         */
        this._CheckIsLocked = (connectionWithMetadata, logContext) => {
            ConnectionLogger.infoContext(logContext, `Checking if connection is locked (_CheckIsLocked)`);
            Common.assert(Logging.LoggingArea.RunnerBackend, !!connectionWithMetadata, "Connection error: connection undefined! (_CheckIsLocked)");
            return connectionWithMetadata.lockStatus !== EConnectionLockStatus.unlocked;
        };
        this._CheckIsUnlocked = (connectionWithMetadata, logContext) => {
            ConnectionLogger.infoContext(logContext, `Checking if connection is unlocked (_CheckIsUnlocked)`);
            Common.assert(Logging.LoggingArea.RunnerBackend, !!connectionWithMetadata, "Connection error: connection undefined! (_CheckIsUnlocked)");
            return !this._CheckIsLocked(connectionWithMetadata, logContext);
        };
        /**
         * A busy wait function that will rerun itself every 500ms if it finds that the connection it is looking at is currently locked.
         * 	will create a new connection if it finds it undefined, but does not check to see if the connection metadata object is undefined,
         * 	as it assumes that object already exists
         * @param userContext
         * @param teamInfoAndFirebase
         * @param connectionKey generated connection cache key
         * @param id_token firebase id token
         * @returns resolves with a connection metadata object that has an initialized connection
         */
        this._CheckIsLockedAndResolve = (userContext, teamInfoAndFirebase, connectionKey, logContext) => {
            const currentConnectionCheck = this._GetConnectionFromConnectionCache(connectionKey, logContext);
            const hasConnection = !!Common.getValueSafe(currentConnectionCheck, ["connection"], null);
            return new Promise((resolve, reject) => {
                if (this._CheckIsLocked(currentConnectionCheck, logContext)) {
                    Common.INJECTION_POINT_ASYNC(exports.createInjectionWaiting, {}).then(() => {
                        setTimeout(() => {
                            this._CheckIsLockedAndResolve(userContext, teamInfoAndFirebase, connectionKey, logContext).then(resolve).catch(reject);
                        }, 500);
                    });
                }
                else {
                    Common.assert(Logging.LoggingArea.RunnerBackend, (currentConnectionCheck && this._CheckIsUnlocked(currentConnectionCheck, logContext)), "Uncaught isLocked mutex conflict (_WaitForValidConnection)");
                    if (!hasConnection) {
                        this._CreateAndSaveSnowflakeConnection(currentConnectionCheck, userContext, teamInfoAndFirebase, connectionKey, logContext)
                            .then(resolve)
                            .catch(reject);
                    }
                    else {
                        resolve(currentConnectionCheck);
                    }
                }
            });
        };
        /**
         * an alternate version of _CheckIsLockedAndResolve that does not create a connection if it finds one missing. This
         * 	method is currently only used for testing, and probably shouldn't actually be used in any real use-cases
         */
        this._CheckIsLockedAndResolveNoCreate = (userContext, teamInfoAndFirebase, connectionKey, id_token, logContext) => {
            const currentConnectionCheck = this._GetConnectionFromConnectionCache(connectionKey, logContext);
            return new Promise((resolve, reject) => {
                if (this._CheckIsLocked(currentConnectionCheck, logContext)) {
                    Common.INJECTION_POINT_ASYNC(exports.createInjectionWaiting, {}).then(() => {
                        setTimeout(() => {
                            this._CheckIsLockedAndResolveNoCreate(userContext, teamInfoAndFirebase, connectionKey, id_token, logContext).then(resolve).catch(reject);
                        }, 500);
                    });
                }
                else {
                    Common.assert(Logging.LoggingArea.RunnerBackend, (currentConnectionCheck && this._CheckIsUnlocked(currentConnectionCheck, logContext)), "Uncaught isLocked mutex conflict (_WaitForValidConnection)");
                    resolve(currentConnectionCheck);
                }
            });
        };
        /**
         * Assertion function for making sure that a connection does not unlock prematurely
         * does not actually crash, need a crashing version later
         * TODO PASS IN CONTEXT
         * @param connectionWithMetadata
         * @param functionName
         */
        this._AssertLocked = (connectionWithMetadata, functionName, logContext) => {
            const assertion = connectionWithMetadata.lockStatus !== EConnectionLockStatus.unlocked;
            Common.assert(Logging.LoggingArea.RunnerBackend, assertion, `Mutex Error: expected connection to be locked in (${functionName}), but was unlocked (_AssertLocked)`);
        };
        /**
         * =====CONNECTION CACHE METHODS=====
         */
        /**
         * runs all the input through an encrypted key generator, while also adding a bit of salt. setting the result toString is required, as otherwise
         * 	it returns an encrypted object that messes up the connection cache
         * @param username
         * @param password
         * @param role
         * @param warehouse
         * @param logContext contains orgID, userID, and envID
         * @returns an encrypted string
         */
        this._GetConnectionKey = (username, password, role, warehouse, accountName, logContext) => {
            const { orgID, userID, envID } = logContext;
            return exports.CryptoJS.PBKDF2(`${accountName}_${orgID}_${envID}_${userID}_${username}_${password}_${role}_${warehouse}`, exports.salt, {
                keySize: 128 / 32
            }).toString();
        };
        /**
         * TODO: replace getValueSafe with assertions
         */
        this._GetConnectionFromConnectionCache = (connectionKey, logContext) => {
            const { orgID, userID, envID } = logContext;
            return Common.getValueSafe(this.connectionCache, [orgID, envID.toString(), userID.toString(), connectionKey], null);
        };
        /**
         * =====CONNECTION CREATION METHODS=====
         */
        /**
         * essentially a wrapper function for _CreateAndSaveSnowflakeConnectionLocked, however it will wait and resolve with the current connection
         * 	if it sees that the connection is locked because it is creating
         * @param connectionWithMetadata
         * @param userContext
         * @param teamInfoAndFirebase
         * @param connectionKey
         * @returns
         */
        this._CreateAndSaveSnowflakeConnection = (connectionWithMetadata, userContext, teamInfoAndFirebase, connectionKey, logContext) => {
            RunnerBackendLogger.info("connectionWithMetadata: ", JSON.stringify(connectionWithMetadata));
            return new Promise((resolve, reject) => {
                const lockStatus = this._CheckLockStatus(connectionWithMetadata, logContext);
                if (lockStatus === EConnectionLockStatus.creating) {
                    this._AssertLocked(connectionWithMetadata, "_CreateAndSaveSnowflakeConnection_alternate", logContext);
                    ConnectionLogger.infoContext(logContext, `Waiting for created connection (_CreateAndSaveSnowflakeConnection)`);
                    this._CheckIsLockedAndResolve(userContext, teamInfoAndFirebase, connectionKey, logContext)
                        .then(resolve).catch(reject);
                }
                else {
                    this._GetLock(connectionWithMetadata, EConnectionLockStatus.creating, logContext)
                        .then(() => {
                        return Common.INJECTION_POINT_ASYNC(exports.createInjection, { this: this });
                    })
                        .then(() => {
                        this._AssertLocked(connectionWithMetadata, "_CreateAndSaveSnowflakeConnection_1", logContext);
                        return this._CreateAndSaveSnowflakeConnectionLocked(userContext, connectionKey, teamInfoAndFirebase, connectionWithMetadata, logContext);
                    })
                        .then((createdConnection) => {
                        this._AssertLocked(connectionWithMetadata, "_CreateAndSaveSnowflakeConnection_2", logContext);
                        resolve(createdConnection);
                    })
                        .catch((err) => {
                        this._AssertLocked(connectionWithMetadata, "_CreateAndSaveSnowflakeConnection_catch", logContext);
                        reject(err);
                    })
                        .finally(() => {
                        //should this be removed? potentially permanently locked connection case
                        this._AssertLocked(connectionWithMetadata, "_CreateAndSaveSnowflakeConnection_finally", logContext);
                        this._UnlockConnection(connectionWithMetadata, logContext);
                    });
                }
            });
        };
        /**
         * This function creates a new snowflake connection, saves it to IConnectionStorage, and then returns that connection object.
         * It should be noted that the new connection object has not been connected to at this point
         * This function will also delete the current connection before creating a new one
         */
        this._CreateAndSaveSnowflakeConnectionLocked = (userContext, connectionKey, teamInfoAndFirebase, connectionWithMetadata, logContext) => {
            const { fbTeamID, fbUserID } = teamInfoAndFirebase.teamInfo;
            const { environmentID } = userContext;
            const connectionType = userContext.authenticator;
            return new Promise((resolve, reject) => {
                //lock is set to 
                this._AssertLocked(connectionWithMetadata, "_CreateAndSaveSnowflakeConnectionLocked_start", logContext);
                return this._DeleteConnectionKeyLocked(connectionKey, logContext)
                    .then(() => {
                    this._AssertLocked(connectionWithMetadata, "_CreateAndSaveSnowflakeConnectionLocked_3", logContext);
                    let connectionPromise;
                    RunnerBackendLogger.info("Creating connection with userContext: ", JSON.stringify(userContext));
                    if (userContext.authenticator === ConnectionOperations.EUserConnectionTypes.basic) {
                        connectionPromise = this._CreateSnowflakeConnectionBasicLocked(userContext, connectionWithMetadata, logContext);
                    }
                    else if (userContext.authenticator === ConnectionOperations.EUserConnectionTypes.keyPair) {
                        connectionPromise = this._CreateSnowflakeConnectionKeyPairLocked(userContext, connectionWithMetadata, logContext);
                    }
                    else if (userContext.authenticator === ConnectionOperations.EUserConnectionTypes.oauth) {
                        connectionPromise = this._CreateSnowflakeConnectionOAuthLocked(userContext, teamInfoAndFirebase, connectionWithMetadata, logContext);
                    }
                    else {
                        throw new Error("ERROR(_CreateAndSaveSnowflakeConnectionLocked): invalid or missing authenticator type");
                    }
                    return connectionPromise;
                })
                    .then((newConnection) => {
                    this._AssertLocked(connectionWithMetadata, "_CreateAndSaveSnowflakeConnectionLocked_4", logContext);
                    newConnection.connect((err, conn) => {
                        if (err) {
                            RunnerBackendLogger.error(`Error connecting to new snowflake connection (_CreateAndSaveSnowflakeConnectionLocked) in org:${fbTeamID} in environment:${environmentID} for user:${fbUserID}, ${err}`);
                            reject(err);
                            return;
                        }
                        this._SaveSnowflakeConnectionLocked(connectionKey, newConnection, connectionWithMetadata, logContext);
                        ConnectionLogger.infoContext(logContext, `Connection of type:${connectionType} created (cached)`);
                        resolve(newConnection);
                    });
                })
                    .catch((err) => {
                    RunnerBackendLogger.error(`Error creating cached snowflake connection (_CreateAndSaveSnowflakeConnectionLocked) in org:${fbTeamID} in environment:${environmentID} for user:${fbUserID}, ${err}`);
                    reject(err);
                });
            });
        };
        /**
         * Method for creating a connection with the basic method
         */
        this._CreateSnowflakeConnectionBasicLocked = (userContext, connectionWithMetadata, logContext) => {
            return new Promise((resolve, reject) => {
                this._AssertLocked(connectionWithMetadata, "_CreateSnowflakeConnectionBasicLocked_start", logContext);
                const { accountName, user, password, role, warehouse } = userContext;
                if (!(0, exports.IsValidConnection)(userContext)) {
                    // adding this log because there seems to be a scenario when we hit this condition even when the credentials look valid in the front end
                    ConnectionLogger.infoContext(logContext, "Error creating basic snowflake connection (CreateSnowflakeConnectionBasic)", { user: Logging.MaskLogValue(user), accountName: Logging.MaskLogValue(accountName) });
                    return reject(new Error("Invalid Connection Information, please correct your account or username and proceed."));
                }
                const newConnection = this._CreateSnowflakeConnectionLocked(accountName, user, password, role || "", warehouse || "", connectionWithMetadata, logContext);
                ConnectionLogger.infoContext(logContext, "create snowflake connection basic", newConnection);
                resolve(newConnection);
            });
        };
        this._CreateSnowflakeConnectionKeyPairLocked = (userContext, connectionWithMetadata, logContext) => {
            return new Promise((resolve, reject) => {
                this._AssertLocked(connectionWithMetadata, "_CreateSnowflakeConnectionKeyPairLocked", logContext);
                const { accountName, user, warehouse } = userContext;
                const newConnection = snowflakeSDK.createConnection({
                    authenticator: "SNOWFLAKE_JWT",
                    privateKey: userContext.keyPairKey,
                    account: accountName,
                    username: user,
                    warehouse: warehouse || "",
                    application: "CoalesceIO_Coalesce"
                });
                ConnectionLogger.infoContext(logContext, "create snowflake connection KeyPair", newConnection);
                resolve(newConnection);
            });
        };
        // snowflake connection wrapper to handle optional parameters
        this._CreateSnowflakeConnectionLocked = (account, username, password, role, warehouse, connectionWithMetadata, logContext) => {
            // add required params
            this._AssertLocked(connectionWithMetadata, "_CreateSnowflakeConnectionLocked_start", logContext);
            let connParams = {
                account: account,
                username: username,
                password: password,
                application: "CoalesceIO_Coalesce"
            };
            // add optional params
            if (role) {
                connParams.role = role;
            }
            if (warehouse) {
                connParams.warehouse = warehouse;
            }
            // return connection
            const sfConn = snowflakeSDK.createConnection(connParams);
            return sfConn;
        };
        /**
         * method for creating an oauth connection
         */
        this._CreateSnowflakeConnectionOAuthLocked = (userContext, tokenData, connectionWithMetadata, logContext) => {
            /*
             * check if information is in firestore
             * check if code is supplied to generate data via tokenURL
             * if neither, send back error
             */
            return new Promise((resolve, reject) => {
                this._AssertLocked(connectionWithMetadata, "_CreateSnowflakeConnectionOAuthLocked_1", logContext);
                this._GetAccessTokenFromSecret(userContext, tokenData, logContext)
                    .then((data) => {
                    const { accountName, user, warehouse } = userContext;
                    const newConnection = snowflakeSDK.createConnection({
                        authenticator: "OAUTH",
                        account: accountName,
                        username: user,
                        token: data.access_token,
                        warehouse: warehouse || "",
                        application: "CoalesceIO_Coalesce"
                    });
                    ConnectionLogger.infoContext(logContext, "create snowflake connection OAuth", newConnection);
                    resolve(newConnection);
                })
                    .catch((err) => {
                    RunnerBackendLogger.error(`Error creating Oauth Snowflake connection (_CreateSnowflakeConnectionOAuthLocked) in org:${userContext.orgID} in environment:${userContext.environmentID}, ${err}`);
                    reject(err);
                });
            });
        };
        /**
         * helper method for initializing a connection metadata object if one does not exist
         */
        this._SaveConnectionObjectWithNullConnection = (connectionKey, logContext) => {
            const { orgID, userID, envID } = logContext;
            const createdConnection = {
                connection: null,
                lockStatus: EConnectionLockStatus.unlocked
            };
            this.connectionCache[orgID] = this.connectionCache[orgID] || {};
            this.connectionCache[orgID][envID] = this.connectionCache[orgID][envID] || {};
            this.connectionCache[orgID][envID][userID] = this.connectionCache[orgID][envID][userID] || {};
            this.connectionCache[orgID][envID][userID][connectionKey] = createdConnection;
            ConnectionLogger.infoContext(logContext, "Generated connection object (_SaveConnectionObjectWithNullConnection)");
            return createdConnection;
        };
        /**
         * This function looks the way it does because of how IConnectionStorage is shaped. There is a good chance that
         * 		[orgID] and [orgID][userID] do not exist yet when a new connection is created, and gauranteed if it is the
         * 		first connection to be created
         */
        this._SaveSnowflakeConnectionLocked = (connectionKey, newConnection, connectionWithMetadata, logContext) => {
            this._AssertLocked(connectionWithMetadata, "_SaveSnowflakeConnectionLocked_start", logContext);
            const { orgID, userID, envID } = logContext;
            /*
             * initialize orgID and userID objects
             *  TODO - CHECK FOR CONNECTION HERE AND THROW IF FOUND
             */
            this.connectionCache[orgID] = this.connectionCache[orgID] || {};
            this.connectionCache[orgID][envID] = this.connectionCache[orgID][envID] || {};
            this.connectionCache[orgID][envID][userID] = this.connectionCache[orgID][envID][userID] || {};
            this.connectionCache[orgID][envID][userID][connectionKey].connection = newConnection;
            ConnectionLogger.infoContext(logContext, "cache snowflake connection");
        };
        /**
         * =====CONNECTION DELETION METHODS=====
         */
        /**
         * method for destroying a connection. Checks if there is an actual connection present, and if not, will simply resolve
         */
        this._DeleteConnectionKeyLocked = (connectionKey, logContext) => {
            //MUTEX: if the given connection is locked, busy wait with _WaitForConnectionLocked until it is done
            return new Promise((resolve, reject) => {
                const { orgID, userID, envID } = logContext;
                //update to this._GetConnectionFromConnectionCache
                const connectionWithMetadata = Common.getValueSafe(this.connectionCache, [orgID, envID, userID, connectionKey], null);
                this._AssertLocked(connectionWithMetadata, "_DeleteConnectionKeyWhenLocked_start", logContext);
                if (!!connectionWithMetadata && !!connectionWithMetadata.connection) {
                    /**
                     * TODO - ASSERT THAT ONLY ONE CONNECTIONS EXISTS
                     * only have one connection per user per env at the moment, when that changes, this will need to change
                     * USING TWO DIFFERENT METHODS TO GET CONNECTION KEYS IN THE SAME FUNCTION
                     */
                    const connectionKey = Object.keys(connectionWithMetadata)[0];
                    const connection = connectionWithMetadata[connectionKey];
                    RunnerBackendLogger.infoContext(logContext, `Starting destroy for connection (_DeleteConnectionKeyLocked)`);
                    connectionWithMetadata.connection = null;
                    return connection.destroy((err, conn) => {
                        this._AssertLocked(connectionWithMetadata, "_DeleteConnectionKeyWhenLocked_2", logContext);
                        if (err) {
                            if (err.message === AlreadyDisconnectedErrorString) {
                                ConnectionLogger.infoContext(logContext, `Unable to destroy connection: connection already destroyed`);
                                resolve(err);
                            }
                            else {
                                ConnectionLogger.errorContext(logContext, `Unable to destroy snowflake cached connection (_DeleteConnectionKeyLocked), error: ${err}`);
                                reject(err);
                            }
                        }
                        else {
                            ConnectionLogger.infoContext(logContext, `Disconnected snowflake connection (_DeleteConnectionKeyLocked)`);
                            connectionWithMetadata.connection = null;
                            resolve();
                        }
                    });
                }
                else {
                    RunnerBackendLogger.infoContext(logContext, `No connection to disconnect (_DeleteConnectionKeyLocked)`);
                    resolve();
                }
            });
        };
        /**
         * =====SECRET MANAGER METHODS=====
         */
        /**
         *
         * @param userContext
         * @returns a promse containting a google secret in the shape of IOAuthSecret
         */
        this._GetAccessTokenFromSecret = (userContext, tokenData, logContext) => {
            const { environmentID } = userContext;
            return new Promise((resolve, reject) => {
                const { teamInfo } = tokenData;
                const logContext = {
                    orgID: teamInfo.fbTeamID,
                    userID: teamInfo.fbUserID,
                    envID: environmentID
                };
                ConnectionOperations.GetCurrentUserSecretFromFirebase(tokenData, environmentID)
                    .then((id) => {
                    return Secrets.GetSecret(teamInfo.fbTeamID, teamInfo.fbUserID, id);
                })
                    .then((data) => {
                    ConnectionLogger.infoContext(logContext, "get access token from secret (_GetAccessTokenFromSecret");
                    resolve(data);
                })
                    .catch((err) => {
                    ConnectionLogger.errorContext(logContext, "Error getting access token from secret (_GetAccessTokenFromSecret)", err);
                    reject(err);
                });
            });
        };
        /**
         * =====Snowflake Refresh Methods=====
         */
        /**
         * @param ISnowflakeConnectionClass
         * @param userContext
         * @param tokenData
         * @param callback
         * @param callbackParam
         */
        this._AttemptRefresh = (userContext, tokenData, logContext) => {
            return new Promise((resolve, reject) => {
                const { teamInfo, firebase } = tokenData;
                const { fbTeamID, fbUserID } = teamInfo;
                const { environmentID, user, password, role, warehouse, accountName } = userContext;
                const connectionKey = this._GetConnectionKey(user || "", password || "", role || "", warehouse || "", accountName || "", logContext);
                const connectionWithMetadata = this._GetConnectionFromConnectionCache(connectionKey, logContext);
                const userDoc = CommonOperations.getDocumentRefAdmin(firebase.firestore(), fbTeamID, environmentID, "users").doc(fbUserID);
                /**
                 * MUTEX: if isLocked is true, wait for isLocked to turn false and then resolve
                 */
                if (this._CheckLockStatus(connectionWithMetadata, logContext) === EConnectionLockStatus.refreshing) {
                    resolve();
                }
                else {
                    this._GetLock(connectionWithMetadata, EConnectionLockStatus.refreshing, logContext)
                        .then(() => {
                        this._AssertLocked(connectionWithMetadata, "_AttemptRefreshLocked_start", logContext);
                        return userDoc.get();
                    })
                        .then((doc) => {
                        var _a, _b;
                        this._AssertLocked(connectionWithMetadata, "_AttemptRefreshLocked_2", logContext);
                        if (((_a = doc.data()) === null || _a === void 0 ? void 0 : _a.connectionType) === ConnectionOperations.EUserConnectionTypes.oauth) {
                            return this._RefreshAccessTokenLocked(userContext, tokenData, connectionWithMetadata, logContext);
                        }
                        else {
                            throw new Error(`Connection type error: expected an oauth connection, but received connection type ${(_b = doc.data()) === null || _b === void 0 ? void 0 : _b.connectionType} (_AttemptRefresh)`);
                        }
                    })
                        .then(() => {
                        this._AssertLocked(connectionWithMetadata, "_AttemptRefreshLocked_3", logContext);
                        ConnectionLogger.infoContext(logContext, `Refresh token updated`);
                        //Delete connection.connectionObj and set connection.isLocked to false. need to update references to whether a connection exists to whether connection.connectionObj exists
                        return this._DeleteConnectionKeyLocked(connectionKey, logContext);
                    })
                        .then(() => {
                        this._AssertLocked(connectionWithMetadata, "_AttemptRefreshLocked_4", logContext);
                        resolve();
                    })
                        .catch((err) => {
                        RunnerBackendLogger.errorContext(logContext, `Error refreshing secret (_AttemptRefresh)`);
                        reject(err);
                    })
                        .finally(() => {
                        this._AssertLocked(connectionWithMetadata, "_AttemptRefreshLocked_finally", logContext);
                        this._UnlockConnection(connectionWithMetadata, logContext);
                    });
                }
            });
        };
        this._RefreshAccessTokenLocked = (userContext, tokenData, connectionWithMetadata, logContext) => {
            /*
             * get secret data for refresh token
             * delete old secret
             * ping token url with refresh token
             * update/create new secret
             */
            this._AssertLocked(connectionWithMetadata, "_RefreshAccessTokenLocked_start", logContext);
            const { fbTeamID, fbUserID } = tokenData.teamInfo;
            const { environmentID } = userContext;
            return new Promise((resolve, reject) => {
                let dataToUse, firebaseData, refresh_token;
                return ConnectionOperations.GetOAuthDetailsFromEnvironment(tokenData.firebase.firestore(), tokenData.teamInfo, environmentID)
                    .then((oauthInfo) => {
                    this._AssertLocked(connectionWithMetadata, "_RefreshAccessTokenLocked_1", logContext);
                    firebaseData = oauthInfo;
                    if (this.forceInvalidTokenFailure) {
                        const fakeOAuthSecret = {
                            access_token: null,
                            refresh_token: null
                        };
                        return fakeOAuthSecret;
                    }
                    else {
                        return this._GetAccessTokenFromSecret(userContext, tokenData, logContext);
                    }
                })
                    .then((data) => {
                    this._AssertLocked(connectionWithMetadata, "_RefreshAccessTokenLocked_2", logContext);
                    if (!data.refresh_token) {
                        throw new Error("Missing or invalid OAuth refresh token, please reauthenticate.");
                    }
                    refresh_token = data.refresh_token;
                    return data.refresh_token;
                })
                    .then((refreshToken) => {
                    this._AssertLocked(connectionWithMetadata, "_RefreshAccessTokenLocked_3", logContext);
                    const authorizationEncoded = Buffer.from(`${firebaseData.oauthClientID}:${firebaseData.oauthClientSecret}`).toString("base64");
                    var payloadData = qs_1.default.stringify({
                        grant_type: "refresh_token",
                        refresh_token: refreshToken
                    });
                    const url = `https://${firebaseData.connectionAccount}.snowflakecomputing.com/oauth/token-request`;
                    var config = {
                        method: "post",
                        url,
                        headers: {
                            Authorization: `Basic ${authorizationEncoded}`,
                            "Content-Type": "application/x-www-form-urlencoded"
                        },
                        data: payloadData
                    };
                    return RunnerOperations.RunDefaultAxiosRequest(config).catch(error => {
                        ConnectionLogger.errorContext(logContext, "RefreshAccessToken", error);
                        throw new Error("An error occured refreshing your OAuth refresh_token.  Please ensure your OAuth Security Integration is still valid or reauthenticate.");
                    });
                })
                    .then((response) => {
                    this._AssertLocked(connectionWithMetadata, "_RefreshAccessTokenLocked_4", logContext);
                    dataToUse = {
                        access_token: response.data.access_token,
                        refresh_token: refresh_token
                    };
                    return;
                })
                    .then(() => {
                    this._AssertLocked(connectionWithMetadata, "_RefreshAccessTokenLocked_5", logContext);
                    return ConnectionOperations.GetCurrentUserSecretFromFirebase(tokenData, environmentID);
                })
                    .then((oldUID) => {
                    this._AssertLocked(connectionWithMetadata, "_RefreshAccessTokenLocked_6", logContext);
                    return Secrets.DeleteSecret(fbTeamID, fbUserID, oldUID);
                })
                    .then(() => {
                    this._AssertLocked(connectionWithMetadata, "_RefreshAccessTokenLocked_7", logContext);
                    return ConnectionOperations.SetAndRetrieveNewUserSecretFirebase(tokenData, environmentID);
                })
                    .then((UID) => {
                    this._AssertLocked(connectionWithMetadata, "_RefreshAccessTokenLocked_8", logContext);
                    return Secrets.UpdateSecret(fbTeamID, fbUserID, UID, dataToUse);
                })
                    .then(() => resolve())
                    .catch((err) => {
                    RunnerBackendLogger.error(`Error refreshing access token (_RefreshAccessTokenLocked) in org:${fbTeamID} in environment:${environmentID} for user:${fbUserID}, ${err}`);
                    reject(err);
                });
            });
        };
        this.connectionCache = {};
        this.forceTestRefresh = false;
        this.forceInvalidTokenFailure = false;
    }
}
exports.ConnectionStorageClass = ConnectionStorageClass;
/**
 * This error string is specifically for when a snowflake query fails because its connection has timed out
 */
const deadConnectionErrorString = "Unable to perform operation using terminated connection.";
const deadConnectionErrorCode = 407002;
/**
 * This error string is specifically for when a snowflake oauth access token expires
 */
exports.deadAccessCodeErrorString = "OAuth access token expired.";
exports.CheckForDeadAccessToken = ((err) => {
    if (err.message.includes(exports.deadAccessCodeErrorString)) {
        return true;
    }
    return false;
});
/**
 *
 * @param err should have either a snowflake error object that has either a code property, or a message property
 */
const CheckForDeadConnectionErrorString = (err) => {
    if (err.code === deadConnectionErrorCode || err.message === deadConnectionErrorString) {
        return true;
    }
    return false;
};
exports.CheckForDeadConnectionErrorString = CheckForDeadConnectionErrorString;
const RunSnowflakeQueryInternal = (sfConn, options, teamID, userID, environmentID) => {
    const initialStats = exports.v8.getHeapStatistics();
    return new Promise((resolve, reject) => {
        sfConn.execute(Object.assign(Object.assign({}, options), { streamResult: true, complete: function (err, stmt, rows) {
                if (err) {
                    reject(err);
                    RunnerBackendLogger.infoContext({ orgID: teamID, envID: environmentID, userID }, `Error executing snowflake query (RunSnowflakeQueryInternal), ${err}`);
                    return;
                }
                let currentSize = 0;
                rows = [];
                const readableStream = stmt.streamRows();
                readableStream
                    .on("error", (error) => {
                    reject(error);
                    RunnerBackendLogger.alertContext({
                        orgID: teamID,
                        userID,
                        envID: environmentID
                    }, `Error on streaming rows: ${error}`);
                })
                    .on("data", (row) => {
                    // determine size of row
                    const rowSize = (0, exports.RoughSizeOfObject)(row);
                    currentSize += rowSize;
                    rows.push(row);
                    /**
                     * IsReachingHeapLimit is not in use - keeping it available as a tool, but we're deciding to solely based
                     * on the chunk limit right now.
                     */
                    if (IsOverChunkLimit(currentSize)) {
                        const currentStats = exports.v8.getHeapStatistics();
                        const currentHeapSize = ConvertBytesToMB(currentStats.total_heap_size);
                        const totalHeapSize = ConvertBytesToMB(initialStats.heap_size_limit);
                        const responseSize = ConvertBytesToMB(currentSize);
                        const maxResponseSize = ConvertBytesToMB(MAX_RESPONSE_SIZE);
                        RunnerBackendLogger.alertContext({
                            orgID: teamID,
                            userID,
                            envID: environmentID
                        }, `Chunk limit reached, 
								current heap size: ${currentHeapSize} MB, total heap size: ${totalHeapSize} MB,
								response size: ${responseSize} MB, max response size: ${maxResponseSize} MB`);
                        resolve({ rows, stmt, status: ESnowflakeDataReturnStatus.overLimit });
                        readableStream.destroy();
                    }
                })
                    .on("end", () => {
                    resolve({ rows, stmt, status: ESnowflakeDataReturnStatus.success });
                });
            } }));
    });
};
const ConvertBytesToMB = (bytes) => {
    return Math.round(bytes / 1000000);
};
const RoughSizeOfObject = (object) => {
    let objectList = [];
    const recurse = function (value) {
        let bytes = 0;
        if (typeof value === "boolean") {
            bytes = 4;
        }
        else if (typeof value === "string") {
            bytes = value.length * 2;
        }
        else if (typeof value === "number") {
            bytes = 8;
        }
        else if (typeof value === "object"
            && objectList.indexOf(value) === -1) {
            objectList[objectList.length] = value;
            for (let i in value) {
                bytes += 8; // an assumed existence overhead
                bytes += recurse(value[i]);
            }
        }
        return bytes;
    };
    return recurse(object);
};
exports.RoughSizeOfObject = RoughSizeOfObject;
// not in use for now - we're only using total response size as criteria for terminating a stream early
const IsReachingHeapLimit = (initialStats) => {
    const currentStats = exports.v8.getHeapStatistics();
    const totalHeapSizeThreshold = CalculateHeapSizeThreshold(initialStats.heap_size_limit);
    if ((currentStats.total_heap_size) > totalHeapSizeThreshold) {
        RunnerBackendLogger.info(`total heap size percentage: ${Math.round((currentStats.total_heap_size / initialStats.heap_size_limit) * 100)}%`);
        RunnerBackendLogger.info(`total heap size: ${currentStats.total_heap_size / 1000000} MB, threshold: ${totalHeapSizeThreshold / 1000000} MB`);
        return true;
    }
    return false;
};
// current max response size in MB: 20
const MAX_RESPONSE_SIZE = parseInt(process.env.MAX_RESPONSE_SIZE || "20428800");
const IsOverChunkLimit = (currentChunkSize) => {
    return currentChunkSize > MAX_RESPONSE_SIZE;
};
const MAX_RESOURCE_PERCENTAGE = parseInt(process.env.MAX_HEAP_ALLOCATION_PERCENTAGE || "20");
const CalculateHeapSizeThreshold = (initialSizeLimit) => {
    return (initialSizeLimit * MAX_RESOURCE_PERCENTAGE) / 100;
};
/**
 * A recursive function for calling the execution of a snowflake query
 * 		The reason this is recursive is to allow to retries of the snowflake query
 * 		in the case where a query fails because the connection given was
 * 		deceased for whatever reason. In this case the function will delete the old connection
 * 		and rerun, creating a new connection to use on the next go around. Any other error
 * 		will cause the error to be sent back to the client
 */
const RunSnowflakeQuery = (queryObj, connectionObj, teamID, userID, environmentID) => {
    return new Promise((resolve, reject) => {
        //with the connectionStorage embedded into the scheduler class, how does this work now?
        RunSnowflakeQueryInternal(connectionObj, queryObj, teamID, userID, environmentID)
            .then((result) => {
            resolve(result);
        })
            .catch((err) => {
            RunnerBackendLogger.error(`Error running snowflake query (RunSnowflakeQuery), ${err}`);
            reject(err);
        });
    });
};
exports.RunSnowflakeQuery = RunSnowflakeQuery;
//# sourceMappingURL=Snowflake.js.map