"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultTreeGridProps = exports.GetRowNodesForSubgraphTree = exports.GetRowNodesForNodeTree = exports.removeEmptyFolders = exports.GetDndNodeType = exports.persistExpandedFolders = exports.GetAgGridTreeSelectionData = exports.UpdateAvailableCategoryRowNodes = exports.EDatasetsTreeStateKeys = exports.ETreeCategories = void 0;
const Common = __importStar(require("./Common"));
const State = __importStar(require("./State"));
const StepOperations = __importStar(require("./StepOperations"));
const Logging = __importStar(require("./Logging"));
var ETreeCategories;
(function (ETreeCategories) {
    ETreeCategories["nodes"] = "nodes";
    ETreeCategories["subgraphs"] = "subgraphs";
    ETreeCategories["jobs"] = "jobs";
})(ETreeCategories = exports.ETreeCategories || (exports.ETreeCategories = {}));
var EDatasetsTreeStateKeys;
(function (EDatasetsTreeStateKeys) {
    EDatasetsTreeStateKeys["categorySelections"] = "gridCategorySelections";
    EDatasetsTreeStateKeys["treeCategory"] = "currentCategory";
})(EDatasetsTreeStateKeys = exports.EDatasetsTreeStateKeys || (exports.EDatasetsTreeStateKeys = {}));
/**
 * Adds available rowNodes to store, which is used by useUpdateTreeSelectionsFromGraph.
 * This provides the information to allows tree selections to mirror what is selected in the graph
 *
 * @param treeCategory
 * @param gridApi
 * @param dispatch
 * @returns
 */
const UpdateAvailableCategoryRowNodes = (treeCategory, gridApi, dispatch) => {
    const availableRows = [];
    gridApi.forEachNode(node => availableRows.push(node.data));
    // since multiple subgraphs exist on the same tree, the data needs to be handled differently, siloing the rownodes to specific folderCounters
    if (treeCategory === ETreeCategories.subgraphs) {
        const availableRowsPayload = availableRows.reduce((rowDataObject, row) => {
            if (row.heirarchy.length < 2)
                return rowDataObject;
            if (!rowDataObject[row.heirarchy[1]]) {
                rowDataObject[row.heirarchy[1]] = new Set();
                rowDataObject[row.heirarchy[1]].add(row);
                return rowDataObject;
            }
            rowDataObject[row.heirarchy[1]].add(row);
            return rowDataObject;
        }, {});
        dispatch(State.getUpdateDatasetsTreeSelections(treeCategory, "availableRowNodes", availableRowsPayload));
        return;
    }
    dispatch(State.getUpdateDatasetsTreeSelections(treeCategory, "availableRowNodes", new Set(availableRows)));
};
exports.UpdateAvailableCategoryRowNodes = UpdateAvailableCategoryRowNodes;
const GetAgGridTreeSelectionData = (gridSelectedNodes) => {
    Common.assert(Logging.LoggingArea.TreeOperations, !!gridSelectedNodes.length && !!gridSelectedNodes[0].data.heirarchy, "No available gridSelectedNodes");
    const firstSelectedRowFolderType = gridSelectedNodes[0].data.heirarchy[0];
    const selectedNodesOfSameFolderType = gridSelectedNodes
        .filter(node => node.data.heirarchy[0] === firstSelectedRowFolderType).map(node => {
        return {
            id: node.id,
            rowData: node.data,
        };
    });
    const agGridSelectionState = {
        rowType: gridSelectedNodes[0].data.type,
        folderType: firstSelectedRowFolderType,
        selectedRows: selectedNodesOfSameFolderType,
        selectedNodesIDs: new Set(selectedNodesOfSameFolderType.map(node => node.id)),
        firstSelectedRow: selectedNodesOfSameFolderType[0],
    };
    return agGridSelectionState;
};
exports.GetAgGridTreeSelectionData = GetAgGridTreeSelectionData;
// persist expanded folders state onMount for component
const persistExpandedFolders = (folders, gridApi) => {
    Object.keys(folders).forEach(folderKey => {
        const rowNode = gridApi.getRowNode(folderKey);
        /*
         * Calling node.setExpanded() causes the grid to get redrawn
         * If you have many nodes you want to expand, then it is best to set node.expanded=true directly,
         * and then call api.onGroupExpandedOrCollapsed() when finished to get the grid to redraw the grid again just once.
         */
        if (rowNode === null || rowNode === void 0 ? void 0 : rowNode.expanded) {
            rowNode.expanded = folders[folderKey];
        }
    });
    gridApi.onGroupExpandedOrCollapsed();
};
exports.persistExpandedFolders = persistExpandedFolders;
////////////// needed to grab node type on drag start
const GetDndNodeType = (e) => {
    const type = e.api.dragAndDropService.dragItem.rowNode.data.type;
    switch (type) {
        case StepOperations.PayloadType.step: {
            return Common.DragItemTypes.AG_Tree_Node;
        }
        case StepOperations.PayloadType.folder: {
            return Common.DragItemTypes.AG_Tree_Subgraph;
        }
    }
};
exports.GetDndNodeType = GetDndNodeType;
const removeEmptyFolders = (rowData) => {
    const parentFolder = rowData[0];
    const steps = rowData.filter(row => row.type === StepOperations.PayloadType.step);
    const subgraphs = rowData.filter(row => row.type === StepOperations.PayloadType.folder);
    const folders = rowData.filter(row => row.type === StepOperations.PayloadType.immutableFolder);
    const jobs = rowData.filter(row => row.type === StepOperations.PayloadType.job);
    const foldersWithContents = folders.reduce((filledFolders, folder) => {
        steps.forEach(step => {
            if (step.key.includes(folder.key) && !filledFolders.includes(folder)) {
                const folderKeyLength = folder.key.length;
                const stepKeySubStr = step.key.substring(0, folderKeyLength);
                if (stepKeySubStr === folder.key) {
                    filledFolders.push(folder);
                }
            }
        });
        return filledFolders;
    }, []);
    const groomedData = [parentFolder, ...foldersWithContents, ...steps, ...subgraphs, ...jobs];
    return groomedData;
};
exports.removeEmptyFolders = removeEmptyFolders;
const GetRowNodesForNodeTree = (datasetsTreeState) => {
    return Array.from(Common.getValueSafe(datasetsTreeState, ["availableRowNodes", ETreeCategories.nodes], []));
};
exports.GetRowNodesForNodeTree = GetRowNodesForNodeTree;
const GetRowNodesForSubgraphTree = (datasetsTreeState, subgraphID) => {
    return Array.from(Common.getValueSafe(datasetsTreeState, ["availableRowNodes", ETreeCategories.subgraphs, subgraphID], []));
};
exports.GetRowNodesForSubgraphTree = GetRowNodesForSubgraphTree;
////////////// Baseline Grid Props
exports.DefaultTreeGridProps = {
    rowClass: "backgroundWhite",
    rowDragManaged: true,
    headerHeight: 40,
    defaultColDef: {
        flex: 1,
        field: "value",
        cellRenderer: "agGroupCellRenderer",
        cellEditor: "agLargeTextCellEditor",
        cellRendererParams: {
            suppressCount: true,
            innerRenderer: "stepContainerCellRenderer",
            suppressDoubleClickExpand: true
        }
    },
    animateRows: true,
    stopEditingWhenGridLosesFocus: true,
    immutableData: true,
    getRowNodeId: data => data.key,
    getDataPath: (data) => data.heirarchy,
    suppressContextMenu: true,
    groupDefaultExpanded: -1,
    suppressMoveWhenRowDragging: true,
    enableMultiRowDragging: true,
    rowSelection: "multiple",
};
//# sourceMappingURL=TreeOperations.js.map