"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FlushFirestoreWorkspaceWithAllWorkspaceData = exports.isValidWorkspace = exports.getWorkspaceEdits = exports.ConvertEnvDetailsToEnvironmentInterface = exports.CreateOldLocationName = exports.CreateWorkspaceEditForRemoveDeploySuffix = exports.CreateWorkspaceEditForCloneTable = exports.CreateWorkspaceEditForSwapCloneTable = exports.CreateWorkspaceEditForDeleteTable = exports.WorkspaceEditType = exports.getAllWorkspaceDataFromFirebase = exports.getCollectionFromRef = exports.GetAllWorkspaceDataFromGitCommit = exports.getCurrentWorkspaceMappings = exports.GetRunTimeParametersFromWorkspacesSafe = exports.getCurrentWorkspaceID = exports.useGetDeploymentSettingsState = exports.IsWorkspaceNameUnique = exports.GetWorkspacesAdmin = exports.GetWorkspaces = exports.UpdateWorkspaceAdmin = exports.updateWorkspaceDeploy = exports.AddWorkspaceAdmin = exports.WorkspaceSetter = exports.TransactionWorkspaceSetter = exports.GetNewDevWorkspace = exports.GetNewEmptyWorkspace = exports.IsFirstEnvironment = exports.IsNewEnvironment = exports.listenWorkspaceChangeCb = exports.getCurrentWorkspace = exports.useNonDevWorkspaces = exports.useWorkspacesIDs = exports.useWorkspaces = exports.AddMissingDefaultWorkspaceValues = exports.useReduxWorkspaces = exports.useWorkspace = exports.MarkWorkspaceStatusFailedWithCatch = exports.MarkWorkspaceStatusFailed = exports.ReleaseWorkspaceStatus = exports.SecureWorkspaceStatus = exports.setStepColumnsAdmin = exports.deleteStepAdmin = exports.setSubgraphAdmin = exports.setStepAdmin = exports.setCollectionDocumentAdmin = exports.GetEnvironmentDocRef = exports.getWorkspacesCollectionRefAdmin = exports.getWorkspacesCollectionRef = void 0;
const react_1 = require("react");
const react_redux_1 = require("react-redux");
const StorageMappings = __importStar(require("./StorageMappings"));
const Common = __importStar(require("./Common"));
const Runner = __importStar(require("./Runner"));
const State = __importStar(require("./State"));
const GitOperations = __importStar(require("./GitOperation"));
const CommonOperations = __importStar(require("./CommonOperations"));
const RunnerOperations = __importStar(require("./RunnerOperations"));
const VersionControlActions = __importStar(require("./VersionControlActions"));
const Diff = __importStar(require("deep-diff"));
const Templates = __importStar(require("./Templates"));
const Immer = __importStar(require("immer"));
const StepTypes = __importStar(require("./StepTypes"));
const MetadataHydration = __importStar(require("./MetadataHydration"));
const Logging = __importStar(require("./Logging"));
const AdvancedDeployOperations = __importStar(require("./AdvancedDeployOperations"));
const Logger = Logging.GetLogger(Logging.LoggingArea.Deploy);
const GitLogger = Logging.GetLogger(Logging.LoggingArea.Git);
const WorkspaceLogger = Logging.GetLogger(Logging.LoggingArea.Workspaces);
const BurncoCD_2676Logger = Logging.GetLogger(Logging.LoggingArea.CD_2676Logger);
const RunnerBackendLogger = Logging.GetLogger(Logging.LoggingArea.RunnerBackend);
const getWorkspacesCollectionRef = (teamID) => {
    return CommonOperations.getOrgConfigDocRef(teamID).collection("workspaces");
};
exports.getWorkspacesCollectionRef = getWorkspacesCollectionRef;
const getWorkspacesCollectionRefAdmin = (firestore, teamID) => {
    return CommonOperations.getOrgConfigDocRefAdmin(firestore, teamID).collection("workspaces");
};
exports.getWorkspacesCollectionRefAdmin = getWorkspacesCollectionRefAdmin;
const GetEnvironmentDocRef = (firestore, teamID, environmentID) => {
    return (0, exports.getWorkspacesCollectionRefAdmin)(firestore, teamID).doc(environmentID.toString());
};
exports.GetEnvironmentDocRef = GetEnvironmentDocRef;
const setCollectionDocumentAdmin = (firebase, teamID, workspaceID, collection, documentID, documentData) => {
    return CommonOperations.getDocumentRefAdmin(firebase.firestore(), teamID, workspaceID, collection).doc(documentID.toString()).set(documentData);
};
exports.setCollectionDocumentAdmin = setCollectionDocumentAdmin;
const setStepAdmin = (firebase, teamID, workspaceID, stepCounter, stepData) => {
    return CommonOperations.getDocumentRefAdmin(firebase.firestore(), teamID, workspaceID, "steps").doc(stepCounter.toString()).set(stepData);
};
exports.setStepAdmin = setStepAdmin;
const setSubgraphAdmin = (firebase, teamID, workspaceID, subgraphID, subgraphData) => {
    return CommonOperations.getDocumentRefAdmin(firebase.firestore(), teamID, workspaceID, "folders").doc(subgraphID.toString()).set(subgraphData);
};
exports.setSubgraphAdmin = setSubgraphAdmin;
const deleteStepAdmin = (firebase, teamID, workspaceID, stepCounter) => {
    return CommonOperations.getDocumentRefAdmin(firebase.firestore(), teamID, workspaceID, "steps").doc(stepCounter.toString()).delete();
};
exports.deleteStepAdmin = deleteStepAdmin;
const setStepColumnsAdmin = (firebase, teamID, workspaceID, stepCounter, key, data) => {
    return CommonOperations.getDocumentRefAdmin(firebase.firestore(), teamID, workspaceID, "steps").doc(stepCounter.toString()).update({
        [key]: data
    });
};
exports.setStepColumnsAdmin = setStepColumnsAdmin;
/**
 * Our mutex lock so that multiple events targeting one workspace: eg. multiple users trying to kick off deploys and refreshes doesn't succeed and only the one of the request succeeds.
 * @param firestore
 * @param teamID
 * @param workspaceID
 * @param newStatus
 * @returns
 */
const ChangeWorkspaceStatus = (firestore, teamID, workspaceID, oldExpectedStatus, newStatus) => {
    const workspaceRef = (0, exports.getWorkspacesCollectionRefAdmin)(firestore, teamID).doc(workspaceID.toString());
    return firestore.runTransaction(transaction => {
        return transaction.get(workspaceRef)
            .then((workspaceDoc) => {
            if (!workspaceDoc.exists) {
                throw new Error("IWorkspace didn't exist");
            }
            const workspaceData = workspaceDoc.data();
            /**
             * in the event that the status is undefined: for old development workspaces... we pass and set status next
             * using an array here allows us to have multiple options eg. deploys can start if we are in failed or
             * waiting state, but refresh is waiting only
             */
            if (workspaceData.status && !oldExpectedStatus.includes(workspaceData.status)) {
                throw Common.CreateErrorWithCode(400, `Unable to start ${newStatus}`, `${workspaceData.name} is ${workspaceData.status}.`);
            }
            else if (workspaceData.devEnv) {
                throw Common.CreateErrorWithCode(400, `Unable to refresh ${workspaceData.name}`, `${workspaceData.name} is a development environment`);
            }
            transaction.update(workspaceRef, { status: newStatus });
        });
    }).then(() => {
        Logger.debugContext({ orgID: teamID, envID: workspaceID }, "Secured mutex lock, new status: ", newStatus);
    }).catch(error => {
        Logger.errorContext({ orgID: teamID, envID: workspaceID }, "Mutex lock failure: ", error);
        throw error;
    });
};
const SecureWorkspaceStatus = (firestore, teamID, workspaceID, newStatus) => {
    //refreshes can start from waiting or failedRefresh
    const oldStatuses = [Runner.EWorkspaceHealth.waiting, Runner.EWorkspaceHealth.failedRefresh];
    //deploys can start from both of those plus failedDeploy
    if (newStatus === Runner.EWorkspaceHealth.deploying) {
        oldStatuses.push(Runner.EWorkspaceHealth.failedDeploy);
    }
    return ChangeWorkspaceStatus(firestore, teamID, workspaceID, oldStatuses, newStatus);
};
exports.SecureWorkspaceStatus = SecureWorkspaceStatus;
const ReleaseWorkspaceStatus = (firestore, teamID, workspaceID, oldExpectedStatus) => {
    return ChangeWorkspaceStatus(firestore, teamID, workspaceID, [oldExpectedStatus], Runner.EWorkspaceHealth.waiting);
};
exports.ReleaseWorkspaceStatus = ReleaseWorkspaceStatus;
const MarkWorkspaceStatusFailed = (firestore, teamID, workspaceID, oldExpectedStatus, newFailedState) => {
    return ChangeWorkspaceStatus(firestore, teamID, workspaceID, [oldExpectedStatus], newFailedState);
};
exports.MarkWorkspaceStatusFailed = MarkWorkspaceStatusFailed;
const MarkWorkspaceStatusFailedWithCatch = (firestore, teamID, environmentID, expectedHealth, desiredHealth, logContext) => {
    return (0, exports.MarkWorkspaceStatusFailed)(firestore, teamID, environmentID, expectedHealth, desiredHealth)
        .catch(error => {
        RunnerBackendLogger.errorContext(logContext, "Error during status release: ", error);
    });
};
exports.MarkWorkspaceStatusFailedWithCatch = MarkWorkspaceStatusFailedWithCatch;
const useWorkspace = (workspaceID) => {
    const workspaces = (0, exports.useWorkspaces)();
    return (0, react_1.useMemo)(() => {
        return workspaces[workspaceID];
    }, [workspaces, workspaceID]);
};
exports.useWorkspace = useWorkspace;
const useReduxWorkspaces = () => {
    return (0, react_redux_1.useSelector)(state => state.workspaces);
};
exports.useReduxWorkspaces = useReduxWorkspaces;
// to supplement for older versions of workspace
const AddMissingDefaultWorkspaceValues = (workspace) => {
    const { oauthEnabled, oauthClientID, oauthClientSecret, connectionAccount, runTimeParameters } = workspace;
    return Immer.produce(workspace, draftWorkspace => {
        if (oauthEnabled === undefined)
            draftWorkspace.oauthEnabled = false;
        if (oauthClientID === undefined)
            draftWorkspace.oauthClientID = "";
        if (oauthClientSecret === undefined)
            draftWorkspace.oauthClientSecret = "";
        if (connectionAccount === undefined)
            draftWorkspace.connectionAccount = "";
        if (runTimeParameters === undefined)
            draftWorkspace.runTimeParameters = "";
    });
};
exports.AddMissingDefaultWorkspaceValues = AddMissingDefaultWorkspaceValues;
const useWorkspaces = () => {
    const allWorkspaces = (0, react_redux_1.useSelector)(state => state.workspaces.workspaces);
    //filter all workspaces to only valid ones 
    const validWorkspaces = (0, react_1.useMemo)(() => {
        const ret = {};
        Object.keys(allWorkspaces).forEach((workspaceID) => {
            if ((0, exports.isValidWorkspace)(allWorkspaces[workspaceID])) {
                ret[workspaceID] = (0, exports.AddMissingDefaultWorkspaceValues)(allWorkspaces[workspaceID]);
            }
        });
        return ret;
    }, [allWorkspaces]);
    return validWorkspaces;
};
exports.useWorkspaces = useWorkspaces;
const useWorkspacesIDs = () => {
    const workspaces = (0, exports.useWorkspaces)();
    const workspaceIDs = (0, react_1.useMemo)(() => Object.keys(workspaces), [workspaces]);
    return workspaceIDs;
};
exports.useWorkspacesIDs = useWorkspacesIDs;
const useNonDevWorkspaces = () => {
    const workspaces = (0, exports.useWorkspaces)();
    const nonDevWorkspaces = [];
    Object.entries(workspaces).forEach((entry, index) => {
        const [key, value] = entry;
        if (!value.devEnv)
            nonDevWorkspaces.push(value);
    });
    return nonDevWorkspaces;
};
exports.useNonDevWorkspaces = useNonDevWorkspaces;
const getCurrentWorkspace = (state) => {
    return Common.getValueSafe(state, ["workspaces", "currentWorkspace"], -1);
};
exports.getCurrentWorkspace = getCurrentWorkspace;
const listenWorkspaceChangeCb = (change, dispatch) => {
    const { type, doc } = change;
    if (type === "added") {
        dispatch(State.getCreateWorkspace(parseInt(doc.id), doc.data()));
    }
    else if (type === "modified") {
        dispatch(State.getUpdateWorkspace(parseInt(doc.id), doc.data()));
    }
    else if (type === "removed") {
        dispatch(State.getDeleteWorkspace(parseInt(doc.id)));
    }
    else {
        console.log("unknown update type", type);
        console.assert(false);
    }
};
exports.listenWorkspaceChangeCb = listenWorkspaceChangeCb;
const IsNewEnvironment = (workspaceID) => workspaceID === 0;
exports.IsNewEnvironment = IsNewEnvironment;
const IsFirstEnvironment = (workspaces) => !Object.keys(workspaces).length;
exports.IsFirstEnvironment = IsFirstEnvironment;
const GetNewEmptyWorkspace = (userID) => {
    return GetNewWorkspace({ createdBy: userID });
};
exports.GetNewEmptyWorkspace = GetNewEmptyWorkspace;
const GetNewDevWorkspace = (userID) => {
    WorkspaceLogger.infoContext({ userID }, "Getting new dev workspace");
    const workspaceInitializationData = {
        createdBy: userID,
        name: "DEV",
        devEnv: true
    };
    return GetNewWorkspace(workspaceInitializationData);
};
exports.GetNewDevWorkspace = GetNewDevWorkspace;
const TransactionWorkspaceSetter = (transaction) => {
    return (teamID, nextWorkspaceCounter, newWorkspaceObj) => {
        const newWorkspaceRef = (0, exports.getWorkspacesCollectionRef)(teamID).doc(nextWorkspaceCounter.toString());
        transaction.set(newWorkspaceRef, newWorkspaceObj);
        return Promise.resolve();
    };
};
exports.TransactionWorkspaceSetter = TransactionWorkspaceSetter;
const WorkspaceSetter = (teamID, nextWorkspaceCounter, newWorkspaceObj) => {
    return (0, exports.getWorkspacesCollectionRef)(teamID).doc(nextWorkspaceCounter.toString()).set(newWorkspaceObj);
};
exports.WorkspaceSetter = WorkspaceSetter;
const GetNewWorkspace = (workspaceInitData) => {
    return Object.assign(Object.assign({}, GetInitialWorkspaceInfo()), workspaceInitData);
};
const GetInitialWorkspaceInfo = () => {
    return {
        name: "",
        id: 0,
        createdAt: new Date(),
        createdBy: "",
        description: "",
        mappings: {},
        devEnv: false,
        status: Runner.EWorkspaceHealth.waiting,
        lastCommit: null,
        connectionAccount: "",
        oauthEnabled: false,
        oauthClientID: "",
        oauthClientSecret: "",
        runTimeParameters: ""
    };
};
/**
 *
 * @param firestore
 * @param teamID
 * @param workspaceName
 * @param createdByUserID
 * @returns
 */
const AddWorkspaceAdmin = (firestore, teamID, workspaceName, createdByUserID) => {
    let workspace = {
        id: 0,
        name: workspaceName,
        createdAt: new Date(),
        createdBy: createdByUserID,
        lastCommit: null,
        description: "",
        mappings: {},
        devEnv: false,
        status: Runner.EWorkspaceHealth.waiting,
        connectionAccount: "",
        runTimeParameters: ""
    };
    let newWorkspaceCounter = 0;
    return CommonOperations.getNextCounter(firestore, CommonOperations.getOrgDataDocRefAdmin(firestore, teamID), "workspaceCounter", 1)
        .then((workspaceCounter) => {
        newWorkspaceCounter = workspaceCounter;
        workspace.id = newWorkspaceCounter;
        const obj = Object.assign({}, workspace);
        return (0, exports.getWorkspacesCollectionRefAdmin)(firestore, teamID).doc(workspaceCounter.toString()).set(obj);
    }).then(() => {
        return newWorkspaceCounter;
    });
};
exports.AddWorkspaceAdmin = AddWorkspaceAdmin;
const updateWorkspaceDeploy = (firebase, teamID, workspaceID, workspaces /*WHAT IS THIS TYPE?*/, deployCommit) => {
    return (0, exports.getWorkspacesCollectionRefAdmin)(firebase.firestore(), teamID).doc(workspaceID.toString()).update({ deployedWorkspaces: workspaces, lastCommit: deployCommit });
};
exports.updateWorkspaceDeploy = updateWorkspaceDeploy;
// use this to update the workspace health status from the scheduler service
const UpdateWorkspaceAdmin = (firestore, teamID, workspaceID, update) => {
    return (0, exports.getWorkspacesCollectionRefAdmin)(firestore, teamID).doc(workspaceID.toString()).set(update, { merge: true });
};
exports.UpdateWorkspaceAdmin = UpdateWorkspaceAdmin;
const GetWorkspaces = (teamID) => {
    return (0, exports.getWorkspacesCollectionRef)(teamID).get().then((querySnapShot) => {
        const workspaceIDs = [];
        querySnapShot.forEach((doc) => {
            workspaceIDs.push(Object.assign({}, doc.data()));
        });
        return workspaceIDs;
    });
};
exports.GetWorkspaces = GetWorkspaces;
const GetWorkspacesAdmin = (firestore, teamID) => {
    return (0, exports.getWorkspacesCollectionRefAdmin)(firestore, teamID).get().then((querySnapShot) => {
        const workspaceIDs = [];
        querySnapShot.forEach((doc) => {
            workspaceIDs.push(Object.assign({}, doc.data()));
        });
        return workspaceIDs;
    });
};
exports.GetWorkspacesAdmin = GetWorkspacesAdmin;
const IsWorkspaceNameUnique = (name, workspaceList) => {
    return !workspaceList.some(workspace => workspace.name === name);
};
exports.IsWorkspaceNameUnique = IsWorkspaceNameUnique;
const useGetDeploymentSettingsState = () => {
    const deployConnections = (0, react_redux_1.useSelector)(state => state.deploymentCredentials);
    return deployConnections;
};
exports.useGetDeploymentSettingsState = useGetDeploymentSettingsState;
const getCurrentWorkspaceID = (state) => {
    return state.workspaces.currentWorkspace;
};
exports.getCurrentWorkspaceID = getCurrentWorkspaceID;
const GetRunTimeParametersFromWorkspacesSafe = (workspaces, workspaceID) => {
    return Common.getValueSafe(workspaces, ["workspaces", workspaceID, "runTimeParameters"], "");
};
exports.GetRunTimeParametersFromWorkspacesSafe = GetRunTimeParametersFromWorkspacesSafe;
const getCurrentWorkspaceMappings = (state) => {
    var _a;
    const currentWorkspaceID = (0, exports.getCurrentWorkspaceID)(state);
    const currentLocations = (_a = state.workspaces.workspaces[currentWorkspaceID]) === null || _a === void 0 ? void 0 : _a.mappings;
    return currentLocations ? currentLocations : {};
};
exports.getCurrentWorkspaceMappings = getCurrentWorkspaceMappings;
const GetAllWorkspaceDataFromGitCommit = (fsRequirements, commitOID, environmentID) => {
    if (commitOID) {
        return Common.timedLoggerPromise(`GetAllWorkspaceDataFromGitCommit when a commitOID exists`, { envID: environmentID === null || environmentID === void 0 ? void 0 : environmentID.toString() }, Logging.LoggingArea.CD_2676Logger, GetAllWorkspaceDataFromGitCommitInternal(fsRequirements, commitOID, environmentID));
    }
    else {
        //if there's no commitOID, then just read the files using fs.readFile....
        return Common.timedLoggerPromise("GetAllWorkspaceDataFromGitCommit when a commitOID does not exist", { envID: environmentID === null || environmentID === void 0 ? void 0 : environmentID.toString() }, Logging.LoggingArea.CD_2676Logger, GetAllWorkspaceDataFromFileSystemDirectory(fsRequirements, environmentID));
    }
};
exports.GetAllWorkspaceDataFromGitCommit = GetAllWorkspaceDataFromGitCommit;
const PromiseifiedReadDir = (filesystem, dir, environmentID) => {
    BurncoCD_2676Logger.infoContext({ envID: environmentID === null || environmentID === void 0 ? void 0 : environmentID.toString() }, `PromiseifiedReadDir executes at: ${new Date()}`);
    return Common.timedLoggerPromise(`PromiseifiedReadDir with filesystem: ${filesystem} and dir: ${dir}`, { envID: environmentID === null || environmentID === void 0 ? void 0 : environmentID.toString() }, Logging.LoggingArea.CD_2676Logger, new Promise((resolve, reject) => {
        filesystem.readdir(dir, (err, files) => {
            if (err) {
                reject(err);
            }
            else {
                resolve(files);
            }
        });
    }));
};
const PromiseifiedReadFile = (filesystem, dir, file, environmentID) => {
    return new Promise((resolve, reject) => {
        BurncoCD_2676Logger.infoContext({ envID: environmentID === null || environmentID === void 0 ? void 0 : environmentID.toString() }, `PromiseifiedReadFile executes at: ${new Date()}`);
        /**
         * utf8 must be used here, if it's utf-8, the lightning-fs browser fs throws an
         * error and refuses to read the files
         */
        filesystem.readFile(`${dir}/${file}`, "utf8", (err, content) => {
            if (err) {
                reject(err);
            }
            else {
                resolve(content);
            }
        });
    });
};
/**
 * Wrapper function for getAllWorkspaceDataFromGitCommit for the node fs when there's no menu to choose the target commitOID
 * @param fsRequirements
 * @returns
 */
const GetAllWorkspaceDataFromFileSystemDirectory = (fsRequirements, environmentID) => {
    return Common.timedLoggerPromise("GetAllWorkspaceDataFromFileSystemDirectory", { envID: environmentID === null || environmentID === void 0 ? void 0 : environmentID.toString() }, Logging.LoggingArea.CD_2676Logger, new Promise((resolve, reject) => {
        let returnSteps = {}, returnWorkspaces = {}, returnFolders = {}, returnJobs = {}, returnMacros = {}, returnStepTypes = {}, returnLocations = {};
        PromiseifiedReadDir(fsRequirements.fs, `${fsRequirements.dir}/nodes`, environmentID)
            .then(nodeFileNames => {
            const nodePromises = nodeFileNames.map(file => {
                return Common.timedLoggerPromise(`file: ${file}`, { envID: environmentID === null || environmentID === void 0 ? void 0 : environmentID.toString() }, Logging.LoggingArea.CD_2676Logger, PromiseifiedReadFile(fsRequirements.fs, `${fsRequirements.dir}/nodes`, file, environmentID));
            });
            return Promise.all(nodePromises);
        }).then(nodeYamlContent => {
            const { steps } = VersionControlActions.getReduxStateFromYaml(nodeYamlContent, "");
            returnSteps = steps;
        })
            .then(() => {
            return Common.timedLoggerPromise(`PromiseifiedReadFile data.yml`, { envID: environmentID === null || environmentID === void 0 ? void 0 : environmentID.toString() }, Logging.LoggingArea.CD_2676Logger, PromiseifiedReadFile(fsRequirements.fs, fsRequirements.dir, "data.yml", environmentID));
        })
            .then(dataYaml => {
            const { workspaces, folders, jobs, macros, stepTypes, mappings } = VersionControlActions.getReduxStateFromYaml([], dataYaml);
            returnWorkspaces = workspaces;
            returnFolders = folders;
            returnJobs = jobs;
            returnMacros = macros;
            returnStepTypes = stepTypes;
            returnLocations = mappings;
        })
            .then(() => {
            resolve({ steps: returnSteps, workspaces: returnWorkspaces, folders: returnFolders, jobs: returnJobs, macros: returnMacros, stepTypes: returnStepTypes, locations: returnLocations });
        }).catch(error => {
            Logger.error("Error during GetAllWorkspaceDataFromFileSystemDirectory: ", error);
            reject(error);
        });
    }));
};
/**
 * need wrapper for using the node fs where we just use the current state of the targeted dir (current history commit)
 * @param fsRequirements
 * @param commit
 * @returns
 */
const GetAllWorkspaceDataFromGitCommitInternal = (fsRequirements, commit, environmentID) => {
    BurncoCD_2676Logger.infoContext({ envID: environmentID === null || environmentID === void 0 ? void 0 : environmentID.toString() }, `GetAllWorkspaceDataFromGitCommitInternal executes at: ${new Date()}`);
    return new Promise((resolve, reject) => {
        // take in commit, listFiles using the commit to know what files to read
        let returnSteps = {}, returnWorkspaces = {}, returnFolders = {}, returnJobs = {}, returnMacros = {}, returnStepTypes = {}, returnLocations = {};
        Common.timedLoggerPromise("listing git files", {}, Logging.LoggingArea.Git, GitOperations.GitListFiles(fsRequirements, commit))
            .then(gitFiles => {
            //remove .gitignore and data.yml, just read nodes
            const nodePromises = gitFiles.filter(name => name.includes("node")).map(file => {
                return GitOperations.ReadFileFromCommit(fsRequirements, commit, file);
            });
            return Common.timedLoggerPromise("reading node files' data", {}, Logging.LoggingArea.Git, Promise.all(nodePromises));
        }).then(nodeYamlContent => {
            const { steps } = VersionControlActions.getReduxStateFromYaml(nodeYamlContent, "");
            returnSteps = steps;
        }).then(() => {
            return Common.timedLoggerPromise(`reading file "data.yaml"`, { envID: environmentID === null || environmentID === void 0 ? void 0 : environmentID.toString() }, Logging.LoggingArea.CD_2676Logger, GitOperations.ReadFileFromCommit(fsRequirements, commit, "data.yml"));
        })
            .then(dataYaml => {
            const { workspaces, folders, jobs, macros, stepTypes, mappings } = VersionControlActions.getReduxStateFromYaml([], dataYaml);
            returnWorkspaces = workspaces;
            returnFolders = folders;
            returnJobs = jobs;
            returnMacros = macros;
            returnStepTypes = stepTypes;
            returnLocations = mappings;
        })
            .then(() => {
            resolve({ steps: returnSteps, workspaces: returnWorkspaces, folders: returnFolders, jobs: returnJobs, macros: returnMacros, stepTypes: returnStepTypes, locations: returnLocations });
        }).catch(error => {
            Logger.error("Error during GetAllWorkspaceDataFromGitCommitInternal: ", error);
            reject(error);
        });
    });
};
//helper function for getAllWorkspaceDataFromFirebase
const getCollectionFromRef = (collectionRef, workspaceID) => {
    return Common.timedLoggerPromise("getCollectionFromRef", { envID: workspaceID === null || workspaceID === void 0 ? void 0 : workspaceID.toString() }, Logging.LoggingArea.CD_2676Logger, new Promise((resolve, reject) => {
        let collection = {};
        collectionRef.get().then((querySnapShot) => {
            BurncoCD_2676Logger.infoContext({ envID: workspaceID === null || workspaceID === void 0 ? void 0 : workspaceID.toString() }, `collectionRef.get() executes at: ${new Date()}`);
            querySnapShot.forEach((doc) => {
                collection[doc.id] = doc.data();
            });
        }).then(() => {
            BurncoCD_2676Logger.infoContext({ envID: workspaceID === null || workspaceID === void 0 ? void 0 : workspaceID.toString() }, `all querySnapShot doc.data() resolve at: ${new Date()}`);
            resolve(collection);
        }).catch(error => {
            reject(error);
        });
    }));
};
exports.getCollectionFromRef = getCollectionFromRef;
/**
 * UPDATE to grab the workspace/users collection so we know what users have connection info
 * @param firestore
 * @param teamID
 * @param workspaceID
 * @returns
 */
const getAllWorkspaceDataFromFirebase = (firestore, teamID, workspaceID) => {
    return Common.timedLoggerPromise(`getAllWorkspaceDataFromFirebase teamID: ${teamID}`, { envID: workspaceID }, Logging.LoggingArea.CD_2676Logger, new Promise((resolve, reject) => {
        const refCollectionArray = [CommonOperations.getDocumentRefAdmin(firestore, teamID, workspaceID, "steps"),
            CommonOperations.getDocumentRefAdmin(firestore, teamID, workspaceID, "folders"),
            CommonOperations.getDocumentRefAdmin(firestore, teamID, workspaceID, "jobs"),
            CommonOperations.getDocumentRefAdmin(firestore, teamID, workspaceID, "macros"),
            CommonOperations.getDocumentRefAdmin(firestore, teamID, workspaceID, "templates"),
            CommonOperations.getDocumentRefAdmin(firestore, teamID, workspaceID, "users")];
        let returnSteps = {}, returnFolders = {}, returnWorkspaces = {}, returnJobs = {}, returnMacros = {}, returnStepTypes = {}, returnUserConnections = {};
        const collectionPromises = refCollectionArray.map(ref => {
            return (0, exports.getCollectionFromRef)(ref, workspaceID);
        });
        Promise.all(collectionPromises).then(collections => {
            collections.forEach(collection => {
                var _a;
                const firstObject = collection[Object.keys(collection)[0]] || {}; //in the event of an empty collection (the most likely case is folders)
                if (firstObject === null || firstObject === void 0 ? void 0 : firstObject.hasOwnProperty("sqlType")) { //steps have sqlType unique to them
                    // makes steps match redux version
                    returnSteps = Object.keys(collection).reduce((accum, stepCounter) => {
                        return Object.assign(Object.assign({}, accum), { [stepCounter]: { operation: collection[stepCounter], stepCounter: stepCounter, name: collection[stepCounter]["name"] } });
                    }, {});
                }
                else if (firstObject.hasOwnProperty("folderCounter")) { //folderCounter is unique to subgraphs
                    returnFolders = collection;
                }
                else if (firstObject["subgraphs"] && firstObject["steps"]) { //jobCounter is unique to jobs
                    returnJobs = collection;
                }
                else if (firstObject.hasOwnProperty("macroString")) { // macroString is unique to macros
                    returnMacros = collection;
                }
                else if ((_a = firstObject.metadata) === null || _a === void 0 ? void 0 : _a.templates) { // metadata.templates is unique to stepTypes
                    returnStepTypes = collection;
                }
                else if (firstObject.hasOwnProperty("connectionType")) {
                    returnUserConnections = { [workspaceID]: collection };
                }
            });
        }).then(() => {
            return (0, exports.getWorkspacesCollectionRefAdmin)(firestore, teamID).doc(workspaceID.toString()).get();
        }).then((workspaceDataSnapshot) => {
            const workspaceData = workspaceDataSnapshot.data();
            if (workspaceData) {
                returnWorkspaces = workspaceData.deployedWorkspaces;
                if (!returnWorkspaces) {
                    returnWorkspaces = {};
                }
                if (!returnWorkspaces[workspaceID]) {
                    returnWorkspaces[workspaceID] = {};
                }
                returnWorkspaces[workspaceID].runTimeParameters = workspaceData.runTimeParameters || "";
            }
            const logContext = {
                envID: workspaceID,
                orgID: teamID
            };
            Logger.debugContext(logContext, "return steps", returnSteps);
            Logger.debugContext(logContext, "got deployed workspaces from workspaceID", workspaceID, returnWorkspaces);
            resolve({ steps: returnSteps, workspaces: { workspaces: returnWorkspaces }, folders: returnFolders, jobs: returnJobs, macros: returnMacros, stepTypes: returnStepTypes, userConnections: returnUserConnections });
        }).catch(error => {
            reject(error);
        });
    }));
};
exports.getAllWorkspaceDataFromFirebase = getAllWorkspaceDataFromFirebase;
var WorkspaceEditType;
(function (WorkspaceEditType) {
    WorkspaceEditType["addedTable"] = "addedTable";
    WorkspaceEditType["clonedTable"] = "clonedTable";
    WorkspaceEditType["alteredTable"] = "alteredTable";
    WorkspaceEditType["deletedTable"] = "deletedTable";
    WorkspaceEditType["addedColumn"] = "addedColumn";
    WorkspaceEditType["alteredColumn"] = "alteredColumn";
    WorkspaceEditType["renamedColumn"] = "renamedColumn";
    WorkspaceEditType["deletedColumn"] = "deletedColumn";
})(WorkspaceEditType = exports.WorkspaceEditType || (exports.WorkspaceEditType = {}));
// iterate through the keys of the columns, if they don't match, create a new piece of data 
const CreateDataForNodeMetadataForColumnAlters = (original, updated) => {
    const edits = [];
    for (const key in original) {
        if (columnKeys.has(key)) {
            const originalValue = original[key];
            const updatedValue = updated[key];
            if (originalValue !== updatedValue) {
                const dataObject = { name: original.name }; //use the original name, since alters happen before renames, we want to work with the original name
                switch (key) {
                    case "nullable":
                        dataObject["nullable"] = updatedValue;
                        break;
                    case "dataType":
                        dataObject["dataType"] = updatedValue;
                        break;
                    case "description":
                        dataObject["description"] = updatedValue;
                        break;
                    default:
                        break;
                }
                //dont want to push on an empty object, since I'm unsure if I need all the columnKeys in the set/I'm unsure what to do with the ones I'm not already working with in the switch statement above
                if (Object.keys(dataObject).length > 1) {
                    edits.push(dataObject);
                }
            }
        }
    }
    return edits;
};
const CreateWorkspaceEditsForSteps = (stepCounters, type, workspaceData, workspaceMappings, platformType, workspaceID, stepTypesParsed, runTimeParameters) => {
    const editsCollection = {};
    stepCounters.forEach(stepCounter => {
        const currentNode = workspaceData.steps[stepCounter];
        const currentStep = currentNode.operation;
        /**
         * new stepTypes will overwrite old stepTypes when spreading like below,
         * old stepTypes will fill in the gaps for missing stepTypes  in the case
         * of a first (successful) deploy, or other cases where workspaceData.stepTypes
         * is {} or contains stepTypes needed at deploy that stepTypesParsed does not contain
         */
        const newAndOldStepTypes = Object.assign(Object.assign({}, workspaceData.stepTypes), stepTypesParsed);
        const deployStrategy = AdvancedDeployOperations.GetStepDeployStrategyFromStepTypesParsed(stepCounter, workspaceData.steps, newAndOldStepTypes); // gathered from the step type
        const isDeployStrategyAdvanced = deployStrategy === AdvancedDeployOperations.EDeployStrategy.advanced;
        const workspaceDataWithID = Object.assign(Object.assign({}, workspaceData.workspaces), { currentWorkspace: workspaceID });
        const workspaceDataWithMappingsFromNode = CreateAndEmbedMappingsFromNode(currentStep.locationID, currentStep.database, currentStep.schema, currentStep.locationName, workspaceID, workspaceDataWithID);
        const workspaceEdits = [];
        let message;
        let sql;
        let dataForNodeMetadata = {};
        let invalid = false;
        let hydratedMetadata = {};
        const hydrateMetadataForWorkspaceEdit = () => {
            return MetadataHydration.hydrateMetadata(currentStep, stepCounter, workspaceData.steps, newAndOldStepTypes, workspaceDataWithMappingsFromNode, null, runTimeParameters);
        };
        switch (type) {
            case WorkspaceEditType.addedTable:
                if (isDeployStrategyAdvanced) {
                    sql = StepTypes.getStepTypeTemplatesSafe(newAndOldStepTypes, currentStep.sqlType).create.templateString; // all custom advanced deploy logic lives under create
                    message = `Advanced Deployment: Created node ${currentStep.name}`;
                    hydratedMetadata = AdvancedDeployOperations.GetAddTableAdvancedHydratedMetadata(currentStep, stepCounter, null, workspaceMappings, workspaceData.steps, newAndOldStepTypes, workspaceDataWithMappingsFromNode, runTimeParameters);
                }
                else {
                    if (currentStep.sqlType === Runner.ESQLType.source) {
                        message = `Adding metadata for ${currentStep.name}`;
                        sql = RunnerOperations.GetTemplateStringFromStepDetails(newAndOldStepTypes, Runner.ESQLType.source, RunnerOperations.OperationTypes.run);
                    }
                    else {
                        message = `Created node ${currentStep.name}`;
                        sql = Templates.getCreateTableTemplate(newAndOldStepTypes, currentStep.sqlType);
                    }
                    hydratedMetadata = hydrateMetadataForWorkspaceEdit();
                }
                break;
            case WorkspaceEditType.deletedTable:
                if (isDeployStrategyAdvanced) {
                    sql = StepTypes.getStepTypeTemplatesSafe(newAndOldStepTypes, currentStep.sqlType).create.templateString;
                    message = `Advanced Deployment: Deleted node ${currentStep.name}`;
                    hydratedMetadata = AdvancedDeployOperations.GetDropTableAdvancedHydratedMetadata(currentStep, stepCounter, null, workspaceMappings, workspaceData, workspaceDataWithMappingsFromNode, newAndOldStepTypes, runTimeParameters);
                    if (!sql) {
                        sql = "";
                        message = `Advanced Deployment: No SQL available for ${currentStep.name}`;
                    }
                }
                else {
                    message = `Deleted node ${currentStep.name}`;
                    //temporary patch, also need to create and tack on the oldFullyQualifiedName for all deletes?
                    let mappings;
                    //in the odd event you have deployed steps but haven't deployed mappings successfully yet, and you start deleting things that did get deployed during the last few failures, catch and skip to next edit
                    try {
                        mappings = workspaceData.workspaces.workspaces[workspaceID].mappings;
                        dataForNodeMetadata["oldName"] = currentStep.name;
                        dataForNodeMetadata["oldLocation"] = (0, exports.CreateOldLocationName)(currentStep, mappings);
                    }
                    catch (error) {
                        console.error("Mappings for node dont exist"); //delete metadata from firestore?
                        invalid = true;
                    }
                    sql = Templates.getDeleteTableTemplate(platformType);
                    // handle SOURCE tables or invalid (error w/getting mappings) table
                    if (currentStep.sqlType === Runner.ESQLType.source || invalid) {
                        sql = "";
                        message = `Deleting metadata for ${currentStep.name}`;
                        dataForNodeMetadata = {};
                        // handle VIEW
                    }
                    else if ((currentStep.materializationType && currentStep.materializationType === Templates.materializationTypes.view) || currentStep.sqlType === Runner.ESQLType.view) {
                        sql = Templates.getDeleteViewTemplate(platformType);
                        // handle all else
                    }
                    else {
                        /**
                         * If a COALESCE_INTERNAL_TABLE is leftover in snowflake because edits failed for whatever reason,
                         * and the COALESCE_INTERNAL_TABLE is not dropped during a proceeding rename/edit,
                         * then the COALESCE_INTERNAL_TABLE will remain on snowflake.
                         * If in this state the table is neither renamed/edited, but instead deleted,
                         * then this dropCloneEdit will ensure that the errant COALESCE_INTERNAL_TABLE will be cleaned up
                         *
                         * this does not apply to advanced deploy - maintainance is entirely dependent on the
                         * user's own custom create script
                         */
                        const dropCloneEdit = CreateWorkspaceEdit(stepCounter, message, type);
                        const oldName = dataForNodeMetadata["oldName"] + Templates.deploySuffix;
                        const oldLocation = dataForNodeMetadata["oldLocation"];
                        dropCloneEdit.dataForNodeMetadata = { oldName, oldLocation };
                        dropCloneEdit.description = `Cleaning any stale data`;
                        dropCloneEdit.sql = Templates.getDeleteTableTemplate(platformType);
                        workspaceEdits.push(dropCloneEdit);
                    }
                }
                break;
            default:
                Common.assert(Logging.LoggingArea.Deploy, false, `Invalid WS Edit type when creating workspace edits - ${type} is not a valid WS Edit type`);
                break;
        }
        const newEdit = Object.assign(Object.assign({}, CreateWorkspaceEdit(stepCounter, message, type)), { sql,
            dataForNodeMetadata });
        workspaceEdits.push(newEdit);
        Common.assert(Logging.LoggingArea.Deploy, !!hydratedMetadata, `hydratedMetadata was ${hydratedMetadata} when creating workspace edits`);
        editsCollection[stepCounter] = {
            operation: currentStep,
            workspaceEdits,
            nodeMetadata: Object.assign({ operationName: currentStep.name, mapping: workspaceMappings, destinationName: currentStep.locationName }, hydratedMetadata)
        };
    });
    return editsCollection;
};
/**
 * Helper function that is used to add and override workspace mappings based on the data from a deployed node
 * rather that the inherint data from the deployWorkspaceData (mappings)
 * @param locationID
 * @param database
 * @param schema
 * @param locationName
 * @param workspaceID
 * @param allWorkspaceData
 * @returns
 */
const CreateAndEmbedMappingsFromNode = (locationID, database, schema, locationName, workspaceID, allWorkspaceData) => {
    const location = { database, schema, id: locationID, locationName };
    const allLocations = { [locationID]: location };
    if (!!database && !!schema) {
        return Immer.produce(allWorkspaceData, draftState => {
            if (!draftState.workspaces[workspaceID]) {
                draftState.workspaces[workspaceID] = {};
            }
            draftState.workspaces[workspaceID].mappings = allLocations;
        });
    }
    else {
        return allWorkspaceData;
    }
};
const CreateWorkspaceEdit = (stepCounter, description, type) => {
    const newEdit = {
        stepCounter: stepCounter,
        description: description,
        sql: "",
        dataForNodeMetadata: {},
        type: type,
        updateFirestore: false
    };
    return newEdit;
};
const CreateWorkspaceEditForDeleteTable = (stepCounter, locationData, platformType) => {
    const edits = [];
    const newEdit = CreateWorkspaceEdit(stepCounter, "Deleting table", WorkspaceEditType.deletedTable);
    newEdit.sql = Templates.getDeleteTableTemplate(platformType);
    newEdit.dataForNodeMetadata = locationData;
    edits.push(newEdit);
    return edits;
};
exports.CreateWorkspaceEditForDeleteTable = CreateWorkspaceEditForDeleteTable;
const CreateWorkspaceEditForSwapCloneTable = (stepCounter, locationData, platformType) => {
    const edits = [];
    const newEdit = CreateWorkspaceEdit(stepCounter, "Swapping cloned table", WorkspaceEditType.clonedTable);
    newEdit.sql = Templates.getSwapClonedTableTemplate(platformType);
    newEdit.dataForNodeMetadata = locationData;
    edits.push(newEdit);
    return edits;
};
exports.CreateWorkspaceEditForSwapCloneTable = CreateWorkspaceEditForSwapCloneTable;
const CreateWorkspaceEditForCloneTable = (stepCounter, locationData, platformType) => {
    const edits = [];
    const newEdit = CreateWorkspaceEdit(stepCounter, "Cloning table", WorkspaceEditType.clonedTable);
    newEdit.sql = Templates.getCloneTableTemplate(platformType);
    newEdit.dataForNodeMetadata = locationData;
    edits.push(newEdit);
    return edits;
};
exports.CreateWorkspaceEditForCloneTable = CreateWorkspaceEditForCloneTable;
const CreateWorkspaceEditForTableChange = (stepCounter, locationData, platformType) => {
    const edits = [];
    const newEdit = CreateWorkspaceEdit(stepCounter, "Renaming table", WorkspaceEditType.alteredTable);
    newEdit.sql = Templates.getRenameTableTemplate(platformType);
    newEdit.dataForNodeMetadata = locationData;
    edits.push(newEdit);
    return edits;
};
const CreateWorkspaceEditForRemoveDeploySuffix = (stepCounter, originalStep, mappings, platformType) => {
    const edits = [];
    const locationID = originalStep.operation.locationID;
    let oldSchema = "";
    let oldDatabase = "";
    try {
        oldSchema = mappings[locationID].schema;
        oldDatabase = mappings[locationID].database;
    }
    catch (error) {
        Logger.error(`Mappings for locationID: ${locationID} do not exist`);
        //swallow
    }
    const data = {
        oldName: originalStep.operation.name,
        oldLocation: `"${oldDatabase}"."${oldSchema}".`
    };
    const newEdit = CreateWorkspaceEdit(stepCounter, "Removing deploy suffix from table", WorkspaceEditType.alteredTable);
    newEdit.sql = Templates.getRemoveDeploySuffixTemplate(platformType);
    newEdit.dataForNodeMetadata = data;
    edits.push(newEdit);
    return edits;
};
exports.CreateWorkspaceEditForRemoveDeploySuffix = CreateWorkspaceEditForRemoveDeploySuffix;
const CreateWorkspaceEditForColumnDeletes = (originalStep, locationData, columnCounters, stepCounter, platformType) => {
    const workspaceEdits = [];
    if (columnCounters.length) {
        const columnCountersSet = new Set(columnCounters);
        const primaryEdit = CreateWorkspaceEdit(stepCounter, "Deleted Columns: ", WorkspaceEditType.deletedColumn);
        primaryEdit.dataForNodeMetadata = Object.assign(Object.assign({}, locationData), { deletedColumns: [] });
        primaryEdit.sql = Templates.getDeleteColumnTemplate(platformType);
        const searchColumns = originalStep.operation.metadata.columns;
        searchColumns.forEach(column => {
            if (columnCountersSet.has(column.columnReference.columnCounter)) {
                //get column name
                const columnName = column.name;
                primaryEdit.description += columnName + " \n";
                //drops: [ {name: string},...]
                primaryEdit.dataForNodeMetadata.deletedColumns.push({ name: columnName });
            }
        });
        workspaceEdits.push(primaryEdit);
    }
    return workspaceEdits;
};
const CreateWorkspaceEditForColumnAlters = (originalStep, updatedStep, locationData, columnCounters, stepCounter, platformType) => {
    const edits = [];
    const columnCounterSet = new Set(columnCounters);
    const primaryEdit = CreateWorkspaceEdit(stepCounter, "Alters Columns", WorkspaceEditType.alteredColumn);
    primaryEdit.sql = Templates.getAlterColumnTemplate(platformType);
    primaryEdit.dataForNodeMetadata = Object.assign(Object.assign({}, locationData), { alteredColumns: [] });
    originalStep.operation.metadata.columns.forEach((column) => {
        //reduce hits by using the set
        if (columnCounterSet.has(column.columnReference.columnCounter)) {
            //should be guaranteed to not be undefined, but check anyway for safety
            const gitColumn = updatedStep.operation.metadata.columns.find(searchColumn => searchColumn.columnReference.columnCounter === column.columnReference.columnCounter);
            if (gitColumn) {
                let newData = CreateDataForNodeMetadataForColumnAlters(column, gitColumn); //need semi colon otherwise the linter thinks I'm calling this line as a function with the next line as arguments
                primaryEdit.dataForNodeMetadata.alteredColumns.push(...newData);
            }
        }
    });
    //only add the edit if the dataForNodeMetadata got filled with at least one item
    if (primaryEdit.dataForNodeMetadata.alteredColumns.length) {
        edits.push(primaryEdit);
    }
    return edits;
};
const CreateWorkspaceEditsForColumnRenames = (originalStep, updatedStep, locationData, columnCounters, stepCounter, platformType) => {
    const edits = [];
    const columnCounterSet = new Set(columnCounters);
    originalStep.operation.metadata.columns.forEach((column) => {
        //reduce finding operations :)
        if (columnCounterSet.has(column.columnReference.columnCounter)) {
            //should always succeed based on the columnCounterSet condition
            const gitColumn = updatedStep.operation.metadata.columns.find(searchColumn => searchColumn.columnReference.columnCounter === column.columnReference.columnCounter);
            if (gitColumn && gitColumn.name !== column.name) {
                const primaryEdit = CreateWorkspaceEdit(stepCounter, `Renamed column: ${column.name} to ${gitColumn.name}`, WorkspaceEditType.renamedColumn);
                primaryEdit.sql = Templates.getRenameColumnTemplate(platformType);
                const data = { oldColumn: column, newColumn: gitColumn };
                primaryEdit.dataForNodeMetadata = Object.assign(Object.assign({}, data), locationData);
                edits.push(primaryEdit);
            }
        }
    });
    return edits;
};
const CreateStringQuotedLocation = (database, schema) => {
    if (!database || !schema) {
        throw new Error("Database or schema are empty or undefined");
    }
    return `"${database}"."${schema}".`;
};
const CreateOldLocationName = (originalOperation, originalLocationData) => {
    let database;
    let schema;
    try {
        if (!!originalOperation.database && !!originalOperation.schema) {
            database = originalOperation.database;
            schema = originalOperation.schema;
        }
        else {
            database = originalLocationData[originalOperation.locationID].database || undefined;
            schema = originalLocationData[originalOperation.locationID].schema || undefined;
        }
        return CreateStringQuotedLocation(database, schema);
    }
    catch (error) {
        Logger.error(error);
        throw error;
    }
};
exports.CreateOldLocationName = CreateOldLocationName;
const CreateWorkspaceEditForColumnAdditions = (updatedStep, locationData, columnCounters, stepCounter, platformType) => {
    const workspaceEdits = [];
    const columnCounterSet = new Set(columnCounters);
    //iterate through all columns, if the column counter is in the set, add to the edit
    if (columnCounters.length) {
        const columns = updatedStep.operation.metadata.columns;
        const primaryEdit = CreateWorkspaceEdit(stepCounter, "Created column(s): ", WorkspaceEditType.addedColumn);
        primaryEdit.sql = Templates.getAddColumnTemplate(platformType);
        primaryEdit.dataForNodeMetadata = Object.assign(Object.assign({}, locationData), { addedColumns: [] });
        columns.forEach((column, index) => {
            if (index === 0) {
                primaryEdit.stepCounter = column.columnReference.stepCounter;
            }
            if (columnCounterSet.has(column.columnReference.columnCounter)) {
                const { name, dataType, nullable, description, defaultValue } = column;
                const dataItem = { name, dataType, nullable, description, defaultValue };
                primaryEdit.dataForNodeMetadata.addedColumns.push(dataItem);
            }
        });
        workspaceEdits.push(primaryEdit);
    }
    return workspaceEdits;
};
const columnKeys = new Set(["nullable", "dataType", "description", "defaultValue"]); // how many of these do I acutally care about/can affect?
const ResetNodeDataBaseAndSchema = (node) => {
    return Immer.produce(node, draftState => {
        draftState.operation.database = "";
        draftState.operation.schema = "";
    });
};
const DetermineIfTwoNodesHaveADifference = (node1, node2) => {
    //strip out replace database/schema with "" to ensure deployed nodes don't always trigger a diff
    const cleanNode1 = ResetNodeDataBaseAndSchema(node1);
    const cleanNode2 = ResetNodeDataBaseAndSchema(node2);
    return !!Diff.diff(cleanNode1, cleanNode2);
};
const CreateWorkspaceEditsForEditedSteps = (stepCounters, originalWorkspaceData, newWorkspaceData, workspaceID, platformType, stepTypesParsed, runTimeParameters) => {
    var _a, _b;
    //set of column names that we track changes for
    let editsCollection = {};
    const newWorkspaceMappings = ((_a = newWorkspaceData.workspaces.workspaces[workspaceID]) === null || _a === void 0 ? void 0 : _a.mappings) || {};
    const oldWorkspaceMappings = ((_b = originalWorkspaceData.workspaces.workspaces[workspaceID]) === null || _b === void 0 ? void 0 : _b.mappings) || {};
    const createWorkspaceDataWithMappings = (stepOperation, workspaceData) => {
        const workspaceDataWithID = Object.assign(Object.assign({}, workspaceData.workspaces), { currentWorkspace: workspaceID });
        return CreateAndEmbedMappingsFromNode(stepOperation.locationID, stepOperation.database, stepOperation.schema, stepOperation.locationName, workspaceID, workspaceDataWithID);
    };
    stepCounters.forEach(stepCounter => {
        const updatedStep = newWorkspaceData.steps[stepCounter];
        const originalStep = originalWorkspaceData.steps[stepCounter];
        const updatedOperation = updatedStep.operation;
        const originalOperation = originalStep.operation;
        const deployStrategy = AdvancedDeployOperations.GetStepDeployStrategyFromStepTypesYAML(stepCounter, newWorkspaceData.steps, newWorkspaceData.stepTypes);
        const isDeployStrategyAdvanced = deployStrategy === AdvancedDeployOperations.EDeployStrategy.advanced;
        const updateEditsCollection = (workspaceEdits, hydratedMetadata) => {
            editsCollection = Immer.produce(editsCollection, (draftState) => {
                draftState[stepCounter] = {
                    operation: updatedOperation,
                    workspaceEdits,
                    nodeMetadata: Object.assign({ operationName: updatedOperation.name, mapping: newWorkspaceMappings, destinationName: updatedOperation.locationName }, hydratedMetadata)
                };
            });
        };
        if (isDeployStrategyAdvanced) {
            const advancedDeployHydratedMetadata = AdvancedDeployOperations.GetAlterTableAdvancedHydratedMetadata(originalOperation, updatedOperation, originalWorkspaceData, newWorkspaceData, stepCounter, null, oldWorkspaceMappings, newWorkspaceMappings, createWorkspaceDataWithMappings(originalOperation, originalWorkspaceData), createWorkspaceDataWithMappings(updatedOperation, newWorkspaceData), runTimeParameters);
            const message = `Advanced Deployment: Altered node ${updatedStep.operation.name}`;
            const newEdit = Object.assign(Object.assign({}, CreateWorkspaceEdit(stepCounter, message, WorkspaceEditType.alteredTable)), { sql: StepTypes.getStepTypeTemplatesSafe(stepTypesParsed, updatedOperation.sqlType).create.templateString });
            updateEditsCollection([newEdit], advancedDeployHydratedMetadata);
            /**
             * originalSteps now include a database and schema field, so when you compare a deployed node vs
             * a non deployed node(updatedStep) there will always be a diff and some dumb edits occur, everytime
             */
        }
        else if (originalStep.operation.sqlType !== Runner.ESQLType.source) {
            const hasDiff = DetermineIfTwoNodesHaveADifference(originalStep, updatedStep);
            //need locationMappingDifference as a backup to create workspaceEdits when there is not a diff between the nodes, but the location data has changed behind the scenes
            /**
             * When a deploy fails, the mappings for that deployment don't get saved to the workspace,
             * therefore, in the event that you fail your first deployment, and you attempt to deploy a different
             * commit, you will have some metadata to compare against, but no mappings to build these
             * qualifiedLocation variables.  Eventually leading to sql with qualified locations like: 'undefined.
             * undefined.STG_CUSTOMER' which is invalid.  So now we try/catch the locations so we don't get
             * undefined.
             *
             * CD-1684 UPDATE
             * We now grab the database.schema from the metadata for a node from the node instead of from the
             * deployed mappings, so we can guarantee that the information always exists, this property will exist
             * only on deployed nodes, (moving forward) so need to have a workaround for if the properties don't
             * exist, ex for our existing customers/developers, we shouldn't be locked out of deploying our
             * environments by this, so finding a backup solution if the database/schema don't exist is important
             */
            let oldQualifiedLocation;
            let newQualifiedLocation;
            // new flow has us look at these values, but for backwards compatibility we check if the values exist before moving forward
            try {
                oldQualifiedLocation = (0, exports.CreateOldLocationName)(originalOperation, oldWorkspaceMappings);
            }
            catch (_a) {
                Logger.error("Missing firestore mapping data");
                oldQualifiedLocation = undefined;
            }
            try {
                newQualifiedLocation = (0, exports.CreateOldLocationName)(updatedOperation, newWorkspaceMappings);
            }
            catch (error) {
                //can happen when you forget to commit your data.yml, so your mappings aren't commited
                Logger.error("missing git mappings", error);
                newQualifiedLocation = undefined;
            }
            //if we don't have an oldQualifiedLocation, don't try to do alter statements or drop statements
            let locationMappingDifference = false;
            locationMappingDifference = oldQualifiedLocation !== undefined ? oldQualifiedLocation !== newQualifiedLocation : false;
            if (hasDiff || locationMappingDifference) {
                const workspaceDataWithID = Object.assign(Object.assign({}, newWorkspaceData.workspaces), { currentWorkspace: workspaceID });
                //setup the current collection[stepCounter]
                const workspaceDataWithMappingsFromNode = CreateAndEmbedMappingsFromNode(originalOperation.locationID, originalOperation.database, originalOperation.schema, originalOperation.locationName, workspaceID, workspaceDataWithID);
                const hydratedMetadata = MetadataHydration.hydrateMetadata(updatedStep.operation, stepCounter, newWorkspaceData.steps, newWorkspaceData.stepTypes, workspaceDataWithMappingsFromNode, null, runTimeParameters);
                updateEditsCollection([], hydratedMetadata);
                const locationData = {
                    oldName: originalStep.operation.name,
                    oldLocation: oldQualifiedLocation
                };
                //handle view differently since they are simpler
                /** CD-764 && 858 */
                let dropEdit = CreateWorkspaceEdit(stepCounter, "", WorkspaceEditType.deletedTable);
                dropEdit.dataForNodeMetadata = Object.assign({}, locationData);
                let recreateEdit = CreateWorkspaceEdit(stepCounter, `Recreated View: ${updatedOperation.name}`, WorkspaceEditType.alteredTable);
                if (originalOperation.materializationType === Templates.materializationTypes.view) {
                    if (oldQualifiedLocation) {
                        dropEdit.description = `Dropped View: ${originalOperation.name}`;
                        dropEdit.sql = Templates.getDeleteViewTemplate(platformType);
                        editsCollection[stepCounter].workspaceEdits.push(dropEdit);
                    }
                    recreateEdit.description = `Recreated View: ${updatedOperation.name}`;
                    recreateEdit.sql = Templates.getCreateTableTemplate(stepTypesParsed, originalOperation.sqlType);
                    editsCollection[stepCounter].workspaceEdits.push(recreateEdit);
                    return;
                }
                /**
                 * First edit is creating a clone (clonedtable_COALESCE_INTERNAL_TABLE)
                 *  The cloned table is what is actually edited. Once edits are complete, the cloned table will be
                 * swapped or renamed
                 */
                const clonedTableEdit = (0, exports.CreateWorkspaceEditForCloneTable)(stepCounter, locationData, platformType);
                editsCollection[stepCounter].workspaceEdits.push(...clonedTableEdit);
                //setup ColumnCounter sets for doing delete, add, and alters + renames in the same manner as with the StepCounters from getWorkspaceEdits
                const originalColumnCounters = originalOperation.metadata.columns.map(column => column.columnReference.columnCounter);
                const updatedColumnCounters = updatedOperation.metadata.columns.map(column => column.columnReference.columnCounter);
                const columnCounterSets = Common.CompareAndCreateSetsFromArrays(originalColumnCounters, updatedColumnCounters);
                const deletedColumns = columnCounterSets.firstOnly;
                const addedColumns = columnCounterSets.secondOnly;
                const columnsWithPossibleEdits = columnCounterSets.inCommon;
                /**
                 * this order of operations is important based on our deployment strategy, since we can batch:
                 * deletes, adds, and alters but not renames.  The order in which we save the edits will allow us
                 * to appropriately update firestore after we have completed the renames, so once for columns, and
                 * once for the table level: as needed.
                 * column drops should come first, in the event that a column name is reused when adding a new column
                 * pass stepCounter into all of these instead of deleted columns
                 *
                 */
                const deletedColumnEdits = CreateWorkspaceEditForColumnDeletes(originalStep, locationData, deletedColumns, stepCounter, platformType);
                editsCollection[stepCounter].workspaceEdits.push(...deletedColumnEdits);
                //column alters
                const alteredColumnEdits = CreateWorkspaceEditForColumnAlters(originalStep, updatedStep, locationData, columnsWithPossibleEdits, stepCounter, platformType);
                editsCollection[stepCounter].workspaceEdits.push(...alteredColumnEdits);
                //column renames
                const renamedColumnEdits = CreateWorkspaceEditsForColumnRenames(originalStep, updatedStep, locationData, columnsWithPossibleEdits, stepCounter, platformType);
                editsCollection[stepCounter].workspaceEdits.push(...renamedColumnEdits);
                //column adds, in the event that I add a new column that reuses a name that I either drop, or just renamed away from, this won't have a conflict
                const addedColumnEdits = CreateWorkspaceEditForColumnAdditions(updatedStep, locationData, addedColumns, stepCounter, platformType);
                editsCollection[stepCounter].workspaceEdits.push(...addedColumnEdits);
                const nameDifference = originalOperation.name !== updatedOperation.name;
                const locationIDDifference = originalOperation.locationID !== updatedOperation.locationID;
                if (nameDifference || locationIDDifference || locationMappingDifference) {
                    //table rename
                    let newEdits = CreateWorkspaceEditForTableChange(stepCounter, locationData, platformType);
                    let dropTable = (0, exports.CreateWorkspaceEditForDeleteTable)(stepCounter, locationData, platformType);
                    editsCollection[stepCounter].workspaceEdits.push(...newEdits, ...dropTable);
                }
                else {
                    //table edits
                    let newEdits = (0, exports.CreateWorkspaceEditForSwapCloneTable)(stepCounter, locationData, platformType);
                    let dropTable = (0, exports.CreateWorkspaceEditForDeleteTable)(stepCounter, Object.assign(Object.assign({}, locationData), { oldName: originalOperation.name + Templates.deploySuffix }), platformType);
                    editsCollection[stepCounter].workspaceEdits.push(...newEdits, ...dropTable);
                }
                Common.assert(Logging.LoggingArea.Deploy, !!editsCollection[stepCounter], `'editsCollection' does not contain an edit for stepCounter: ${stepCounter}`);
            }
        }
    });
    return editsCollection;
};
const ConvertEnvDetailsToEnvironmentInterface = (envDetails) => {
    const { eName, eDescription, eMapping, connectionAccount, oauthClientID, oauthClientSecret, oAuthEnabled, workspaceID, eCreatedBy, workspaceCreatedAt, runTimeParameters } = envDetails;
    return {
        name: eName,
        description: eDescription,
        mappings: eMapping,
        status: Runner.EWorkspaceHealth.waiting,
        devEnv: false,
        createdBy: eCreatedBy,
        createdAt: workspaceCreatedAt,
        // addWorkspace will update ID
        id: workspaceID,
        connectionAccount: connectionAccount || "",
        oauthEnabled: oAuthEnabled || false,
        oauthClientID: oauthClientID || "",
        oauthClientSecret: oauthClientSecret || "",
        runTimeParameters: runTimeParameters || ""
    };
};
exports.ConvertEnvDetailsToEnvironmentInterface = ConvertEnvDetailsToEnvironmentInterface;
/**
 *
 * 1. Attempt column datatypes / nullability  for a table, bundle all column alters in to one statement/workspaceEdit
 * 1b. DROP COLUMN allows for multiple column drops, batch for efficiency?
 * 2. Attempt column renames; individual workspaceEdits for each rename/add ?order?
 * 3. Update firebase column metadata / mass step.operation.metadata.columns overwrite
 * 4. Attempt rename; move table as needed
 * 5. Update firebase w/ rename finish with ste.operation.name update as needed or locationName etc.
 *
 * when I generate the mass exodus protocol, tack those on as the last workspaceEdit for all nodes
 */
//still need to check for a difference in the deployed workspace in terms of the physical locations, if they have changed and that is where the edits will need to be done, everything needs to be recreated...
const getWorkspaceEdits = (fromAllWorkspaceData, toAllWorkspaceData, workspaceID, platformType, runTimeParameters) => {
    const { subgraphEdits, jobEdits, macroEdits, stepTypeEdits } = CalculateEntityUpdatesForFirestoreFlush(fromAllWorkspaceData, toAllWorkspaceData);
    const stepDataEdits = CalculateDividedStepDataWithWorkspaceEdits(fromAllWorkspaceData, toAllWorkspaceData, workspaceID, platformType, runTimeParameters);
    return { nodeEdits: stepDataEdits, AllWorkspaceData: fromAllWorkspaceData, subgraphEdits, jobEdits, macroEdits, stepTypeEdits };
};
exports.getWorkspaceEdits = getWorkspaceEdits;
const ConvertStepsCollectionForFirestore = (stepsCollection) => {
    const convertedCollection = {};
    Object.entries(stepsCollection).forEach(([nodeID, nodeData]) => {
        const updatedNodeData = nodeData.operation;
        convertedCollection[nodeID] = updatedNodeData;
    });
    return convertedCollection;
};
const CalculateEntityUpdatesForFirestoreFlush = (fromAllWorkspaceData, toAllWorkspaceData) => {
    //reshape steps collection before feeding it into the updater function, need to have nodes that have already accessed the .operation
    const fromNodesUpdated = ConvertStepsCollectionForFirestore(fromAllWorkspaceData.steps);
    const toNodesUpdated = ConvertStepsCollectionForFirestore(toAllWorkspaceData.steps);
    const nodeEdits = CalculateDividedEntityUpdates(fromNodesUpdated, toNodesUpdated);
    const subgraphEdits = CalculateDividedEntityUpdates(fromAllWorkspaceData.folders, toAllWorkspaceData.folders);
    const jobEdits = CalculateDividedEntityUpdates(fromAllWorkspaceData.jobs, toAllWorkspaceData.jobs);
    const macroEdits = CalculateDividedEntityUpdates(fromAllWorkspaceData.macros, toAllWorkspaceData.macros);
    const stepTypeEdits = CalculateDividedEntityUpdates(fromAllWorkspaceData.stepTypes, toAllWorkspaceData.stepTypes);
    return { nodeEdits, subgraphEdits, jobEdits, macroEdits, stepTypeEdits };
};
const CompareAndSplitEntityIDsBetweenEntityCollections = (fromEntityCollection, toEntityCollection) => {
    const fromEntityIDs = Object.keys(fromEntityCollection);
    const toEntityIDs = Object.keys(toEntityCollection);
    const dividedEntityIDsByCommonality = Common.CompareAndCreateSetsFromArrays(toEntityIDs, fromEntityIDs);
    const deletedEntityIDs = dividedEntityIDsByCommonality.firstOnly;
    const addedEntityIDs = dividedEntityIDsByCommonality.secondOnly;
    const entitiesWithPossibleEdits = dividedEntityIDsByCommonality.inCommon;
    return { deletedEntityIDs, addedEntityIDs, entitiesWithPossibleEdits };
};
const CalculateDividedEntityUpdates = (newEntityCollection, oldEntityCollection) => {
    const entityUpdates = { changed: {}, deleted: {} };
    const { deletedEntityIDs, addedEntityIDs, entitiesWithPossibleEdits } = CompareAndSplitEntityIDsBetweenEntityCollections(newEntityCollection, oldEntityCollection);
    //add all added to changed
    addedEntityIDs.forEach(newEntityID => {
        const newEntityData = newEntityCollection[newEntityID];
        const entityData = newEntityData;
        entityUpdates.changed[newEntityID] = entityData;
    });
    //add all deleted to deleted
    deletedEntityIDs.forEach(entityID => {
        const oldEntityData = oldEntityCollection[entityID];
        const entityData = oldEntityData;
        entityUpdates.deleted[entityID] = entityData;
    });
    //do diff on possibleEdits and add to added as needed
    entitiesWithPossibleEdits.forEach(entityID => {
        const newEntityData = newEntityCollection[entityID];
        const oldEntityData = oldEntityCollection[entityID];
        //do diff first
        if (Diff.diff(newEntityData, oldEntityData)) {
            entityUpdates.changed[entityID] = newEntityData;
        }
    });
    return entityUpdates;
};
/**
 * Internal function responsible to determining which Nodes require metadata/SQL edits during a deploy action
 * @param fromAllWorkspaceData
 * @param toAllWorkspaceData
 * @param workspaceID
 * @param platformType
 * @returns
 */
const CalculateDividedStepDataWithWorkspaceEdits = (fromAllWorkspaceData, toAllWorkspaceData, workspaceID, platformType, runTimeParameters) => {
    var _a, _b;
    const stepDataEdits = { [WorkspaceEditType.addedTable]: {}, [WorkspaceEditType.alteredTable]: {}, [WorkspaceEditType.deletedTable]: {} };
    const { deletedEntityIDs, addedEntityIDs, entitiesWithPossibleEdits } = CompareAndSplitEntityIDsBetweenEntityCollections(fromAllWorkspaceData.steps, toAllWorkspaceData.steps);
    const newWorkspaceMappings = ((_b = (_a = fromAllWorkspaceData.workspaces) === null || _a === void 0 ? void 0 : _a.workspaces[workspaceID]) === null || _b === void 0 ? void 0 : _b.mappings) || {};
    stepDataEdits.deletedTable = CreateWorkspaceEditsForSteps(deletedEntityIDs, WorkspaceEditType.deletedTable, toAllWorkspaceData, newWorkspaceMappings, platformType, workspaceID, fromAllWorkspaceData.stepTypes, runTimeParameters);
    stepDataEdits.addedTable = CreateWorkspaceEditsForSteps(addedEntityIDs, WorkspaceEditType.addedTable, fromAllWorkspaceData, newWorkspaceMappings, platformType, workspaceID, fromAllWorkspaceData.stepTypes, runTimeParameters);
    stepDataEdits.alteredTable = CreateWorkspaceEditsForEditedSteps(entitiesWithPossibleEdits, toAllWorkspaceData, fromAllWorkspaceData, workspaceID, platformType, fromAllWorkspaceData.stepTypes, runTimeParameters);
    return stepDataEdits;
};
const isValidWorkspace = (workspaceInfo) => {
    return workspaceInfo.hasOwnProperty("createdBy"); //createdBy is a required field - previous versions only had an id field
};
exports.isValidWorkspace = isValidWorkspace;
const CreateFirestoreUpdatesForEntityCollections = (changedAndDeletedIDForEntityCollection, collectionRef) => {
    const entityUpdates = {
        set: Object.entries(changedAndDeletedIDForEntityCollection.changed).map(([entityID, entityData]) => {
            return { id: entityID, value: entityData };
        }),
        deletes: Object.entries(changedAndDeletedIDForEntityCollection.deleted).map(([entityID, entityData]) => {
            return { id: entityID, value: entityData };
        }),
        collectionRef: collectionRef
    };
    return entityUpdates;
};
const CreateAllEntityUpdatesForBatching = (desiredWorkspaceDataState, workspaceDataToReplace) => {
    const { nodeEdits, subgraphEdits, jobEdits, macroEdits, stepTypeEdits } = CalculateEntityUpdatesForFirestoreFlush(desiredWorkspaceDataState, workspaceDataToReplace);
    const nodeFirestoreEdits = CreateFirestoreUpdatesForEntityCollections(nodeEdits, CommonOperations.DocRefs.steps);
    const subgraphFirestoreEdits = CreateFirestoreUpdatesForEntityCollections(subgraphEdits, CommonOperations.DocRefs.folders);
    const jobFirestoreEdits = CreateFirestoreUpdatesForEntityCollections(jobEdits, CommonOperations.DocRefs.jobs);
    const macroFirestoreEdits = CreateFirestoreUpdatesForEntityCollections(macroEdits, CommonOperations.DocRefs.macros);
    const stepTypeFirestoreEdits = CreateFirestoreUpdatesForEntityCollections(stepTypeEdits, CommonOperations.DocRefs.templates);
    return [nodeFirestoreEdits, subgraphFirestoreEdits, jobFirestoreEdits, macroFirestoreEdits, stepTypeFirestoreEdits];
};
const FlushFirestoreWorkspaceWithAllWorkspaceData = (firestore, teamID, workspaceIDForTargetFlush, desiredWorkspaceDataState, workspaceDataToReplace) => {
    return new Promise((resolve, reject) => {
        //no workspaces means no mappings (normal for build envs) so always assume this?
        GitLogger.debugContext({ orgID: teamID, envID: workspaceIDForTargetFlush }, "received Firestore data to compare");
        const updatesWithoutBatch = CreateAllEntityUpdatesForBatching(desiredWorkspaceDataState, workspaceDataToReplace);
        const allUpdatesWithAddToBatchFunction = updatesWithoutBatch
            .map((entityUpdates) => {
            // use collection ref to target correct data to update
            const { set, deletes, collectionRef } = entityUpdates;
            const collectionReference = CommonOperations.getDocumentRefAdmin(firestore, teamID, workspaceIDForTargetFlush, collectionRef);
            const setsToBatch = set.map(entity => {
                GitLogger.debug("Entity for update", entity);
                const docRef = collectionReference.doc(entity.id);
                return Common.BatchSetFirestoreDoc(docRef, entity.value);
            });
            const deletesToBatch = deletes.map(entity => {
                GitLogger.debug("Entity for update", entity);
                const docRef = collectionReference.doc(entity.id);
                return Common.BatchDeleteFirestoreDoc(docRef);
            });
            return [...setsToBatch, ...deletesToBatch];
        }).flat();
        const storageLocationUpdates = CreateUpdatesForStorageLocations(desiredWorkspaceDataState, teamID, workspaceIDForTargetFlush);
        allUpdatesWithAddToBatchFunction.push(...storageLocationUpdates);
        GitLogger.debugContext({ orgID: teamID, envID: workspaceIDForTargetFlush }, "# of Firestore updates to flush", allUpdatesWithAddToBatchFunction.length);
        return Common.FirestoreBatchBarrel(allUpdatesWithAddToBatchFunction, 500)
            .then(() => resolve())
            .catch(error => {
            GitLogger.errorContext({ orgID: teamID, envID: workspaceIDForTargetFlush }, "Error during FlushFirestoreWorkspaceWithAllWorkspaceData: ", error);
            reject(error);
        });
    });
};
exports.FlushFirestoreWorkspaceWithAllWorkspaceData = FlushFirestoreWorkspaceWithAllWorkspaceData;
/**
 * Helper function for abstracting the creation of storage locations during a firestore flush
 * @param nodeCollection
 * @param teamID
 * @param envID
 * @returns
 */
const CreateUpdatesForStorageLocations = (allWorkspaceData, teamID, envID) => {
    const locations = allWorkspaceData.locations || allWorkspaceData.mappings;
    if (!locations) {
        return [];
    }
    const updates = Object.values(locations)
        .map((location) => {
        const documentRef = StorageMappings.getMappingCollectionRef(teamID).doc(location.id.toString());
        Logger.debugContext({ orgID: teamID, envID }, "creating location batch function:", location);
        return Common.BatchSetMergeFirestoreDoc(documentRef, location);
    });
    return updates;
};
//# sourceMappingURL=Workspaces.js.map